@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-
-# Copyright (C) Duncan Macleod (2013-2015)
+# -*- coding: utf-8 -*-
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with GWpy.  If not, see <http://www.gnu.org/licenses/>
+# along with GWpy.  If not, see <http://www.gnu.org/licenses/>.
 
 """Calculating and plotting a `SpectralVariance` histogram
 
@@ -55,10 +55,10 @@ ax.grid()
 ax.set_xlim(20, 1500)
 ax.set_ylim(1e-24, 1e-20)
 ax.set_xlabel('Frequency [Hz]')
-ax.set_ylabel(r'[strain/\rtHz]')
+ax.set_ylabel(r'[strain/$\sqrt{\mathrm{Hz}}$]')
 ax.set_title('LIGO-Livingston sensitivity variance')
 plot.show()
 
-# From this we see that in general the sensitivity varies a few parts in 
+# From this we see that in general the sensitivity varies a few parts in
 # 10 :sup:`-23`, while many of the lines (narrow-band peaks in the spectrum)
 # are much more stationary.
