@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-
-# Copyright (C) Duncan Macleod (2013)
+# -*- coding: utf-8 -*-
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -63,11 +63,11 @@ plot = specgram.plot(norm='log', cmap='viridis', yscale='log')
 ax = plot.gca()
 ax.set_title('LIGO-Hanford strain data around GW150914')
 ax.set_xlim(1126259462, 1126259463)
-ax.colorbar(label=r'Strain ASD [1/\rtHz]')
+ax.colorbar(label=r'Strain ASD [1/$\sqrt{\mathrm{Hz}}$]')
 plot.show()
 
 # Here we can see the trace of a high-mass binary black hole system,
 # referred to as GW150914.
-# For more details on this signal, please see 
+# For more details on this signal, please see
 # `Abbott et al. (2016) <https://doi.org/10.1103/PhysRevLett.116.061102>`_
 # (the joint LSC-Virgo publication announcing this detection).
