@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-
-# Copyright (C) Duncan Macleod (2013-2015)
+# -*- coding: utf-8 -*-
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -56,7 +56,7 @@ for ifo, ax in zip(('Hanford', 'Livingston'), (ax1, ax2)):
     ax.legend(['X', 'Y', 'Z'])
     ax.text(1.01, 0.5, ifo, ha='left', va='center', transform=ax.transAxes,
             fontsize=18)
-ax1.set_ylabel('$1-3$\,Hz motion [nm/s]', y=-0.1)
+ax1.set_ylabel(r'$1-3$\,Hz motion [nm/s]', y=-0.1)
 ax2.set_ylabel('')
 ax1.set_title('Magnitude 7.1 earthquake impact on LIGO')
 plot.show()
