@@ -1,7 +1,6 @@
 #!/usr/bin/env python
-
-
-# Copyright (C) Duncan Macleod (2013)
+# -*- coding: utf-8 -*-
+# Copyright (C) Alex Urban (2018-2021)
 #
 # This file is part of GWpy.
 #
@@ -39,11 +38,11 @@ from numpy import random
 from gwpy.timeseries import TimeSeries
 noise = TimeSeries(random.normal(scale=.1, size=16384), sample_rate=16384)
 
-# Then we can download a simulation of the GW150914 signal from LOSC:
+# Then we can download a simulation of the GW150914 signal from GWOSC:
 
 from astropy.utils.data import get_readable_fileobj
-source = 'https://losc.ligo.org/s/events/GW150914/P150914/'
-url = '%s/fig2-unfiltered-waveform-H.txt' % source
+url = ("https://www.gw-openscience.org/s/events/GW150914/P150914/"
+       "fig2-unfiltered-waveform-H.txt")
 with get_readable_fileobj(url) as f:
     signal = TimeSeries.read(f, format='txt')
 signal.t0 = .5  # make sure this intersects with noise time samples
