@@ -1,7 +1,5 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (C) Joseph Areeda (2015)
+# Copyright (C) Joseph Areeda (2015-2021)
 #
 # This file is part of GWpy.
 #
@@ -17,38 +15,31 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with GWpy.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 """Base class for CLI (`gwpy-plot`) products.
 """
 
-from __future__ import print_function
 import abc
 import os.path
 import re
 import time
-import warnings
 import sys
-from collections import OrderedDict
 from functools import wraps
 
-from six import add_metaclass
-
-from matplotlib import rcParams
-try:
-    from matplotlib.cm import viridis as DEFAULT_CMAP
-except ImportError:
-    from matplotlib.cm import YlOrRd as DEFAULT_CMAP
+from matplotlib import (rcParams, style)
 
 from astropy.time import Time
 from astropy.units import Quantity
 
+from ..utils import unique
 from ..signal import filter_design
 from ..signal.window import recommended_overlap
 from ..time import to_gps
 from ..timeseries import TimeSeriesDict
 from ..plot.gps import (GPS_SCALES, GPSTransform)
 from ..plot.tex import label_to_latex
+from ..segments import DataQualityFlag
+
 
 __author__ = 'Joseph Areeda <joseph.areeda@ligo.org>'
 
@@ -94,21 +85,9 @@ to_hz = to_float('Hz')  # pylint: disable=invalid-name
 to_s = to_float('s')  # pylint: disable=invalid-name
 
 
-def unique(list_):
-    """Returns a unique version of the input list preserving order
-
-    Examples
-    --------
-    >>> unique(['b', 'c', 'a', 'a', 'd', 'e', 'd', 'a'])
-    ['b', 'c', 'a', 'd', 'e']
-    """
-    return list(OrderedDict.fromkeys(list_).keys())
-
-
 # -- base product class -------------------------------------------------------
 
-@add_metaclass(abc.ABCMeta)
-class CliProduct(object):
+class CliProduct(object, metaclass=abc.ABCMeta):
     """Base class for all cli plot products
 
     Parameters
@@ -151,7 +130,6 @@ class CliProduct(object):
     action = None
 
     def __init__(self, args):
-        self._finalize_arguments(args)  # post-process args
 
         #: input argument Namespace
         self.args = args
@@ -159,15 +137,11 @@ class CliProduct(object):
         #: verbosity
         self.verbose = 0 if args.silent else args.verbose
 
+        # NB: finalizing may want to log if we're being verbose
+        self._finalize_arguments(args)  # post-process args
+
         if args.style:  # apply custom styling
-            try:
-                from matplotlib import style
-            except ImportError:
-                from matplotlib import __version__ as mpl_version
-                warnings.warn('--style can only be used with matplotlib >= '
-                              '1.4.0, you have {0}'.format(mpl_version))
-            else:
-                style.use(args.style)
+            style.use(args.style)
 
         #: the current figure object
         self.plot = None
@@ -185,6 +159,10 @@ class CliProduct(object):
         #: channels to load
         self.chan_list = unique(c for clist in args.chan for c in clist)
 
+        # use reduced as an alias for rds in channel name
+        for idx in range(len(self.chan_list)):
+            self.chan_list[idx] = self.chan_list[idx].replace('reduced', 'rds')
+
         # total number of datasets that _should_ be acquired
         self.n_datasets = len(self.chan_list) * len(self.start_list)
 
@@ -335,15 +313,16 @@ class CliProduct(object):
         group.add_argument('--interactive', action='store_true',
                            help='when running from ipython '
                                 'allows experimentation')
-        group.add_argument('--title', action='append',
-                           help='One or more title lines')
+        group.add_argument('--title', action='store',
+                           help='Set title (below suptitle, defaults to '
+                                'parameter summary')
         group.add_argument('--suptitle',
                            help='1st title line (larger than the others)')
         group.add_argument('--out', default='gwpy.png',
                            help='output filename')
 
         # legends match input files in position are displayed if specified.
-        group.add_argument('--legend', nargs='*', action='append',
+        group.add_argument('--legend', nargs='+', action='append', default=[],
                            help='strings to match data files')
         group.add_argument('--nolegend', action='store_true',
                            help='do not display legend')
@@ -578,10 +557,12 @@ class CliProduct(object):
         max_ = getattr(self.args, '{}max'.format(axis))
 
         # parse limits
-        if scale == 'auto-gps' and (
-                min_ is not None and
-                max_ is not None and
-                max_ < 1e8):
+        if (
+            scale == 'auto-gps'
+            and min_ is not None
+            and max_ is not None
+            and max_ < 1e8
+        ):
             limits = (min_, min_ + max_)
         else:
             limits = (min_, max_)
@@ -629,7 +610,7 @@ class CliProduct(object):
         """Create a legend for this product (if applicable)
         """
         leg = self.ax.legend(prop={'size': 10})
-        if self.n_datasets == 1 and leg:
+        if leg and self.n_datasets == 1:
             try:
                 leg.remove()
             except NotImplementedError:
@@ -637,18 +618,22 @@ class CliProduct(object):
         return leg
 
     def set_title(self, title):
-        """Set the title for this plot.
+        """Set the title(s) for this plot.
 
         The `Axes.title` actually serves at the sub-title for the plot,
         typically giving processing parameters and information.
         """
+
         if title is None:
-            title = self.get_title().rstrip(', ')
+            title_line = self.get_title().rstrip(', ')
+        else:
+            title_line = title
+
         if self.usetex:
-            title = label_to_latex(title)
-        if title:
-            self.ax.set_title(title, fontsize=12)
-            self.log(3, ('Title is: %s' % title))
+            title_line = label_to_latex(title_line)
+        if title_line:
+            self.ax.set_title(title_line, fontsize=12)
+            self.log(3, ('Title is: %s' % title_line))
 
     def set_suptitle(self, suptitle):
         """Set the super title for this plot.
@@ -663,8 +648,13 @@ class CliProduct(object):
     def set_grid(self, enable):
         """Set the grid parameters for this plot.
         """
-        self.ax.grid(b=enable, which='major', color='k', linestyle='solid')
-        self.ax.grid(b=enable, which='minor', color='0.06', linestyle='dotted')
+        if enable:
+            self.ax.grid(b=True, which='major',
+                         color='k', linestyle='solid')
+            self.ax.grid(b=True, which='minor',
+                         color='0.06', linestyle='dotted')
+        else:
+            self.ax.grid(b=False)
 
     def save(self, outfile):
         """Save this product to the target `outfile`.
@@ -706,6 +696,7 @@ class CliProduct(object):
             self._make_plot()
             if self.plot:
                 self.set_plot_properties()
+                self.add_segs(self.args)
                 if self.args.interactive:
                     self.log(3, 'Interactive manipulation of '
                                 'image should be available.')
@@ -720,18 +711,60 @@ class CliProduct(object):
                 show_error = False
             self.plot_num += 1
 
+    def add_segs(self, args):
+        """ If requested add DQ segments
+        """
+        std_segments = [
+            '{ifo}:DMT-GRD_ISC_LOCK_NOMINAL:1',
+            '{ifo}:DMT-DC_READOUT_LOCKED:1',
+            '{ifo}:DMT-CALIBRATED:1',
+            '{ifo}:DMT-ANALYSIS_READY:1'
+        ]
+        segments = list()
+        if hasattr(args, 'std_seg'):
+            if args.std_seg:
+                segments = std_segments
+            if args.seg:
+                for seg in args.seg:
+                    # NB: args.seg may be list of lists
+                    segments += seg
+
+            chan = args.chan[0][0]
+            m = re.match('^([A-Za-z][0-9]):', chan)
+            ifo = m.group(1) if m else '?:'
+
+            start = None
+            end = 0
+            for ts in self.timeseries:
+                if start is not None:
+                    start = min(ts.t0, start)
+                    end = max(ts.t0 + ts.duration, end)
+                else:
+                    start = ts.t0
+                    end = start + ts.duration
+
+            for segment in segments:
+                seg_name = segment.replace('{ifo}', ifo)
+                seg_data = DataQualityFlag.query_dqsegdb(
+                    seg_name,
+                    start,
+                    end,
+                )
+
+                self.plot.add_segments_bar(seg_data, label=seg_name)
+
 
 # -- extensions ---------------------------------------------------------------
 
-@add_metaclass(abc.ABCMeta)
-class ImageProduct(CliProduct):
+class ImageProduct(CliProduct, metaclass=abc.ABCMeta):
     """Base class for all x/y/color plots
     """
+    DEFAULT_CMAP = "viridis"
     MAX_DATASETS = 1
 
     @classmethod
     def init_plot_options(cls, parser):
-        super(ImageProduct, cls).init_plot_options(parser)
+        super().init_plot_options(parser)
         cls.arg_color_axis(parser)
 
     @classmethod
@@ -744,6 +777,7 @@ class ImageProduct(CliProduct):
         group.add_argument('--imax', type=float,
                            help='maximum value for colorbar')
         group.add_argument('--cmap',
+                           default=cls.DEFAULT_CMAP,
                            help='Colormap. See '
                                 'https://matplotlib.org/examples/color/'
                                 'colormaps_reference.html for options')
@@ -756,11 +790,6 @@ class ImageProduct(CliProduct):
         group.add_argument('--nocolorbar', action='store_true',
                            help='hide the colour bar')
 
-    def _finalize_arguments(self, args):
-        if args.cmap is None:
-            args.cmap = DEFAULT_CMAP.name
-        return super(ImageProduct, self)._finalize_arguments(args)
-
     @staticmethod
     def get_color_label():
         """Returns the default colorbar label
@@ -771,7 +800,7 @@ class ImageProduct(CliProduct):
         """Set properties for each axis (scale, limits, label) and create
         a colorbar.
         """
-        super(ImageProduct, self).set_axes_properties()
+        super().set_axes_properties()
         if not self.args.nocolorbar:
             self.set_colorbar()
 
@@ -786,8 +815,7 @@ class ImageProduct(CliProduct):
         return  # image plots don't have legends
 
 
-@add_metaclass(abc.ABCMeta)
-class FFTMixin(object):
+class FFTMixin(object, metaclass=abc.ABCMeta):
     """Mixin for `CliProduct` class that will perform FFTs
 
     This just adds FFT-based command line options
@@ -796,7 +824,7 @@ class FFTMixin(object):
     def init_data_options(cls, parser):
         """Set up data input and signal processing options including FFTs
         """
-        super(FFTMixin, cls).init_data_options(parser)
+        super().init_data_options(parser)
         cls.arg_fft(parser)
 
     @classmethod
@@ -861,11 +889,10 @@ class FFTMixin(object):
                 args.overlap = recommended_overlap(args.window)
             except ValueError:
                 args.overlap = .5
-        return super(FFTMixin, self)._finalize_arguments(args)
+        return super()._finalize_arguments(args)
 
 
-@add_metaclass(abc.ABCMeta)
-class TimeDomainProduct(CliProduct):
+class TimeDomainProduct(CliProduct, metaclass=abc.ABCMeta):
     """`CliProduct` with time on the X-axis
     """
     @classmethod
@@ -875,10 +902,15 @@ class TimeDomainProduct(CliProduct):
         This method includes the standard X-axis options, as well as a new
         ``--epoch`` option for the time axis.
         """
-        group = super(TimeDomainProduct, cls).arg_xaxis(parser)
+        group = super().arg_xaxis(parser)
         group.add_argument('--epoch', type=to_gps,
                            help='center X axis on this GPS time, may be'
                                 'absolute date/time or delta')
+
+        group.add_argument('--std-seg', action='store_true',
+                           help='add DQ segment describing IFO state')
+        group.add_argument('--seg', type=str, nargs='+', action='append',
+                           help='specify one or more DQ segment names')
         return group
 
     def _finalize_arguments(self, args):
@@ -889,12 +921,12 @@ class TimeDomainProduct(CliProduct):
             args.xmin = min(starts)
         if args.epoch is None:
             args.epoch = args.xmin
-        elif (args.epoch < 1e8):
-                args.epoch += min(starts)
+        elif args.epoch < 1e8:
+            args.epoch += min(starts)
 
         if args.xmax is None:
             args.xmax = max(starts) + args.duration
-        return super(TimeDomainProduct, self)._finalize_arguments(args)
+        return super()._finalize_arguments(args)
 
     def get_xlabel(self):
         """Default X-axis label for plot
@@ -910,8 +942,7 @@ class TimeDomainProduct(CliProduct):
         return ''
 
 
-@add_metaclass(abc.ABCMeta)
-class FrequencyDomainProduct(CliProduct):
+class FrequencyDomainProduct(CliProduct, metaclass=abc.ABCMeta):
     """`CliProduct` with frequency on the X-axis
     """
     def get_xlabel(self):
