@@ -1,7 +1,5 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (C) Joseph Areeda (2015)
+# Copyright (C) Joseph Areeda (2015-2021)
 #
 # This file is part of GWpy.
 #
@@ -17,7 +15,6 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with GWpy.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 """Coherence plots
 """
@@ -40,7 +37,7 @@ class Coherence(Spectrum):
     MIN_DATASETS = 2
 
     def __init__(self, *args, **kwargs):
-        super(Coherence, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.ref_chan = self.args.ref or self.chan_list[0]
         # deal with channel type appendages
         if ',' in self.ref_chan:
@@ -48,7 +45,7 @@ class Coherence(Spectrum):
 
     @classmethod
     def arg_channels(cls, parser):
-        group = super(Coherence, cls).arg_channels(parser)
+        group = super().arg_channels(parser)
         group.add_argument('--ref', help='Reference channel against which '
                                          'others will be compared')
         return group
@@ -61,7 +58,7 @@ class Coherence(Spectrum):
                 args.ymin = 0
             if not args.ymax:
                 args.ymax = 1.05
-        return super(Coherence, self)._finalize_arguments(args)
+        return super()._finalize_arguments(args)
 
     def get_ylabel(self):
         """Text for y-axis label
@@ -128,7 +125,7 @@ class Coherence(Spectrum):
     def set_legend(self):
         """Create a legend for this product
         """
-        leg = super(Coherence, self).set_legend()
+        leg = super().set_legend()
         if leg is not None:
             leg.set_title('Coherence with:')
         return leg
