@@ -1,7 +1,5 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (C) Joseph Areeda (2015)
+# Copyright (C) Joseph Areeda (2015-2021)
 #
 # This file is part of GWpy.
 #
@@ -17,16 +15,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with GWpy.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 """Coherence spectrogram
 """
 
-try:
-    from matplotlib.cm import plasma as DEFAULT_CMAP
-except ImportError:
-    DEFAULT_CMAP = None
-
 from .spectrogram import Spectrogram
 
 __author__ = 'Joseph Areeda <joseph.areeda@ligo.org>'
@@ -35,12 +27,13 @@ __author__ = 'Joseph Areeda <joseph.areeda@ligo.org>'
 class Coherencegram(Spectrogram):
     """Plot the coherence-spectrogram comparing two time series
     """
+    DEFAULT_CMAP = "plasma"
     MIN_DATASETS = 2
     MAX_DATASETS = 2
     action = 'coherencegram'
 
     def __init__(self, *args, **kwargs):
-        super(Coherencegram, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.ref_chan = self.args.ref or self.chan_list[0]
         # deal with channel type (e.g. m-trend)
         if ',' in self.ref_chan:
@@ -48,7 +41,7 @@ class Coherencegram(Spectrogram):
 
     @classmethod
     def arg_channels(cls, parser):
-        group = super(Coherencegram, cls).arg_channels(parser)
+        group = super().arg_channels(parser)
         group.add_argument('--ref', help='Reference channel against which '
                                          'others will be compared')
         return group
@@ -61,9 +54,7 @@ class Coherencegram(Spectrogram):
                 args.imin = 0.
             if args.imax is None:
                 args.imax = 1.
-        if args.cmap is None and DEFAULT_CMAP is not None:
-            args.cmap = DEFAULT_CMAP.name
-        return super(Coherencegram, self)._finalize_arguments(args)
+        return super()._finalize_arguments(args)
 
     def get_ylabel(self):
         """Text for y-axis label
