@@ -1,7 +1,5 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (C) Joseph Areeda (2015)
+# Copyright (C) Joseph Areeda (2015-2021)
 #
 # This file is part of GWpy.
 #
@@ -17,13 +15,14 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with GWpy.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 """ Spectrogram plots
 """
 
 from numpy import percentile
-from .cliproduct import (FFTMixin, TimeDomainProduct, ImageProduct, unique)
+
+from .cliproduct import (FFTMixin, TimeDomainProduct, ImageProduct)
+from ..utils import unique
 
 __author__ = 'Joseph Areeda <joseph.areeda@ligo.org>'
 
@@ -34,7 +33,7 @@ class Spectrogram(FFTMixin, TimeDomainProduct, ImageProduct):
     action = 'spectrogram'
 
     def __init__(self, *args, **kwargs):
-        super(Spectrogram, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         #: attribute to hold calculated Spectrogram data array
         self.result = None
@@ -46,7 +45,7 @@ class Spectrogram(FFTMixin, TimeDomainProduct, ImageProduct):
     def _finalize_arguments(self, args):
         if args.color_scale is None:
             args.color_scale = 'log'
-        super(Spectrogram, self)._finalize_arguments(args)
+        super()._finalize_arguments(args)
 
     @property
     def units(self):
@@ -74,7 +73,7 @@ class Spectrogram(FFTMixin, TimeDomainProduct, ImageProduct):
                 self.units[0].to_string('latex').strip('$'))
         elif len(self.units) == 1:
             return 'ASD ({0})'.format(self.units[0].to_string('generic'))
-        return super(Spectrogram, self).get_color_label()
+        return super().get_color_label()
 
     def get_stride(self):
         """Calculate the stride for the spectrogram
@@ -144,7 +143,7 @@ class Spectrogram(FFTMixin, TimeDomainProduct, ImageProduct):
             specgram = self.get_spectrogram()
 
             # there may be data that can't be processed
-            if specgram:
+            if specgram is not None:  # <-DMM: why is this here?
                 # apply normalisation
                 if args.norm:
                     specgram = specgram.ratio(args.norm)
@@ -175,16 +174,20 @@ class Spectrogram(FFTMixin, TimeDomainProduct, ImageProduct):
             args.ymax = self.result.yspan[1]
 
         specgram = self.result.crop(
-            args.xmin, min(args.xmax, self.result.xspan[1]),
-        ).crop_frequencies(args.ymin, args.ymax)
+            args.xmin, min(args.xmax, self.result.xspan[1])
+        )
+
+        # y axis cannot be cropped if non-linear
+        ax = self.plot.gca()
+        ax.set_ylim(args.ymin, args.ymax)
 
         # auto scale colours
         if args.norm:
             imin = specgram.value.min()
             imax = specgram.value.max()
         else:
-            imin = percentile(specgram, 1)
-            imax = percentile(specgram, 100)
+            imin = percentile(specgram.value, 1)
+            imax = percentile(specgram.value, 100)
         imin = args.imin if args.imin is not None else imin
         imax = args.imax if args.imax is not None else imax
 
