@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,26 +19,26 @@
 """This module defines the `Channel` and `ChannelList` classes.
 """
 
-from __future__ import print_function
-
 import re
 from copy import copy
 from math import ceil
-
-from six.moves.urllib.parse import urlparse
-
-import numpy
+from urllib.parse import urlparse
 
 from astropy import units
 from astropy.io import registry as io_registry
 
-from ..io import (datafind, nds2 as io_nds2)
+from ..io import nds2 as io_nds2
 from ..time import to_gps
 from ..utils.misc import if_not_none
 from .units import parse_unit
 
 __author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
 
+try:  # python >= 3.7
+    Pattern = re.Pattern
+except AttributeError:  # python < 3.7
+    Pattern = re._pattern_type
+
 QUOTE_REGEX = re.compile(r'^[\s\"\']+|[\s\"\']+$')
 
 
@@ -264,8 +264,6 @@ class Channel(object):
     def type(self, type_):
         if type_ is None:
             self._type = None
-        elif isinstance(type_, int):
-            self._type = io_nds2.Nds2ChannelType(type_).name
         else:
             self._type = io_nds2.Nds2ChannelType.find(type_).name
 
@@ -295,7 +293,7 @@ class Channel(object):
         if type_ is None:
             self._dtype = None
         else:
-            self._dtype = numpy.dtype(type_)
+            self._dtype = io_nds2.Nds2DataType.find(type_).dtype
 
     @property
     def url(self):
@@ -394,14 +392,14 @@ class Channel(object):
     def ndsname(self):
         """Name of this channel as stored in the NDS database
         """
-        if self.type not in [None, 'raw', 'reduced']:
+        if self.type not in [None, 'raw', 'reduced', 'online']:
             return '%s,%s' % (self.name, self.type)
         return self.name
 
     # -- classmethods ---------------------------
 
     @classmethod
-    def query(cls, name, use_kerberos=None, debug=False):
+    def query(cls, name, kerberos=None):
         """Query the LIGO Channel Information System for the `Channel`
         matching the given name
 
@@ -410,23 +408,18 @@ class Channel(object):
         name : `str`
             name of channel
 
-        use_kerberos : `bool`, optional
+        kerberos : `bool`, optional
             use an existing Kerberos ticket as the authentication credential,
             default behaviour will check for credentials and request username
             and password if none are found (`None`)
 
-        debug : `bool`, optional
-            print verbose HTTP connection status for debugging,
-            default: `False`
-
         Returns
         -------
         c : `Channel`
              a new `Channel` containing all of the attributes set from
              its entry in the CIS
         """
-        channellist = ChannelList.query(name, use_kerberos=use_kerberos,
-                                        debug=debug)
+        channellist = ChannelList.query(name, kerberos=kerberos)
         if not channellist:
             raise ValueError("No channels found matching '%s'" % name)
         if len(channellist) > 1:
@@ -477,24 +470,13 @@ class Channel(object):
     def from_nds2(cls, nds2channel):
         """Generate a new channel using an existing nds2.channel object
         """
-        # extract metadata
-        name = nds2channel.name
-        sample_rate = nds2channel.sample_rate
-        unit = nds2channel.signal_units
-        if not unit:
-            unit = None
-        ctype = nds2channel.channel_type_to_string(nds2channel.channel_type)
-        # get dtype
-        dtype = {  # pylint: disable: no-member
-            nds2channel.DATA_TYPE_INT16: numpy.int16,
-            nds2channel.DATA_TYPE_INT32: numpy.int32,
-            nds2channel.DATA_TYPE_INT64: numpy.int64,
-            nds2channel.DATA_TYPE_FLOAT32: numpy.float32,
-            nds2channel.DATA_TYPE_FLOAT64: numpy.float64,
-            nds2channel.DATA_TYPE_COMPLEX32: numpy.complex64,
-        }.get(nds2channel.data_type)
-        return cls(name, sample_rate=sample_rate, unit=unit, dtype=dtype,
-                   type=ctype)
+        return cls(
+            nds2channel.name,
+            sample_rate=nds2channel.sample_rate,
+            unit=nds2channel.signal_units or None,
+            dtype=nds2channel.data_type,
+            type=nds2channel.channel_type,
+        )
 
     # -- methods --------------------------------
 
@@ -583,6 +565,7 @@ class Channel(object):
             the first matching frametype containing the this channel
             (`return_all=False`, or a `list` of all matches
         """
+        from ..io import datafind
         return datafind.find_frametype(
             self, gpstime=gpstime, frametype_match=frametype_match,
             host=host, port=port, return_all=return_all,
@@ -740,7 +723,7 @@ class ChannelList(list):
             a new `ChannelList` containing the matching channels
         """
         # format name regex
-        if isinstance(name, re._pattern_type):
+        if isinstance(name, Pattern):
             flags = name.flags
             name = name.pattern
         else:
@@ -760,24 +743,26 @@ class ChannelList(list):
             sample_rate = (sample_rate.value if
                            isinstance(sample_rate, units.Quantity) else
                            float(sample_rate))
-            matched = [entry for entry in matched if entry.sample_rate and
-                       entry.sample_rate.value == sample_rate]
+            matched = [entry for entry in matched if entry.sample_rate
+                       and entry.sample_rate.value == sample_rate]
 
         if sample_range is not None:
-            matched = [entry for entry in matched if
-                       sample_range[0] <= entry.sample_rate.value <=
-                       sample_range[1]]
+            matched = [
+                e for e in matched
+                if sample_range[0] <= e.sample_rate.value <= sample_range[1]
+            ]
 
         for attr, val in others.items():
             if val is not None:
-                matched = [entry for entry in matched if
-                           (hasattr(entry, attr) and
-                            getattr(entry, attr) == val)]
+                matched = [
+                    e for e in matched
+                    if hasattr(e, attr) and getattr(e, attr) == val
+                ]
 
         return self.__class__(matched)
 
     @classmethod
-    def query(cls, name, use_kerberos=None, debug=False):
+    def query(cls, name, kerberos=None):
         """Query the LIGO Channel Information System a `ChannelList`.
 
         Parameters
@@ -785,22 +770,18 @@ class ChannelList(list):
         name : `str`
             name of channel, or part of it.
 
-        use_kerberos : `bool`, optional
+        kerberos : `bool`, optional
             use an existing Kerberos ticket as the authentication credential,
             default behaviour will check for credentials and request username
             and password if none are found (`None`)
 
-        debug : `bool`, optional
-            print verbose HTTP connection status for debugging,
-            default: `False`
-
         Returns
         -------
         channels : `ChannelList`
             a new list containing all `Channels <Channel>` found.
         """
         from .io import cis
-        return cis.query(name, use_kerberos=use_kerberos, debug=debug)
+        return cis.query(name, kerberos=kerberos)
 
     @classmethod
     def query_nds2(cls, names, host=None, port=None, connection=None,
