@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,14 +19,12 @@
 """This module provides a spectral-variation histogram class
 """
 
-from six.moves import range
-
 import numpy
 
-from astropy import units
 from astropy.io import registry as io_registry
+from astropy.units import Quantity
 
-from ..types import (Quantity, Array2D)
+from ..types import Array2D
 from ..types.sliceutils import null_slice
 from ..segments import Segment
 from . import FrequencySeries
@@ -180,6 +178,11 @@ class SpectralVariance(Array2D):
         **kwargs
             Other keywords are (in general) specific to the given ``format``.
 
+        Raises
+        ------
+        IndexError
+            if ``source`` is an empty list
+
         Notes
         -----"""
         return io_registry.read(cls, source, *args, **kwargs)
@@ -210,7 +213,7 @@ class SpectralVariance(Array2D):
     def __getitem__(self, item):
         # disable slicing bins
         if not isinstance(item, tuple) or null_slice(item[1]):
-            return super(SpectralVariance, self).__getitem__(item)
+            return super().__getitem__(item)
         raise NotImplementedError("cannot slice SpectralVariance across bins")
     __getitem__.__doc__ = Array2D.__getitem__.__doc__
 
@@ -254,10 +257,10 @@ class SpectralVariance(Array2D):
         specvar : `SpectralVariance`
             2D-array of spectral frequency-amplitude counts
 
-        See Also
+        See also
         --------
-        :func:`numpy.histogram`
-            for details on specifying bins and weights
+        numpy.histogram
+            The histogram function
         """
         # parse args and kwargs
         if not spectrograms:
@@ -275,7 +278,7 @@ class SpectralVariance(Array2D):
 
         # get data and bins
         spectrogram = spectrograms[0]
-        data = numpy.vstack(s.value for s in spectrograms)
+        data = numpy.vstack([s.value for s in spectrograms])
         if bins is None:
             if low is None and log:
                 low = numpy.log10(data.min() / 2)
@@ -294,7 +297,7 @@ class SpectralVariance(Array2D):
             else:
                 bins = numpy.linspace(low, high, num=nbins+1)
         nbins = bins.size-1
-        bins = bins * spectrogram.unit
+        qbins = bins * spectrogram.unit
 
         # loop over frequencies
         out = numpy.zeros((data.shape[1], nbins))
@@ -306,7 +309,7 @@ class SpectralVariance(Array2D):
 
         # return SpectralVariance
         name = '%s variance' % spectrogram.name
-        new = cls(out, bins, epoch=spectrogram.epoch, name=name,
+        new = cls(out, qbins, epoch=spectrogram.epoch, name=name,
                   channel=spectrogram.channel, f0=spectrogram.f0,
                   df=spectrogram.df)
         return new
@@ -333,9 +336,7 @@ class SpectralVariance(Array2D):
             cumsumvals = numpy.cumsum(self.value[i, :])
 
             # Find value nearest requested percentile
-            abs_cumsumvals_minus_percentile = numpy.abs(cumsumvals -
-                                                        percentile)
-            minindex = abs_cumsumvals_minus_percentile.argmin()
+            minindex = numpy.abs(cumsumvals - percentile).argmin()
             val = self.bins[minindex]
             out[i] = val
 
@@ -347,9 +348,11 @@ class SpectralVariance(Array2D):
         if method == 'imshow':
             raise TypeError("plotting a {0} with {1}() is not "
                             "supported".format(type(self).__name__, method))
-        bins = self.bins
-        if (numpy.all(bins > 0) and
-                numpy.allclose(numpy.diff(numpy.log10(bins), n=2), 0)):
+        bins = self.bins.value
+        if (
+            numpy.all(bins > 0)
+            and numpy.allclose(numpy.diff(numpy.log10(bins), n=2), 0)
+        ):
             kwargs.setdefault('yscale', 'log')
         kwargs.update(method=method, xscale=xscale)
-        return super(SpectralVariance, self).plot(**kwargs)
+        return super().plot(**kwargs)
