@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -29,8 +29,7 @@ import os
 import re
 import subprocess
 import sys
-
-from six.moves import input
+from collections import OrderedDict
 
 from ..utils.shell import which
 
@@ -38,6 +37,11 @@ __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 
 __all__ = ['kinit']
 
+try:
+    _IPYTHON = __IPYTHON__
+except NameError:
+    _IPYTHON = False
+
 
 class KerberosError(RuntimeError):
     """Kerberos (krb5) operation failed
@@ -46,7 +50,7 @@ class KerberosError(RuntimeError):
 
 
 def kinit(username=None, password=None, realm=None, exe=None, keytab=None,
-          krb5ccname=None, verbose=False):
+          krb5ccname=None, verbose=None):
     """Initialise a kerberos (krb5) ticket.
 
     This allows authenticated connections to, amongst others, NDS2
@@ -55,38 +59,49 @@ def kinit(username=None, password=None, realm=None, exe=None, keytab=None,
     Parameters
     ----------
     username : `str`, optional
-        name of user, will be prompted for if not given
+        name of user, will be prompted for if not given.
+
     password : `str`, optional
         cleartext password of user for given realm, will be prompted for
-        if not given
-    realm : `str`
-        name of realm to authenticate against, defaults to 'LIGO.ORG'
-        if not given or parsed from the keytab
-    exe : `str`
-        path to kinit executable
-    keytab : `str`
+        if not given.
+
+    realm : `str`, optional
+        name of realm to authenticate against, read from keytab if available,
+        defaults to ``'LIGO.ORG'``.
+
+    exe : `str`, optional
+        path to kinit executable.
+
+    keytab : `str`, optional
         path to keytab file. If not given this will be read from the
-        ``KRB5_KTNAME`` environment variable. See notes for more details
+        ``KRB5_KTNAME`` environment variable. See notes for more details.
+
+    krb5ccname : `str`, optional
+        path to Kerberos credentials cache.
+
+    verbose : `bool`, optional
+        print verbose output (if `True`), or not (`False)`; default is `True`
+        if any user-prompting is needed, otherwise `False`.
 
     Notes
     -----
-    If a keytab is given, or is read from the KRB5_KTNAME environment
+    If a keytab is given, or is read from the ``KRB5_KTNAME`` environment
     variable, this will be used to guess the username and realm, if it
-    contains only a single credential
+    contains only a single credential.
 
     Examples
     --------
     Example 1: standard user input, with password prompt::
 
-        >>> kinit('albert.einstein')
-        Password for albert.einstein@LIGO.ORG:
-        Kerberos ticket generated for albert.einstein@LIGO.ORG
+    >>> kinit('albert.einstein')
+    Password for albert.einstein@LIGO.ORG:
+    Kerberos ticket generated for albert.einstein@LIGO.ORG
 
     Example 2: extract username and realm from keytab, and use that
     in authentication::
 
-        >>> kinit(keytab='~/.kerberos/ligo.org.keytab')
-        Kerberos ticket generated for albert.einstein@LIGO.ORG
+    >>> kinit(keytab='~/.kerberos/ligo.org.keytab', verbose=True)
+    Kerberos ticket generated for albert.einstein@LIGO.ORG
     """
     # get kinit path
     if exe is None:
@@ -114,6 +129,18 @@ def kinit(username=None, password=None, realm=None, exe=None, keytab=None,
             else:
                 keytab = None
 
+    # refuse to prompt if we can't get an answer
+    # note: jupyter streams are not recognised as interactive
+    #       (isatty() returns False) so we have a special case here
+    if (
+        not sys.stdout.isatty()
+        and not _IPYTHON
+        and (username is None or (not keytab and password is None))
+    ):
+        raise KerberosError("cannot generate kerberos ticket in a "
+                            "non-interactive session, please manually create "
+                            "a ticket, or consider using a keytab file")
+
     # get credentials
     if realm is None:
         realm = 'LIGO.ORG'
@@ -124,8 +151,7 @@ def kinit(username=None, password=None, realm=None, exe=None, keytab=None,
     identity = '{}@{}'.format(username, realm)
     if not keytab and password is None:
         verbose = True
-        password = getpass.getpass(prompt="Password for {}: ".format(identity),
-                                   stream=sys.stdout)
+        password = getpass.getpass(prompt="Password for {}: ".format(identity))
 
     # format kinit command
     if keytab:
@@ -138,13 +164,14 @@ def kinit(username=None, password=None, realm=None, exe=None, keytab=None,
         krbenv = None
 
     # execute command
-    kget = subprocess.Popen(cmd, env=krbenv,
-                            stdout=subprocess.PIPE,
-                            stderr=subprocess.PIPE,
+    kget = subprocess.Popen(cmd, env=krbenv, stdout=subprocess.PIPE,
                             stdin=subprocess.PIPE)
     if not keytab:
-        kget.communicate(password)
+        kget.communicate(password.encode('utf-8'))
     kget.wait()
+    retcode = kget.poll()
+    if retcode:
+        raise subprocess.CalledProcessError(kget.returncode, ' '.join(cmd))
     if verbose:
         print("Kerberos ticket generated for {}".format(identity))
 
@@ -157,6 +184,18 @@ def parse_keytab(keytab):
     ----------
     keytab : `str`
         path to keytab file
+
+    Returns
+    -------
+    creds : `list` of `tuple`
+        the (unique) list of `(username, realm, kvno)` as read from the
+        keytab file
+
+    Examples
+    --------
+    >>> from gwpy.io.kerberos import parse_keytab
+    >>> print(parse_keytab("creds.keytab"))
+    [('albert.einstein', 'LIGO.ORG', 1)]
     """
     try:
         out = subprocess.check_output(['klist', '-k', keytab],
@@ -167,12 +206,15 @@ def parse_keytab(keytab):
         raise KerberosError("Cannot read keytab {!r}".format(keytab))
     principals = []
     for line in out.splitlines():
+        if isinstance(line, bytes):
+            line = line.decode('utf-8')
         try:
-            num, principal, = re.split(r'\s+', line.strip(' '), 1)
+            kvno, principal, = re.split(r'\s+', line.strip(' '), 1)
         except ValueError:
             continue
         else:
-            if not num.isdigit():
+            if not kvno.isdigit():
                 continue
-            principals.append(principal.split('@'))
-    return principals
+            principals.append(tuple(principal.split('@')) + (int(kvno),))
+    # return unique, ordered list
+    return list(OrderedDict.fromkeys(principals).keys())
