@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2014)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -22,26 +22,83 @@ All specific unified input/output for class objects should be placed in
 an 'io' subdirectory of the containing directory for that class.
 """
 
+import builtins
 import os.path
+import re
 from contextlib import contextmanager
 from functools import wraps
-
-from six import string_types
+from importlib import import_module
+from numbers import Integral
 
 import numpy
 
-from .cache import (file_list, FILE_LIKE)
+try:
+    from ligo.lw.ligolw import (
+        ElementError as LigolwElementError,
+        LIGOLWContentHandler,
+    )
+except ImportError:  # no ligo.lw
+    LigolwElementError = None
+    LIGOLWContentHandler = None
+
+from .utils import (file_list, FILE_LIKE)
 
 __author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
 
+# record actual import function
+_real_import = builtins.__import__
+
+# XML elements
 XML_SIGNATURE = b'<?xml'
 LIGOLW_SIGNATURE = b'<!doctype ligo_lw'
 LIGOLW_ELEMENT = b'<ligo_lw>'
 
+LIGO_LW_MODULE_REGEX = re.compile(r"\Aligo.lw")
+
+# regex to catch all errors associated with reading an old-format file
+# with the new library
+_compat_errors = {
+    r"invalid Column \'process_id\'",
+    r"invalid type \'ilwd:char\'"
+}
+LIGO_LW_COMPAT_ERROR = re.compile("({})".format("|".join(_compat_errors)))
+
+
+# -- import hackery to support ligo.lw and glue.ligolw concurrently -----------
+
+def _ligolw_compat_import(name, *args, **kwargs):
+    name = LIGO_LW_MODULE_REGEX.sub("glue.ligolw", name)
+    return _real_import(name, *args, **kwargs)
+
+
+def _is_glue_ligolw_object(obj):
+    if not isinstance(obj, type):
+        obj = type(obj)
+    return str(obj.__module__).startswith("glue.ligolw")
+
+
+def ilwdchar_compat(func):
+    @wraps(func)
+    def wrapped(*args, **kwargs):
+        use_compat = kwargs.pop(
+            "ilwdchar_compat",
+            any(map(
+                _is_glue_ligolw_object,
+                list(args) + list(kwargs.values()),
+            )),
+        )
+        try:
+            if use_compat:
+                builtins.__import__ = _ligolw_compat_import
+            return func(*args, **kwargs)
+        finally:
+            builtins.__import__ = _real_import
+    return wrapped
+
 
 # -- hack around around TypeError from LIGOTimeGPS(numpy.int32(...)) ----------
 
-def _ligotimegps(s, ns):
+def _ligotimegps(s, ns=0):
     """Catch TypeError and cast `s` and `ns` to `int`
     """
     from lal import LIGOTimeGPS
@@ -52,72 +109,72 @@ def _ligotimegps(s, ns):
 
 
 @contextmanager
-def patch_ligotimegps():
+def patch_ligotimegps(module="ligo.lw.lsctables"):
     """Context manager to on-the-fly patch LIGOTimeGPS to accept all int types
     """
-    from glue.ligolw import lsctables
-    orig = lsctables.LIGOTimeGPS
-    lsctables.LIGOTimeGPS = _ligotimegps
-    yield
-    lsctables.LIGOTimeGPS = orig
+    module = import_module(module)
+    orig = module.LIGOTimeGPS
+    module.LIGOTimeGPS = _ligotimegps
+    try:
+        yield
+    finally:
+        module.LIGOTimeGPS = orig
 
 
 # -- content handling ---------------------------------------------------------
 
+@ilwdchar_compat
 def get_partial_contenthandler(element):
     """Build a `PartialLIGOLWContentHandler` to read only this element
 
     Parameters
     ----------
-    element : `type`
-        the element class to be read, subclass of
-        :class:`~glue.ligolw.ligolw.Element`
+    element : `type`, subclass of :class:`~ligo.lw.ligolw.Element`
+        the element class to be read,
 
     Returns
     -------
     contenthandler : `type`
-        a subclass of :class:`~glue.ligolw.ligolw.PartialLIGOLWContentHandler`
+        a subclass of :class:`~ligo.lw.ligolw.PartialLIGOLWContentHandler`
         to read only the given `element`
     """
-    from glue.ligolw.ligolw import PartialLIGOLWContentHandler
-    from glue.ligolw.table import Table
+    from ligo.lw.ligolw import PartialLIGOLWContentHandler
+    from ligo.lw.table import Table
 
     if issubclass(element, Table):
         def _element_filter(name, attrs):
-            # pylint: disable=unused-argument
             return element.CheckProperties(name, attrs)
     else:
-        def _element_filter(name, attrs):
-            # pylint: disable=unused-argument
+        def _element_filter(name, _):
             return name == element.tagName
 
     return build_content_handler(PartialLIGOLWContentHandler, _element_filter)
 
 
+@ilwdchar_compat
 def get_filtering_contenthandler(element):
     """Build a `FilteringLIGOLWContentHandler` to exclude this element
 
     Parameters
     ----------
-    element : `type`, subclass of :class:`~glue.ligolw.ligolw.Element`
+    element : `type`, subclass of :class:`~ligo.lw.ligolw.Element`
         the element to exclude (and its children)
 
     Returns
     -------
     contenthandler : `type`
         a subclass of
-        :class:`~glue.ligolw.ligolw.FilteringLIGOLWContentHandler` to
+        :class:`~ligo.lw.ligolw.FilteringLIGOLWContentHandler` to
         exclude an element and its children
     """
-    from glue.ligolw.ligolw import FilteringLIGOLWContentHandler
-    from glue.ligolw.table import Table
+    from ligo.lw.ligolw import FilteringLIGOLWContentHandler
+    from ligo.lw.table import Table
 
     if issubclass(element, Table):
         def _element_filter(name, attrs):
-            # pylint: disable=unused-argument
             return ~element.CheckProperties(name, attrs)
     else:
-        def _element_filter(name, attrs):
+        def _element_filter(name, _):
             # pylint: disable=unused-argument
             return name != element.tagName
 
@@ -125,29 +182,30 @@ def get_filtering_contenthandler(element):
                                  _element_filter)
 
 
+@ilwdchar_compat
 def build_content_handler(parent, filter_func):
     """Build a `~xml.sax.handler.ContentHandler` with a given filter
     """
-    from glue.ligolw.lsctables import use_in
+    from ligo.lw.lsctables import use_in
 
     class _ContentHandler(parent):
         # pylint: disable=too-few-public-methods
         def __init__(self, document):
-            super(_ContentHandler, self).__init__(document, filter_func)
+            super().__init__(document, filter_func)
 
     return use_in(_ContentHandler)
 
 
 # -- reading ------------------------------------------------------------------
 
-def read_ligolw(source, contenthandler=None, verbose=False,
-                non_lsc_tables_ok=True):
+@ilwdchar_compat
+def read_ligolw(source, contenthandler=LIGOLWContentHandler, **kwargs):
     """Read one or more LIGO_LW format files
 
     Parameters
     ----------
-    source : `str`, `file`, `list`
-        one or more open files or file paths to read
+    source : `str`, `file`
+        the open file or file path to read
 
     contenthandler : `~xml.sax.handler.ContentHandler`, optional
         content handler used to parse document
@@ -155,18 +213,15 @@ def read_ligolw(source, contenthandler=None, verbose=False,
     verbose : `bool`, optional
         be verbose when reading files, default: `False`
 
-    non_lsc_tables_ok : `bool`, optional
-        if `False` error on unrecognised tables in documents, default: `True`
-
     Returns
     -------
-    xmldoc : :class:`~glue.ligolw.ligolw.Document`
+    xmldoc : :class:`~ligo.lw.ligolw.Document`
         the document object as parsed from the file(s)
     """
-    from glue.ligolw.ligolw import (Document, LIGOLWContentHandler)
-    from glue.ligolw import types
-    from glue.ligolw.lsctables import use_in
-    from glue.ligolw.utils.ligolw_add import ligolw_add
+    from ligo.lw.ligolw import Document
+    from ligo.lw import types
+    from ligo.lw.lsctables import use_in
+    from ligo.lw.utils import (load_url, ligolw_add)
 
     # mock ToPyType to link to numpy dtypes
     topytype = types.ToPyType.copy()
@@ -174,15 +229,37 @@ def read_ligolw(source, contenthandler=None, verbose=False,
         if key in types.ToNumPyType:
             types.ToPyType[key] = numpy.dtype(types.ToNumPyType[key]).type
 
-    # set default content handler
-    if contenthandler is None:
-        contenthandler = use_in(LIGOLWContentHandler)
+    contenthandler = use_in(contenthandler)
 
     # read one or more files into a single Document
+    source = file_list(source)
     try:
-        return ligolw_add(Document(), file_list(source),
-                          contenthandler=contenthandler, verbose=verbose,
-                          non_lsc_tables_ok=non_lsc_tables_ok)
+        if len(source) == 1:
+            return load_url(
+                source[0],
+                contenthandler=contenthandler,
+                **kwargs
+            )
+        return ligolw_add.ligolw_add(
+            Document(),
+            source,
+            contenthandler=contenthandler,
+            **kwargs
+        )
+    except LigolwElementError as exc:
+        # failed to read with ligo.lw,
+        # try again with glue.ligolw (ilwdchar_compat)
+        if LIGO_LW_COMPAT_ERROR.search(str(exc)):
+            try:
+                return read_ligolw(
+                    source,
+                    contenthandler=contenthandler,
+                    ilwdchar_compat=True,
+                    **kwargs
+                )
+            except Exception:  # if fails for any reason, use original error
+                pass
+        raise
     finally:  # replace ToPyType
         types.ToPyType = topytype
 
@@ -191,20 +268,20 @@ def with_read_ligolw(func=None, contenthandler=None):
     """Decorate a LIGO_LW-reading function to open a filepath if needed
 
     ``func`` should be written to presume a
-    :class:`~glue.ligolw.ligolw.Document` as the first positional argument
+    :class:`~ligo.lw.ligolw.Document` as the first positional argument
     """
     def decorator(func_):
         # pylint: disable=missing-docstring
         @wraps(func_)
         def decorated_func(source, *args, **kwargs):
             # pylint: disable=missing-docstring
-            from glue.ligolw.ligolw import Document
-            if not isinstance(source, Document):
+            from ligo.lw.ligolw import Document
+            from glue.ligolw.ligolw import Document as GlueDocument
+            if not isinstance(source, (Document, GlueDocument)):
                 read_kw = {
                     'contenthandler': kwargs.pop('contenthandler',
                                                  contenthandler),
                     'verbose': kwargs.pop('verbose', False),
-                    'non_lsc_tables_ok': kwargs.pop('non_lsc_tables_ok', True),
                 }
                 return func_(read_ligolw(source, **read_kw), *args, **kwargs)
             return func_(source, *args, **kwargs)
@@ -218,16 +295,17 @@ def with_read_ligolw(func=None, contenthandler=None):
 
 # -- reading ------------------------------------------------------------------
 
+@ilwdchar_compat
 def read_table(source, tablename=None, columns=None, contenthandler=None,
                **kwargs):
-    """Read a :class:`~glue.ligolw.table.Table` from one or more LIGO_LW files
+    """Read a :class:`~ligo.lw.table.Table` from one or more LIGO_LW files
 
     Parameters
     ----------
     source : `Document`, `file`, `str`, `CacheEntry`, `list`
         object representing one or more files. One of
 
-        - a LIGO_LW :class:`~glue.ligolw.ligolw.Document`
+        - a LIGO_LW :class:`~ligo.lw.ligolw.Document`
         - an open `file`
         - a `str` pointing to a file path on disk
         - a formatted :class:`~lal.utils.CacheEntry` representing one file
@@ -247,15 +325,21 @@ def read_table(source, tablename=None, columns=None, contenthandler=None,
 
     Returns
     -------
-    table : :class:`~glue.ligolw.table.Table`
+    table : :class:`~ligo.lw.table.Table`
         `Table` of data
     """
-    from glue.ligolw.ligolw import Document
-    from glue.ligolw import (table, lsctables)
+    from ligo.lw.ligolw import Document
+    from ligo.lw import (table, lsctables)
+
+    # get ilwdchar_compat to pass to read_ligolw()
+    if Document.__module__.startswith("glue"):
+        kwargs["ilwdchar_compat"] = True
 
     # get content handler to read only this table (if given)
     if tablename is not None:
-        tableclass = lsctables.TableByName[table.Table.TableName(tablename)]
+        tableclass = lsctables.TableByName[
+            table.Table.TableName(tablename)
+        ]
         if contenthandler is None:
             contenthandler = get_partial_contenthandler(tableclass)
 
@@ -268,8 +352,9 @@ def read_table(source, tablename=None, columns=None, contenthandler=None,
     if isinstance(source, Document):
         xmldoc = source
     else:
+        if contenthandler is None:
+            contenthandler = lsctables.use_in(LIGOLWContentHandler)
         try:
-
             xmldoc = read_ligolw(source, contenthandler=contenthandler,
                                  **kwargs)
         finally:  # reinstate original set of loading column names
@@ -295,6 +380,7 @@ def read_table(source, tablename=None, columns=None, contenthandler=None,
 
 # -- writing ------------------------------------------------------------------
 
+@ilwdchar_compat
 def open_xmldoc(fobj, **kwargs):
     """Try and open an existing LIGO_LW-format file, or create a new Document
 
@@ -305,59 +391,76 @@ def open_xmldoc(fobj, **kwargs):
 
     **kwargs
         other keyword arguments to pass to
-        :func:`~glue.ligolw.utils.load_filename`, or
-        :func:`~glue.ligolw.utils.load_fileobj` as appropriate
+        :func:`~ligo.lw.utils.load_filename`, or
+        :func:`~ligo.lw.utils.load_fileobj` as appropriate
 
     Returns
     --------
-    xmldoc : :class:`~glue.ligolw.ligolw.Document`
+    xmldoc : :class:`~ligo.lw.ligolw.Document`
         either the `Document` as parsed from an existing file, or a new, empty
         `Document`
     """
-    from glue.ligolw.lsctables import use_in
-    from glue.ligolw.ligolw import (Document, LIGOLWContentHandler)
-    from glue.ligolw.utils import load_filename, load_fileobj
+    from ligo.lw.ligolw import (Document, LIGOLWContentHandler)
+    from ligo.lw.lsctables import use_in
+    from ligo.lw.utils import (load_filename, load_fileobj)
+
+    use_in(kwargs.setdefault('contenthandler', LIGOLWContentHandler))
+
     try:  # try and load existing file
-        if isinstance(fobj, string_types):
-            kwargs.setdefault('contenthandler', use_in(LIGOLWContentHandler))
+        if isinstance(fobj, str):
             return load_filename(fobj, **kwargs)
         if isinstance(fobj, FILE_LIKE):
-            kwargs.setdefault('contenthandler', use_in(LIGOLWContentHandler))
             return load_fileobj(fobj, **kwargs)[0]
     except (OSError, IOError):  # or just create a new Document
         return Document()
+    except LigolwElementError as exc:
+        if LIGO_LW_COMPAT_ERROR.search(str(exc)):
+            try:
+                return open_xmldoc(fobj, ilwdchar_compat=True, **kwargs)
+            except Exception:  # for any reason, raise original
+                pass
+        raise
 
 
+@ilwdchar_compat
 def get_ligolw_element(xmldoc):
     """Find an existing <LIGO_LW> element in this XML Document
     """
-    from glue.ligolw.ligolw import LIGO_LW
-    if isinstance(xmldoc, LIGO_LW):
+    from ligo.lw.ligolw import LIGO_LW
+    try:
+        from glue.ligolw.ligolw import LIGO_LW as LIGO_LW2
+    except ImportError:
+        ligolw_types = (LIGO_LW,)
+    else:
+        ligolw_types = (LIGO_LW, LIGO_LW2)
+
+    if isinstance(xmldoc, ligolw_types):
         return xmldoc
     else:
         for node in xmldoc.childNodes:
-            if isinstance(node, LIGO_LW):
+            if isinstance(node, ligolw_types):
                 return node
     raise ValueError("Cannot find LIGO_LW element in XML Document")
 
 
+@ilwdchar_compat
 def write_tables_to_document(xmldoc, tables, overwrite=False):
     """Write the given LIGO_LW table into a :class:`Document`
 
     Parameters
     ----------
-    xmldoc : :class:`~glue.ligolw.ligolw.Document`
+    xmldoc : :class:`~ligo.lw.ligolw.Document`
         the document to write into
 
-    tables : `list` of :class:`~glue.ligolw.table.Table`
+    tables : `list` of :class:`~ligo.lw.table.Table`
         the set of tables to write
 
     overwrite : `bool`, optional, default: `False`
         if `True`, delete an existing instance of the table type, otherwise
         append new rows
     """
-    from glue.ligolw.ligolw import LIGO_LW
-    from glue.ligolw import lsctables
+    from ligo.lw.ligolw import LIGO_LW
+    from ligo.lw import lsctables
 
     # find or create LIGO_LW tag
     try:
@@ -383,15 +486,16 @@ def write_tables_to_document(xmldoc, tables, overwrite=False):
     return xmldoc
 
 
+@ilwdchar_compat
 def write_tables(target, tables, append=False, overwrite=False, **kwargs):
     """Write an LIGO_LW table to file
 
     Parameters
     ----------
-    target : `str`, `file`, :class:`~glue.ligolw.ligolw.Document`
+    target : `str`, `file`, :class:`~ligo.lw.ligolw.Document`
         the file or document to write into
 
-    tables : `list`, `tuple` of :class:`~glue.ligolw.table.Table`
+    tables : `list`, `tuple` of :class:`~ligo.lw.table.Table`
         the tables to write
 
     append : `bool`, optional, default: `False`
@@ -403,23 +507,26 @@ def write_tables(target, tables, append=False, overwrite=False, **kwargs):
 
     **kwargs
         other keyword arguments to pass to
-        :func:`~glue.ligolw.utils.load_filename`, or
-        :func:`~glue.ligolw.utils.load_fileobj` as appropriate
+        :func:`~ligo.lw.utils.load_filename`, or
+        :func:`~ligo.lw.utils.load_fileobj` as appropriate
     """
-    from glue.ligolw.ligolw import (Document, LIGO_LW, LIGOLWContentHandler)
-    from glue.ligolw import utils as ligolw_utils
+    from ligo.lw.ligolw import (Document, LIGO_LW, LIGOLWContentHandler)
+    from ligo.lw import utils as ligolw_utils
 
     # allow writing directly to XML
     if isinstance(target, (Document, LIGO_LW)):
         xmldoc = target
     # open existing document, if possible
-    elif append and not overwrite:
+    elif append:
         xmldoc = open_xmldoc(
             target, contenthandler=kwargs.pop('contenthandler',
                                               LIGOLWContentHandler))
     # fail on existing document and not overwriting
-    elif (not overwrite and isinstance(target, string_types) and
-          os.path.isfile(target)):
+    elif (
+        not overwrite
+        and isinstance(target, str)
+        and os.path.isfile(target)
+    ):
         raise IOError("File exists: {}".format(target))
     else:  # or create a new document
         xmldoc = Document()
@@ -427,24 +534,34 @@ def write_tables(target, tables, append=False, overwrite=False, **kwargs):
     # convert table to format
     write_tables_to_document(xmldoc, tables, overwrite=overwrite)
 
-    # write file
-    if isinstance(target, string_types):
-        kwargs.setdefault('gz', target.endswith('.gz'))
-        ligolw_utils.write_filename(xmldoc, target, **kwargs)
-    elif isinstance(target, FILE_LIKE):
-        kwargs.setdefault('gz', target.name.endswith('.gz'))
-        ligolw_utils.write_fileobj(xmldoc, target, **kwargs)
+    # find writer function and target filename
+    if isinstance(target, FILE_LIKE):
+        writer = ligolw_utils.write_fileobj
+        name = target.name
+    else:
+        writer = ligolw_utils.write_filename
+        name = str(target)
+
+    # handle gzip compression kwargs
+    if name.endswith('.gz') and writer.__module__.startswith('glue'):
+        kwargs.setdefault('gz', True)
+    elif name.endswith('.gz'):
+        kwargs.setdefault('compress', 'gz')
+
+    # write XML
+    writer(xmldoc, target, **kwargs)
 
 
 # -- utilities ----------------------------------------------------------------
 
+@ilwdchar_compat
 def list_tables(source):
     # pylint: disable=line-too-long
     """List the names of all tables in this file(s)
 
     Parameters
     ----------
-    source : `file`, `str`, :class:`~glue.ligolw.ligolw.Document`, `list`
+    source : `file`, `str`, :class:`~ligo.lw.ligolw.Document`, `list`
         one or more open files, file paths, or LIGO_LW `Document`s
 
     Examples
@@ -452,9 +569,11 @@ def list_tables(source):
     >>> from gwpy.io.ligolw import list_tables
     >>> print(list_tables('H1-LDAS_STRAIN-968654552-10.xml.gz'))
     ['process', 'process_params', 'sngl_burst', 'search_summary', 'segment_definer', 'segment_summary', 'segment']
-    """  # nopep8
-    from glue.ligolw.ligolw import (Document, Stream)
-    from glue.ligolw.table import Table
+    """  # noqa: E501
+    try:
+        from ligo.lw.ligolw import (Document, Stream)
+    except ImportError:  # no python-ligo-lw
+        from glue.ligolw.ligolw import Document, Stream
 
     # read file object
     if isinstance(source, Document):
@@ -466,11 +585,14 @@ def list_tables(source):
     # get list of table names
     tables = []
     for tbl in xmldoc.childNodes[0].childNodes:
-        if isinstance(tbl, Table):
+        try:
             tables.append(tbl.TableName(tbl.Name))
+        except AttributeError:  # not a table
+            continue
     return tables
 
 
+@ilwdchar_compat
 def to_table_type(val, cls, colname):
     """Cast a value to the correct type for inclusion in a LIGO_LW table
 
@@ -482,8 +604,8 @@ def to_table_type(val, cls, colname):
     val : `object`
         The input object to convert, of any type
 
-    cls : `type`
-        The sub-class of :class:`~glue.ligolw.table.Table` to map against
+    cls : `type`, subclass of :class:`~ligo.lw.table.Table`
+        the table class to map against
 
     colname : `str`
         The name of the mapping column
@@ -496,7 +618,7 @@ def to_table_type(val, cls, colname):
     Examples
     --------
     >>> from gwpy.io.ligolw import to_table_type as to_ligolw_type
-    >>> from glue.ligolw.lsctables import SnglBurstTable
+    >>> from ligo.lw.lsctables import SnglBurstTable
     >>> print(to_ligolw_type(1.0, SnglBurstTable, 'central_freq')))
     1.0
 
@@ -507,13 +629,15 @@ def to_table_type(val, cls, colname):
 
     Formatted fancy ILWD objects are left untouched:
 
-    >>> from glue.ligolw.ilwd import ilwdchar
+    >>> from ligo.lw.ilwd import ilwdchar
     >>> pid = ilwdchar('process:process_id:0')
     >>> print(to_ligolw_type(pid, SnglBurstTable, 'process_id')))
     process:process_id:1
     """
-    from glue.ligolw.types import (ToNumPyType as numpytypes,
-                                   ToPyType as pytypes)
+    from ligo.lw.types import (
+        ToNumPyType as numpytypes,
+        ToPyType as pytypes,
+    )
 
     # if nothing to do...
     if val is None or colname not in cls.validcolumns:
@@ -523,7 +647,8 @@ def to_table_type(val, cls, colname):
 
     # don't mess with formatted IlwdChar
     if llwtype == 'ilwd:char':
-        return _to_ilwd(val, cls.tableName, colname)
+        return _to_ilwd(val, cls.tableName, colname,
+                        ilwdchar_compat=_is_glue_ligolw_object(cls))
     # otherwise map to numpy or python types
     try:
         return numpy.typeDict[numpytypes[llwtype]](val)
@@ -531,16 +656,17 @@ def to_table_type(val, cls, colname):
         return pytypes[llwtype](val)
 
 
+@ilwdchar_compat
 def _to_ilwd(value, tablename, colname):
-    from glue.ligolw.ilwd import (ilwdchar, get_ilwdchar_class)
-    from glue.ligolw._ilwd import ilwdchar as IlwdChar
+    from ligo.lw.ilwd import (ilwdchar, get_ilwdchar_class)
+    from ligo.lw._ilwd import ilwdchar as IlwdChar
 
     if isinstance(value, IlwdChar) and value.column_name != colname:
         raise ValueError("ilwdchar '{0!s}' doesn't match column "
                          "{1!r}".format(value, colname))
     if isinstance(value, IlwdChar):
         return value
-    if isinstance(value, int):
+    if isinstance(value, Integral):
         return get_ilwdchar_class(tablename, colname)(value)
     return ilwdchar(value)
 
@@ -558,20 +684,31 @@ def is_ligolw(origin, filepath, fileobj, *args, **kwargs):
             line1 = fileobj.readline().lower()
             line2 = fileobj.readline().lower()
             try:
-                return (line1.startswith(XML_SIGNATURE) and
-                        line2.startswith((LIGOLW_SIGNATURE, LIGOLW_ELEMENT)))
+                return (
+                    line1.startswith(XML_SIGNATURE)
+                    and line2.startswith((LIGOLW_SIGNATURE, LIGOLW_ELEMENT))
+                )
             except TypeError:  # bytes vs str
-                return (line1.startswith(XML_SIGNATURE.decode('utf-8')) and
-                        line2.startswith((LIGOLW_SIGNATURE.decode('utf-8'),
-                                          LIGOLW_ELEMENT.decode('utf-8'))))
+                return (
+                    line1.startswith(XML_SIGNATURE.decode('utf-8'))
+                    and line2.startswith((
+                        LIGOLW_SIGNATURE.decode('utf-8'),
+                        LIGOLW_ELEMENT.decode('utf-8'),
+                    ))
+                )
         finally:
             fileobj.seek(loc)
     try:
-        from glue.ligolw.ligolw import Element
+        from ligo.lw.ligolw import Element
     except ImportError:
         return False
+    try:
+        from glue.ligolw.ligolw import Element as GlueElement
+    except ImportError:
+        element_types = (Element,)
     else:
-        return len(args) > 0 and isinstance(args[0], Element)
+        element_types = (Element, GlueElement)
+    return len(args) > 0 and isinstance(args[0], element_types)
 
 
 def is_xml(origin, filepath, fileobj, *args, **kwargs):
