@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -22,20 +22,21 @@
 import itertools
 import importlib
 import warnings
-from collections import (KeysView, ValuesView)
-
-from six.moves import zip_longest
+from collections.abc import (KeysView, ValuesView)
+from itertools import zip_longest
 
 import numpy
 
 from matplotlib import (figure, get_backend, _pylab_helpers)
 from matplotlib.artist import setp
 from matplotlib.gridspec import GridSpec
+from matplotlib.ticker import LogFormatterSciNotation
 from matplotlib.projections import get_projection_class
 
 from . import (colorbar as gcbar, utils)
-from .rc import (rcParams, MPL_RCPARAMS, get_subplot_params)
 from .gps import GPS_SCALES
+from .log import LogFormatter
+from .rc import (rcParams, MPL_RCPARAMS, get_subplot_params)
 
 __all__ = ['Plot']
 
@@ -96,8 +97,10 @@ class Plot(figure.Figure):
             _group_axes_data(data, flat=True)))
 
         # set default size for time-axis figures
-        if (kwargs.get('projection', None) == 'segments' or
-                kwargs.get('xscale') in GPS_SCALES):
+        if (
+            kwargs.get('projection', None) == 'segments'
+            or kwargs.get('xscale') in GPS_SCALES
+        ):
             kwargs.setdefault('figsize', (12, 6))
             kwargs.setdefault('xscale', 'auto-gps')
 
@@ -107,7 +110,8 @@ class Plot(figure.Figure):
         self._init_figure(**figure_kw)
 
         # initialise axes with data
-        self._init_axes(data, **kwargs)
+        if data or kwargs.get("geometry"):
+            self._init_axes(data, **kwargs)
 
     def _init_figure(self, **kwargs):
         from matplotlib import pyplot
@@ -119,7 +123,7 @@ class Plot(figure.Figure):
         # create Figure
         num = kwargs.pop('num', max(pyplot.get_fignums() or {0}) + 1)
         self._parse_subplotpars(kwargs)
-        super(Plot, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.number = num
 
         # add interactivity (scraped from pyplot.figure())
@@ -127,8 +131,10 @@ class Plot(figure.Figure):
         try:
             manager = backend_mod.new_figure_manager_given_figure(num, self)
         except AttributeError:
-            canvas = backend_mod.FigureCanvas(self)
-            manager = backend_mod.FigureManagerBase(canvas, 1)
+            upstream_mod = importlib.import_module(
+                pyplot.new_figure_manager.__module__)
+            canvas = upstream_mod.FigureCanvasBase(self)
+            manager = upstream_mod.FigureManagerBase(canvas, 1)
         manager._cidgcf = manager.canvas.mpl_connect(
             'button_press_event',
             lambda ev: _pylab_helpers.Gcf.set_active(manager))
@@ -214,12 +220,13 @@ class Plot(figure.Figure):
     def _parse_subplotpars(kwargs):
         # dynamically set the subplot positions based on the figure size
         # -- only if the user hasn't customised the subplot params
-        figsize = kwargs.get('figsize', rcParams['figure.figsize'])
+        figsize = kwargs.get('figsize') or rcParams['figure.figsize']
         subplotpars = get_subplot_params(figsize)
         use_subplotpars = 'subplotpars' not in kwargs and all([
-            rcParams['figure.subplot.%s' % pos] ==
-            MPL_RCPARAMS['figure.subplot.%s' % pos] for
-            pos in ('left', 'bottom', 'right', 'top')])
+            rcParams['figure.subplot.%s' % pos]
+            == MPL_RCPARAMS['figure.subplot.%s' % pos]
+            for pos in ('left', 'bottom', 'right', 'top')
+        ])
         if use_subplotpars:
             kwargs['subplotpars'] = subplotpars
 
@@ -269,7 +276,7 @@ class Plot(figure.Figure):
                 block = False
 
         # render
-        super(Plot, self).show(warn=warn)
+        super().show(warn=warn)
 
         # don't block on ipython with interactive backends
         if block is None and interactive_backend():
@@ -322,7 +329,7 @@ class Plot(figure.Figure):
     # -- colour bars ----------------------------
 
     def colorbar(self, mappable=None, cax=None, ax=None, fraction=0.,
-                 label=None, emit=True, **kwargs):
+                 emit=True, **kwargs):
         """Add a colorbar to the current `Plot`
 
         A colorbar must be associated with an `Axes` on this `Plot`,
@@ -356,7 +363,7 @@ class Plot(figure.Figure):
         cbar : `~matplotlib.colorbar.Colorbar`
             the newly added `Colorbar`
 
-        See Also
+        See also
         --------
         matplotlib.figure.Figure.colorbar
         matplotlib.colorbar.Colorbar
@@ -388,10 +395,26 @@ class Plot(figure.Figure):
             self, mappable, ax, cax=cax, fraction=fraction, **kwargs)
 
         # generate colour bar
-        cbar = super(Plot, self).colorbar(mappable, **kwargs)
+        cbar = super().colorbar(mappable, **kwargs)
+
+        # force the minor ticks to be the same as the major ticks
+        # in practice, this normally swaps out LogFormatterSciNotation to
+        # gwpy's LogFormatter; # this is hacky, and would be improved using a
+        # subclass of Colorbar in the first place, but matplotlib's
+        # cbar_factory doesn't support that
+        longaxis = (cbar.ax.yaxis if cbar.orientation == "vertical" else
+                    cbar.ax.xaxis)
+        if (
+                isinstance(cbar.formatter, LogFormatter)
+                and isinstance(
+                    longaxis.get_minor_formatter(),
+                    LogFormatterSciNotation,
+                )
+        ):
+            longaxis.set_minor_formatter(type(cbar.formatter)())
+
+        # record colorbar in parent object
         self.colorbars.append(cbar)
-        if label:  # mpl<1.3 doesn't accept label in Colorbar constructor
-            cbar.set_label(label)
 
         # update mappables for this axis
         if emit:
@@ -468,6 +491,12 @@ class Plot(figure.Figure):
         if axes_kw['sharex'] is ax and not ax.get_autoscalex_on():
             axes_kw['xlim'] = ax.get_xlim()
 
+        # if axes uses GPS scaling, copy the epoch as well
+        try:
+            axes_kw['epoch'] = ax.get_epoch()
+        except AttributeError:
+            pass
+
         # add new axes
         if ax.get_axes_locator():
             divider = ax.get_axes_locator()._axes_divider
@@ -553,7 +582,7 @@ def _group_axes_data(inputs, separate=None, flat=False):
         if any(isinstance(x, iterable_types + (dict,)) for x in inputs):
             separate = True
         # if data are of different types, default to separate
-        elif not all(type(x) is type(inputs[0]) for x in inputs):  # nopep8
+        elif not all(type(x) is type(inputs[0]) for x in inputs):  # noqa: E721
             separate = True
 
     # build list of lists
@@ -567,9 +596,11 @@ def _group_axes_data(inputs, separate=None, flat=False):
         #     structures, unless its a list of scalars in which case we
         #     should plot them all as one
         if (
-                isinstance(x, (KeysView, ValuesView)) or
-                isinstance(x, (list, tuple)) and (
-                    not x or not numpy.isscalar(x[0]))
+                isinstance(x, (KeysView, ValuesView))
+                or isinstance(x, (list, tuple)) and (
+                    not x
+                    or not numpy.isscalar(x[0])
+                )
         ):
             out.append(x)
 
