@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2018)
+# Copyright (C) Duncan Macleod (2018-2021)
 #
 # This file is part of GWpy.
 #
@@ -31,7 +31,7 @@ class LatexInlineDimensional(LatexInline):
 
     @classmethod
     def to_string(cls, unit):
-        u = '[{0}]'.format(super(LatexInlineDimensional, cls).to_string(unit))
+        u = '[{0}]'.format(super().to_string(unit))
 
         if unit.physical_type not in {None, 'unknown', 'dimensionless'}:
             ptype = cls._latex_escape(unit.physical_type.title())
