@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -26,30 +26,31 @@ The `DataQualityDict` is just a `dict` of flags, provided as a convenience
 for handling multiple flags over the same global time interval.
 """
 
-from __future__ import absolute_import
-
 import datetime
 import json
 import operator
 import os
 import re
 import warnings
-from io import StringIO
+from io import BytesIO
 from collections import OrderedDict
 from copy import (copy as shallowcopy, deepcopy)
+from functools import reduce
 from math import (floor, ceil)
+from queue import Queue
 from threading import Thread
-
-from six.moves import reduce
-from six.moves.urllib.error import (URLError, HTTPError)
-from six.moves.urllib.parse import urlparse
-from six.moves.queue import Queue
+from urllib.error import (URLError, HTTPError)
+from urllib.parse import urlparse
 
 from numpy import inf
 
 from astropy.io import registry as io_registry
 from astropy.utils.data import get_readable_fileobj
 
+from gwosc import timeline
+
+from dqsegdb2.query import query_segments
+
 from ..io.mp import read_multi as io_read_multi
 from ..time import to_gps, LIGOTimeGPS
 from ..utils.misc import if_not_none
@@ -66,8 +67,8 @@ re_TAG_VERSION = re.compile(r"\A(?P<tag>[^/]+):(?P<version>\d+)\Z")
 DEFAULT_SEGMENT_SERVER = os.getenv('DEFAULT_SEGMENT_SERVER',
                                    'https://segments.ligo.org')
 
-# -- utilities ----------------------------------------------------------------
 
+# -- utilities ----------------------------------------------------------------
 
 def _select_query_method(cls, url):
     """Select the correct query method based on the URL
@@ -383,7 +384,7 @@ class DataQualityFlag(object):
             ``$DEFAULT_SEGMENT_SERVER`` environment variable, or
             ``'https://segments.ligo.org'``
 
-        See Also
+        See also
         --------
         DataQualityFlag.query_segdb
         DataQualityFlag.query_dqsegdb
@@ -425,6 +426,9 @@ class DataQualityFlag(object):
             A new `DataQualityFlag`, with the `known` and `active` lists
             filled appropriately.
         """
+        warnings.warn("query_segdb is deprecated and will be removed in a "
+                      "future release", DeprecationWarning)
+
         # parse arguments
         qsegs = _parse_query_segments(args, cls.query_segdb)
 
@@ -471,14 +475,11 @@ class DataQualityFlag(object):
             A new `DataQualityFlag`, with the `known` and `active` lists
             filled appropriately.
         """
-        from dqsegdb import apicalls
-
         # parse arguments
         qsegs = _parse_query_segments(args, cls.query_dqsegdb)
 
         # get server
-        protocol, server = kwargs.pop(
-            'url', DEFAULT_SEGMENT_SERVER).split('://', 1)
+        url = kwargs.pop('url', DEFAULT_SEGMENT_SERVER)
 
         # parse flag
         out = cls(name=flag)
@@ -486,36 +487,26 @@ class DataQualityFlag(object):
             raise ValueError("Cannot parse ifo or tag (name) for flag %r"
                              % flag)
 
-        # other keyword arguments
-        request = kwargs.pop('request', 'metadata,active,known')
-
         # process query
         for start, end in qsegs:
+            # handle infinities
             if float(end) == +inf:
-                end = to_gps('now').seconds
-            if out.version is None:
-                data, versions, _ = apicalls.dqsegdbCascadedQuery(
-                    protocol, server, out.ifo, out.tag, request,
-                    int(start), int(end))
-                data['metadata'] = versions[-1]['metadata']
-            else:
-                try:
-                    data, _ = apicalls.dqsegdbQueryTimes(
-                        protocol, server, out.ifo, out.tag, out.version,
-                        request, int(start), int(end))
-                except HTTPError as exc:
-                    if exc.code == 404:  # if not found, annotate flag name
-                        exc.msg += ' [{0}]'.format(flag)
-                    raise
-            # read from json buffer
+                end = int(to_gps('now'))
+
+            # query
             try:
-                new = cls.read(StringIO(json.dumps(data)), format='json')
-            except TypeError as exc:
-                if 'initial_value must be unicode' in str(exc):  # python2
-                    new = cls.read(StringIO(json.dumps(data).decode('utf-8')),
-                                   format='json')
-                else:
-                    raise
+                data = query_segments(flag, int(start), int(end), host=url)
+            except HTTPError as exc:
+                if exc.code == 404:  # if not found, annotate flag name
+                    exc.msg += ' [{0}]'.format(flag)
+                raise
+
+            # read from json buffer
+            new = cls.read(
+                BytesIO(json.dumps(data).encode('utf-8')),
+                format='json',
+            )
+
             # restrict to query segments
             segl = SegmentList([Segment(start, end)])
             new.known &= segl
@@ -558,7 +549,7 @@ class DataQualityFlag(object):
         --------
         >>> from gwpy.segments import DataQualityFlag
         >>> print(DataQualityFlag.fetch_open_data('H1_DATA', 'Jan 1 2010',
-        ...                                       'Jan 2 2010'))"
+        ...                                       'Jan 2 2010'))
         <DataQualityFlag('H1:DATA',
                          known=[[946339215 ... 946425615)],
                          active=[[946340946 ... 946351800)
@@ -571,11 +562,10 @@ class DataQualityFlag(object):
                                  [946415770 ... 946422986)],
                          description=None)>
         """
-        from .io.losc import get_segments
-        start = to_gps(start)
-        end = to_gps(end)
+        start = to_gps(start).gpsSeconds
+        end = to_gps(end).gpsSeconds
         known = [(start, end)]
-        active = get_segments(flag, start, end, **kwargs)
+        active = timeline.get_segments(flag, start, end, **kwargs)
         return cls(flag.replace('_', ':', 1), known=known, active=active,
                    label=flag)
 
@@ -604,6 +594,10 @@ class DataQualityFlag(object):
             require segment start and stop times match printed duration,
             only valid for ``format='segwizard'``.
 
+        coalesce : `bool`, optional
+            if `True` coalesce the all segment lists before returning,
+            otherwise return exactly as contained in file(s).
+
         nproc : `int`, optional, default: 1
             number of CPUs to use for parallel reading of multiple files
 
@@ -616,17 +610,31 @@ class DataQualityFlag(object):
             formatted `DataQualityFlag` containing the active and known
             segments read from file.
 
+        Raises
+        ------
+        IndexError
+            if ``source`` is an empty list
+
         Notes
         -----"""
         if 'flag' in kwargs:  # pragma: no cover
             warnings.warn('\'flag\' keyword was renamed \'name\', this '
                           'warning will result in an error in the future')
             kwargs.setdefault('name', kwargs.pop('flags'))
+        coalesce = kwargs.pop('coalesce', False)
 
-        def _combine(flags):
-            return reduce(operator.or_, flags)
+        def combiner(flags):
+            """Combine `DataQualityFlag` from each file into a single object
+            """
+            out = flags[0]
+            for flag in flags[1:]:
+                out.known += flag.known
+                out.active += flag.active
+            if coalesce:
+                return out.coalesce()
+            return out
 
-        return io_read_multi(_combine, cls, source, *args, **kwargs)
+        return io_read_multi(combiner, cls, source, *args, **kwargs)
 
     @classmethod
     def from_veto_def(cls, veto):
@@ -634,7 +642,7 @@ class DataQualityFlag(object):
 
         Parameters
         ----------
-        veto : :class:`~glue.ligolw.lsctables.VetoDef`
+        veto : :class:`~ligo.lw.lsctables.VetoDef`
             veto definition to convert from
         """
         name = '%s:%s' % (veto.ifo, veto.name)
@@ -642,9 +650,7 @@ class DataQualityFlag(object):
             name += ':%d' % int(veto.version)
         except TypeError:
             pass
-        if veto.end_time == 0:
-            veto.end_time = +inf
-        known = Segment(veto.start_time, veto.end_time)
+        known = Segment(veto.start_time, veto.end_time or +inf)
         pad = (veto.start_pad, veto.end_pad)
         return cls(name=name, known=[known], category=veto.category,
                    description=veto.comment, padding=pad)
@@ -804,11 +810,11 @@ class DataQualityFlag(object):
         """
         def _round(seg):
             if contract:  # round inwards
-                a = ceil(seg[0])
-                b = floor(seg[1])
+                a = type(seg[0])(ceil(seg[0]))
+                b = type(seg[1])(floor(seg[1]))
             else:  # round outwards
-                a = floor(seg[0])
-                b = ceil(seg[1])
+                a = type(seg[0])(floor(seg[0]))
+                b = type(seg[1])(ceil(seg[1]))
             if a >= b:  # if segment is too short, return 'null' segment
                 return type(seg)(0, 0)  # will get coalesced away
             return type(seg)(a, b)
@@ -843,22 +849,35 @@ class DataQualityFlag(object):
         return self
 
     def __repr__(self):
-        indent = " " * len("<%s(" % self.__class__.__name__)
-        known = str(self.known).replace("\n",
-                                        "\n%s      " % indent).split('\n')
-        if len(known) > 10:
-            known = known[:3] + ['%s      ...' % indent] + known[-3:]
-        active = str(self.active).replace("\n",
-                                          "\n%s       " % indent).split('\n')
-        if len(active) > 10:
-            active = active[:3] + ['%s        ...' % indent] + active[-3:]
-        return ("<{1}({2},\n{0}known={3},\n{0}active={4},\n"
-                "{0}description={5})>".format(indent, self.__class__.__name__,
-                                              (self.name and repr(self.name) or
-                                               'No name'),
-                                              '\n'.join(known),
-                                              '\n'.join(active),
-                                              repr(self.description)))
+        prefix = "<{}(".format(type(self).__name__)
+        suffix = ")>"
+        indent = " " * len(prefix)
+
+        # format segment lists
+        known = str(self.known).replace(
+            "\n",
+            "\n{}      ".format(indent),
+        ).split("\n")
+        if len(known) > 10:  # use ellipsis
+            known = known[:3] + ['{}      ...'.format(indent)] + known[-3:]
+        active = str(self.active).replace(
+            "\n",
+            "\n{}       ".format(indent),
+        ).split('\n')
+        if len(active) > 10:  # use ellipsis
+            active = active[:3] + ['{}      ...'.format(indent)] + active[-3:]
+
+        # print the thing
+        return "".join((
+            prefix,
+            "\n{}".format(indent).join([
+                "{},".format(repr(self.name)),
+                "known={}".format("\n".join(known)),
+                "active={}".format("\n".join(active)),
+                "description={}".format(repr(self.description)),
+            ]),
+            suffix,
+        ))
 
     def copy(self):
         """Build an exact copy of this flag.
@@ -884,7 +903,7 @@ class DataQualityFlag(object):
         figure : `~matplotlib.figure.Figure`
             the newly created figure, with populated Axes.
 
-        See Also
+        See also
         --------
         matplotlib.pyplot.figure
             for documentation of keyword arguments used to create the
@@ -930,6 +949,8 @@ class DataQualityFlag(object):
             If the input ``name`` cannot be parsed into
             {ifo}:{tag}:{version} format.
         """
+        if isinstance(name, bytes):
+            name = name.decode('utf-8')
         if name is None:
             self.ifo = None
             self.tag = None
@@ -1070,7 +1091,7 @@ class DataQualityDict(OrderedDict):
             ``$DEFAULT_SEGMENT_SERVER`` environment variable, or
             ``'https://segments.ligo.org'``
 
-        See Also
+        See also
         --------
         DataQualityDict.query_segdb
         DataQualityDict.query_dqsegdb
@@ -1109,6 +1130,9 @@ class DataQualityDict(OrderedDict):
             An ordered `DataQualityDict` of (name, `DataQualityFlag`)
             pairs.
         """
+        warnings.warn("query_segdb is deprecated and will be removed in a "
+                      "future release", DeprecationWarning)
+
         # parse segments
         qsegs = _parse_query_segments(args, cls.query_segdb)
 
@@ -1136,7 +1160,7 @@ class DataQualityDict(OrderedDict):
                 vers = dqflag.version
             for gpsstart, gpsend in qsegs:
                 if float(gpsend) == +inf:
-                    gpsend = to_gps('now').seconds
+                    gpsend = int(to_gps('now'))
                 gpsstart = float(gpsstart)
                 if not gpsstart.is_integer():
                     raise ValueError("Segment database queries can only"
@@ -1260,8 +1284,9 @@ class DataQualityDict(OrderedDict):
         names : `list`, optional, default: read all names found
             list of names to read, by default all names are read separately.
 
-        coalesce : `bool`, optional, default: `True`
-            coalesce all `SegmentLists` before returning.
+        coalesce : `bool`, optional
+            if `True` coalesce the all segment lists before returning,
+            otherwise return exactly as contained in file(s).
 
         nproc : `int`, optional, default: 1
             number of CPUs to use for parallel reading of multiple files
@@ -1279,16 +1304,20 @@ class DataQualityDict(OrderedDict):
         Notes
         -----"""
         on_missing = kwargs.pop('on_missing', 'error')
+        coalesce = kwargs.pop('coalesce', False)
 
         if 'flags' in kwargs:  # pragma: no cover
             warnings.warn('\'flags\' keyword was renamed \'names\', this '
                           'warning will result in an error in the future')
             names = kwargs.pop('flags')
 
-        def _combine(inputs):
-            out = reduce(operator.or_, inputs)
-            missing = set(names or []) - set(out.keys())
-            for name in missing:  # validate all requested names are found
+        def combiner(inputs):
+            out = cls()
+
+            # check all names are contained
+            required = set(names or [])
+            found = set(name for dqdict in inputs for name in dqdict)
+            for name in required - found:  # validate all names are found once
                 msg = '{!r} not found in any input file'.format(name)
                 if on_missing == 'ignore':
                     continue
@@ -1296,10 +1325,21 @@ class DataQualityDict(OrderedDict):
                     warnings.warn(msg)
                 else:
                     raise ValueError(msg)
+
+            # combine flags
+            for dqdict in inputs:
+                for flag in dqdict:
+                    try:  # repeated occurence
+                        out[flag].known.extend(dqdict[flag].known)
+                        out[flag].active.extend(dqdict[flag].active)
+                    except KeyError:  # first occurence
+                        out[flag] = dqdict[flag]
+            if coalesce:
+                return out.coalesce()
             return out
 
-        return io_read_multi(_combine, cls, source, names=names, format=format,
-                             on_missing='ignore', **kwargs)
+        return io_read_multi(combiner, cls, source, names=names,
+                             format=format, on_missing='ignore', **kwargs)
 
     @classmethod
     def from_veto_definer_file(cls, fp, start=None, end=None, ifo=None,
@@ -1381,13 +1421,13 @@ class DataQualityDict(OrderedDict):
 
         Parameters
         ----------
-        segmentdeftable : :class:`~glue.ligolw.lsctables.SegmentDefTable`
+        segmentdeftable : :class:`~ligo.lw.lsctables.SegmentDefTable`
             the ``segment_definer`` table to read
 
-        segmentsumtable : :class:`~glue.ligolw.lsctables.SegmentSumTable`
+        segmentsumtable : :class:`~ligo.lw.lsctables.SegmentSumTable`
             the ``segment_summary`` table to read
 
-        segmenttable : :class:`~glue.ligolw.lsctables.SegmentTable`
+        segmenttable : :class:`~ligo.lw.lsctables.SegmentTable`
             the ``segment`` table to read
 
         names : `list` of `str`, optional
@@ -1418,7 +1458,7 @@ class DataQualityDict(OrderedDict):
 
         # read segment definers and generate DataQualityFlag object
         for row in segmentdeftable:
-            ifos = row.get_ifos()
+            ifos = sorted(row.instruments)
             ifo = ''.join(ifos) if ifos else None
             tag = row.name
             version = row.version
@@ -1426,10 +1466,11 @@ class DataQualityDict(OrderedDict):
                              k is not None])
             if names is None or name in names:
                 out[name] = DataQualityFlag(name)
+                thisid = int(row.segment_def_id)
                 try:
-                    id_[name].append(row.segment_def_id)
+                    id_[name].append(thisid)
                 except (AttributeError, KeyError):
-                    id_[name] = [row.segment_def_id]
+                    id_[name] = [thisid]
 
         # verify all requested flags were found
         for flag in names or []:
@@ -1441,52 +1482,71 @@ class DataQualityDict(OrderedDict):
                 else:
                     raise ValueError(msg)
 
+        # parse a table into the target DataQualityDict
+        def _parse_segments(table, listattr):
+            for row in table:
+                for flag in out:
+                    # match row ID to list of IDs found for this flag
+                    if int(row.segment_def_id) in id_[flag]:
+                        getattr(out[flag], listattr).append(
+                            Segment(*map(gpstype, row.segment)),
+                        )
+                        break
+
         # read segment summary table as 'known'
-        for row in segmentsumtable:
-            for flag in out:
-                # match row ID to list of IDs found for this flag
-                if row.segment_def_id in id_[flag]:
-                    out[flag].known.append(
-                        Segment(*map(gpstype, row.segment)))
-                    break
+        _parse_segments(segmentsumtable, "known")
 
         # read segment table as 'active'
-        for row in segmenttable:
-            for flag in out:
-                if row.segment_def_id in id_[flag]:
-                    out[flag].active.append(
-                        Segment(*map(gpstype, row.segment)))
-                    break
+        _parse_segments(segmenttable, "active")
 
         return out
 
-    def to_ligolw_tables(self, **attrs):
+    def to_ligolw_tables(self, ilwdchar_compat=None, **attrs):
         """Convert this `DataQualityDict` into a trio of LIGO_LW segment tables
 
         Parameters
         ----------
+        ilwdchar_compat : `bool`, optional
+            whether to write in the old format, compatible with
+            ILWD characters (`True`), or to use the new format (`False`);
+            the current default is `True` to maintain backwards
+            compatibility, but this will change for gwpy-1.0.0.
+
         **attrs
             other attributes to add to all rows in all tables
             (e.g. ``'process_id'``)
 
         Returns
         -------
-        segmentdeftable : :class:`~glue.ligolw.lsctables.SegmentDefTable`
+        segmentdeftable : :class:`~ligo.lw.lsctables.SegmentDefTable`
             the ``segment_definer`` table
 
-        segmentsumtable : :class:`~glue.ligolw.lsctables.SegmentSumTable`
+        segmentsumtable : :class:`~ligo.lw.lsctables.SegmentSumTable`
             the ``segment_summary`` table
 
-        segmenttable : :class:`~glue.ligolw.lsctables.SegmentTable`
+        segmenttable : :class:`~ligo.lw.lsctables.SegmentTable`
             the ``segment`` table
         """
-        from glue.ligolw.lsctables import (SegmentTable, SegmentSumTable,
-                                           SegmentDefTable, New as new_table)
+        if ilwdchar_compat is None:
+            warnings.warn("ilwdchar_compat currently defaults to `True`, "
+                          "but this will change to `False` in the future, to "
+                          "maintain compatibility in future releases, "
+                          "manually specify `ilwdchar_compat=True`",
+                          PendingDeprecationWarning)
+            ilwdchar_compat = True
+
+        if ilwdchar_compat:
+            from glue.ligolw import lsctables
+        else:
+            from ligo.lw import lsctables
         from ..io.ligolw import to_table_type as to_ligolw_table_type
 
-        segdeftab = new_table(SegmentDefTable)
-        segsumtab = new_table(SegmentSumTable)
-        segtab = new_table(SegmentTable)
+        SegmentDefTable = lsctables.SegmentDefTable
+        SegmentSumTable = lsctables.SegmentSumTable
+        SegmentTable = lsctables.SegmentTable
+        segdeftab = lsctables.New(SegmentDefTable)
+        segsumtab = lsctables.New(SegmentSumTable)
+        segtab = lsctables.New(SegmentTable)
 
         def _write_attrs(table, row):
             for key, val in attrs.items():
@@ -1498,7 +1558,7 @@ class DataQualityDict(OrderedDict):
             segdef = segdeftab.RowType()
             for col in segdeftab.columnnames:  # default all columns to None
                 setattr(segdef, col, None)
-            segdef.set_ifos([flag.ifo])
+            segdef.instruments = {flag.ifo}
             segdef.name = flag.tag
             segdef.version = flag.version
             segdef.comment = flag.description
@@ -1513,7 +1573,7 @@ class DataQualityDict(OrderedDict):
                 for col in segsumtab.columnnames:  # default columns to None
                     setattr(segsum, col, None)
                 segsum.segment_def_id = segdef.segment_def_id
-                segsum.set(map(LIGOTimeGPS, vseg))
+                segsum.segment = map(LIGOTimeGPS, vseg)
                 segsum.comment = None
                 segsum.segment_sum_id = SegmentSumTable.get_next_id()
                 _write_attrs(segsumtab, segsum)
@@ -1525,7 +1585,7 @@ class DataQualityDict(OrderedDict):
                 for col in segtab.columnnames:  # default all columns to None
                     setattr(seg, col, None)
                 seg.segment_def_id = segdef.segment_def_id
-                seg.set(map(LIGOTimeGPS, aseg))
+                seg.segment = map(LIGOTimeGPS, aseg)
                 seg.segment_id = SegmentTable.get_next_id()
                 _write_attrs(segtab, seg)
                 segtab.append(seg)
@@ -1541,6 +1601,20 @@ class DataQualityDict(OrderedDict):
         -----"""
         return io_registry.write(self, target, *args, **kwargs)
 
+    def coalesce(self):
+        """Coalesce all segments lists in this `DataQualityDict`.
+
+        **This method modifies this object in-place.**
+
+        Returns
+        -------
+        self
+            a view of this flag, not a copy.
+        """
+        for flag in self:
+            self[flag].coalesce()
+        return self
+
     def populate(self, source=DEFAULT_SEGMENT_SERVER,
                  segments=None, pad=True, on_error='raise', **kwargs):
         """Query the segment database for each flag's active segments.
@@ -1640,7 +1714,7 @@ class DataQualityDict(OrderedDict):
         """
         if deep:
             return deepcopy(self)
-        return super(DataQualityDict, self).copy()
+        return super().copy()
 
     def __iand__(self, other):
         for key, value in other.items():
@@ -1651,8 +1725,10 @@ class DataQualityDict(OrderedDict):
         return self
 
     def __and__(self, other):
-        if (sum(len(s.active) for s in self.values()) <=
-                sum(len(s.active) for s in other.values())):
+        if (
+            sum(len(s.active) for s in self.values())
+            <= sum(len(s.active) for s in other.values())
+        ):
             return self.copy(deep=True).__iand__(other)
         return other.copy(deep=True).__iand__(self)
 
@@ -1665,8 +1741,10 @@ class DataQualityDict(OrderedDict):
         return self
 
     def __or__(self, other):
-        if (sum(len(s.active) for s in self.values()) >=
-                sum(len(s.active) for s in other.values())):
+        if (
+            sum(len(s.active) for s in self.values())
+            >= sum(len(s.active) for s in other.values())
+        ):
             return self.copy(deep=True).__ior__(other)
         return other.copy(deep=True).__ior__(self)
 
@@ -1746,7 +1824,7 @@ class DataQualityDict(OrderedDict):
         figure : `~matplotlib.figure.Figure`
             the newly created figure, with populated Axes.
 
-        See Also
+        See also
         --------
         matplotlib.pyplot.figure
             for documentation of keyword arguments used to create the
