@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -26,6 +26,9 @@ from astropy.io import registry as io_registry
 
 from ligo.segments import (segment, segmentlist, segmentlistdict)
 
+from ..io.mp import read_multi as io_read_multi
+from ..utils.decorators import return_as
+
 __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 __credits__ = "Kipp Cannon <kipp.cannon@ligo.org>"
 __all__ = ['Segment', 'SegmentList', 'SegmentListDict']
@@ -122,23 +125,46 @@ class SegmentList(segmentlist):
      Segment(30, infinity)]
     """
 
+    # -- representations ------------------------
+
     def __repr__(self):
         return "<SegmentList([%s])>" % "\n              ".join(map(repr, self))
 
+    def __str__(self):
+        return "[%s]" % "\n ".join(map(str, self))
+
+    # -- type casting ---------------------------
+
+    extent = return_as(Segment)(segmentlist.extent)
+
     def coalesce(self):
-        self = super(SegmentList, self).coalesce()
+        super().coalesce()
         for i, seg in enumerate(self):
             self[i] = Segment(seg[0], seg[1])
         return self
     coalesce.__doc__ = segmentlist.coalesce.__doc__
 
-    def __str__(self):
-        return "[%s]" % "\n ".join(map(str, self))
+    def to_table(self):
+        """Convert this `SegmentList` to a `~astropy.table.Table`
+
+        The resulting `Table` has four columns: `index`, `start`, `end`, and
+        `duration`, corresponding to the zero-counted list index, GPS start
+        and end times, and total duration in seconds, respectively.
+
+        This method exists mainly to provide a way to write `SegmentList`
+        objects in comma-separated value (CSV) format, via the
+        :meth:`~astropy.table.Table.write` method.
+        """
+        from astropy.table import Table
+        return Table(
+            rows=[(i, s[0], s[1], abs(s)) for i, s in enumerate(self)],
+            names=('index', 'start', 'end', 'duration'),
+        )
 
     # -- i/o ------------------------------------
 
     @classmethod
-    def read(cls, source, format=None, **kwargs):
+    def read(cls, source, format=None, coalesce=False, **kwargs):
         # pylint: disable=redefined-builtin
         """Read segments from file into a `SegmentList`
 
@@ -152,14 +178,35 @@ class SegmentList(segmentlist):
             detected if possible. See below for list of acceptable
             formats.
 
+        coalesce : `bool`, optional
+            if `True` coalesce the segment list before returning,
+            otherwise return exactly as contained in file(s).
+
+        **kwargs
+            other keyword arguments depend on the format, see the online
+            documentation for details (:ref:`gwpy-segments-io`)
+
         Returns
         -------
         segmentlist : `SegmentList`
             `SegmentList` active and known segments read from file.
 
+        Raises
+        ------
+        IndexError
+            if ``source`` is an empty list
+
         Notes
         -----"""
-        return io_registry.read(cls, source, format=format, **kwargs)
+        def combiner(listofseglists):
+            """Combine `SegmentList` from each file into a single object
+            """
+            out = cls(seg for seglist in listofseglists for seg in seglist)
+            if coalesce:
+                return out.coalesce()
+            return out
+
+        return io_read_multi(combiner, cls, source, format=format, **kwargs)
 
     def write(self, target, *args, **kwargs):
         """Write this `SegmentList` to a file
@@ -219,5 +266,6 @@ class SegmentListDict(segmentlistdict):
     """
     pass
 
+
 # clean up the namespace
 del segment, segmentlist, segmentlistdict
