@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,23 +19,20 @@
 """Custom filtering utilities for the `TimeSeries`
 """
 
-from __future__ import division
 import operator
-from functools import wraps
+from functools import reduce
 from math import (pi, log10)
 
-from six.moves import reduce
-
 import numpy
+from numpy import fft as npfft
 
 from scipy import signal
-try:
-    from scipy.signal.ltisys import LinearTimeInvariant
-except ImportError:  # scipy < 0.18
-    from scipy.signal import lti as LinearTimeInvariant
+from scipy.signal.ltisys import LinearTimeInvariant
 
 from astropy.units import (Unit, Quantity)
 
+from .window import planck
+
 __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 __all__ = ['lowpass', 'highpass', 'bandpass', 'notch', 'concatenate_zpks']
 
@@ -126,8 +123,12 @@ def num_taps(sample_rate, transitionwidth, gpass, gstop):
     """
     gpass = 10 ** (-gpass / 10.)
     gstop = 10 ** (-gstop / 10.)
-    return int(2/3. * log10(1 / (10 * gpass * gstop)) *
-               sample_rate / transitionwidth)
+    return int(
+        2/3.
+        * log10(1 / (10 * gpass * gstop))
+        * sample_rate
+        / transitionwidth
+    )
 
 
 def is_zpk(zpktup):
@@ -140,11 +141,120 @@ def is_zpk(zpktup):
         otherwise `False`
     """
     return (
-        isinstance(zpktup, (tuple, list)) and
-        len(zpktup) == 3 and
-        isinstance(zpktup[0], (list, tuple, numpy.ndarray)) and
-        isinstance(zpktup[1], (list, tuple, numpy.ndarray)) and
-        isinstance(zpktup[2], float))
+        isinstance(zpktup, (tuple, list))
+        and len(zpktup) == 3
+        and isinstance(zpktup[0], (list, tuple, numpy.ndarray))
+        and isinstance(zpktup[1], (list, tuple, numpy.ndarray))
+        and isinstance(zpktup[2], float)
+    )
+
+
+def truncate_transfer(transfer, ncorner=None):
+    """Smoothly zero the edges of a frequency domain transfer function
+
+    Parameters
+    ----------
+    transfer : `numpy.ndarray`
+        transfer function to start from, must have at least ten samples
+
+    ncorner : `int`, optional
+        number of extra samples to zero off at low frequency, default: `None`
+
+    Returns
+    -------
+    out : `numpy.ndarray`
+        the smoothly truncated transfer function
+
+    Notes
+    -----
+    By default, the input transfer function will have five samples tapered
+    off at the left and right boundaries. If `ncorner` is not `None`, then
+    `ncorner` extra samples will be zeroed on the left as a hard highpass
+    filter.
+
+    See :func:`~gwpy.signal.window.planck` for more information.
+    """
+    nsamp = transfer.size
+    ncorner = ncorner if ncorner else 0
+    out = transfer.copy()
+    out[0:ncorner] = 0
+    out[ncorner:nsamp] *= planck(nsamp-ncorner, nleft=5, nright=5)
+    return out
+
+
+def truncate_impulse(impulse, ntaps, window='hanning'):
+    """Smoothly truncate a time domain impulse response
+
+    Parameters
+    ----------
+    impulse : `numpy.ndarray`
+        the impulse response to start from
+
+    ntaps : `int`
+        number of taps in the final filter
+
+    window : `str`, `numpy.ndarray`, optional
+        window function to truncate with, default: ``'hanning'``
+        see :func:`scipy.signal.get_window` for details on acceptable formats
+
+    Returns
+    -------
+    out : `numpy.ndarray`
+        the smoothly truncated impulse response
+    """
+    out = impulse.copy()
+    trunc_start = int(ntaps / 2)
+    trunc_stop = out.size - trunc_start
+    window = signal.get_window(window, ntaps)
+    out[0:trunc_start] *= window[trunc_start:ntaps]
+    out[trunc_stop:out.size] *= window[0:trunc_start]
+    out[trunc_start:trunc_stop] = 0
+    return out
+
+
+def fir_from_transfer(transfer, ntaps, window='hanning', ncorner=None):
+    """Design a Type II FIR filter given an arbitrary transfer function
+
+    Parameters
+    ----------
+    transfer : `numpy.ndarray`
+        transfer function to start from, must have at least ten samples
+
+    ntaps : `int`
+        number of taps in the final filter, must be an even number
+
+    window : `str`, `numpy.ndarray`, optional
+        window function to truncate with, default: ``'hanning'``
+        see :func:`scipy.signal.get_window` for details on acceptable formats
+
+    ncorner : `int`, optional
+        number of extra samples to zero off at low frequency, default: `None`
+
+    Returns
+    -------
+    out : `numpy.ndarray`
+        A time domain FIR filter of length `ntaps`
+
+    Notes
+    -----
+    The final FIR filter will use `~numpy.fft.rfft` FFT normalisation.
+
+    If `ncorner` is not `None`, then `ncorner` extra samples will be zeroed
+    on the left as a hard highpass filter.
+
+    See also
+    --------
+    scipy.signal.remez
+        an alternative FIR filter design using the Remez exchange algorithm
+    """
+    # truncate and highpass the transfer function
+    transfer = truncate_transfer(transfer, ncorner=ncorner)
+    # compute and truncate the impulse response
+    impulse = npfft.irfft(transfer)
+    impulse = truncate_impulse(impulse, ntaps=ntaps, window=window)
+    # wrap around and normalise to construct the filter
+    out = numpy.roll(impulse, int(ntaps/2 - 1))[0:ntaps]
+    return out
 
 
 def bilinear_zpk(zeros, poles, gain, fs=1.0, unit='Hz'):
@@ -240,17 +350,17 @@ def parse_filter(args, analog=False, sample_rate=None):
     # parse IIR filter
     if isinstance(args, LinearTimeInvariant):
         lti = args
-    elif (isinstance(args, numpy.ndarray) and
-            args.ndim == 2 and args.shape[1] == 6):
+    elif (
+        isinstance(args, numpy.ndarray)
+        and args.ndim == 2
+        and args.shape[1] == 6
+    ):
         lti = signal.lti(*signal.sos2zpk(args))
     else:
         lti = signal.lti(*args)
 
     # convert to zpk format
-    try:
-        lti = lti.to_zpk()
-    except AttributeError:  # scipy < 0.18, doesn't matter
-        pass
+    lti = lti.to_zpk()
 
     # convert to digital components
     if analog:
@@ -308,9 +418,9 @@ def lowpass(frequency, sample_rate, fstop=None, gpass=2, gstop=30, type='iir',
     >>> from gwpy.signal.filter_design import lowpass
     >>> lp = lowpass(1000, 4096)
 
-    To view the filter, you can use the `~gwpy.plotter.BodePlot`:
+    To view the filter, you can use the `~gwpy.plot.BodePlot`:
 
-    >>> from gwpy.plotter import BodePlot
+    >>> from gwpy.plot import BodePlot
     >>> plot = BodePlot(lp, sample_rate=4096)
     >>> plot.show()
     """
@@ -370,9 +480,9 @@ def highpass(frequency, sample_rate, fstop=None, gpass=2, gstop=30, type='iir',
     >>> from gwpy.signal.filter_design import highpass
     >>> hp = highpass(100, 4096)
 
-    To view the filter, you can use the `~gwpy.plotter.BodePlot`:
+    To view the filter, you can use the `~gwpy.plot.BodePlot`:
 
-    >>> from gwpy.plotter import BodePlot
+    >>> from gwpy.plot import BodePlot
     >>> plot = BodePlot(hp, sample_rate=4096)
     >>> plot.show()
     """
@@ -436,9 +546,9 @@ def bandpass(flow, fhigh, sample_rate, fstop=None, gpass=2, gstop=30,
     >>> from gwpy.signal.filter_design import bandpass
     >>> bp = bandpass(100, 1000, 4096)
 
-    To view the filter, you can use the `~gwpy.plotter.BodePlot`:
+    To view the filter, you can use the `~gwpy.plot.BodePlot`:
 
-    >>> from gwpy.plotter import BodePlot
+    >>> from gwpy.plot import BodePlot
     >>> plot = BodePlot(bp, sample_rate=4096)
     >>> plot.show()
     """
@@ -476,7 +586,7 @@ def notch(frequency, sample_rate, type='iir', **kwargs):
     zpk : `tuple` of `complex` or `float`
        the filter components in digital zero-pole-gain format
 
-    See Also
+    See also
     --------
     scipy.signal.iirdesign
         for details on the IIR filter design method
@@ -493,9 +603,9 @@ def notch(frequency, sample_rate, type='iir', **kwargs):
     >>> from gwpy.signal.filter_design import notch
     >>> n = notch(100, 4096)
 
-    To view the filter, you can use the `~gwpy.plotter.BodePlot`:
+    To view the filter, you can use the `~gwpy.plot.BodePlot`:
 
-    >>> from gwpy.plotter import BodePlot
+    >>> from gwpy.plot import BodePlot
     >>> plot = BodePlot(n, sample_rate=4096)
     >>> plot.show()
     """
@@ -549,7 +659,7 @@ def concatenate_zpks(*zpks):
 
     Plot the filter:
 
-    >>> from gwpy.plotter import BodePlot
+    >>> from gwpy.plot import BodePlot
     >>> plot = BodePlot(zpk, sample_rate=4096)
     >>> plot.show()
     """
