@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,7 +19,7 @@
 """Create, manipulate, read, and write spectrogram data
 """
 
-from .core import (Spectrogram, SpectrogramList)
+from .spectrogram import (Spectrogram, SpectrogramList)
 
 from . import io  # register I/O
 
