@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -20,14 +20,12 @@
 time-frequency coherence spectrogram from a pair of time-series.
 """
 
-from __future__ import division
-
 from multiprocessing import (Process, Queue as ProcessQueue)
 from math import ceil
 
 from numpy import zeros
 
-from .core import (Spectrogram, SpectrogramList)
+from .spectrogram import (Spectrogram, SpectrogramList)
 
 __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 
@@ -35,7 +33,7 @@ __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 def _from_timeseries(ts1, ts2, stride, fftlength=None, overlap=None,
                      window=None, **kwargs):
     """Generate a time-frequency coherence
-    :class:`~gwpy.spectrogram.core.Spectrogram` from a pair of
+    :class:`~gwpy.spectrogram.Spectrogram` from a pair of
     :class:`~gwpy.timeseries.TimeSeries`.
 
     For each `stride`, a PSD :class:`~gwpy.frequencyseries.FrequencySeries`
@@ -108,7 +106,7 @@ def from_timeseries(ts1, ts2, stride, fftlength=None, overlap=None,
 
     Returns
     -------
-    spectrogram : :class:`~gwpy.spectrogram.core.Spectrogram`
+    spectrogram : :class:`~gwpy.spectrogram.Spectrogram`
         time-frequency power spectrogram as generated from the
         input time-series.
     """
