@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -21,13 +21,8 @@
 
 import warnings
 
-from six import string_types
-
 import numpy
 
-import scipy
-from scipy import signal
-
 from astropy import units
 from astropy.io import registry as io_registry
 
@@ -162,8 +157,8 @@ class Spectrogram(Array2D):
             kwargs['yindex'] = frequencies
 
         # generate Spectrogram
-        return super(Spectrogram, cls).__new__(cls, data, unit=unit, name=name,
-                                               channel=channel, **kwargs)
+        return super().__new__(cls, data, unit=unit, name=name,
+                               channel=channel, **kwargs)
 
     # -- Spectrogram properties -----------------
 
@@ -310,7 +305,7 @@ class Spectrogram(Array2D):
         ValueError
             if ``operand`` is given as a `str` that isn't supported
         """
-        if isinstance(operand, string_types):
+        if isinstance(operand, str):
             if operand == 'mean':
                 operand = self.mean(axis=0)
             elif operand == 'median':
@@ -336,7 +331,7 @@ class Spectrogram(Array2D):
         plot : `~gwpy.plot.Plot`
             the `Plot` containing the data
 
-        See Also
+        See also
         --------
         matplotlib.pyplot.figure
             for documentation of keyword arguments used to create the
@@ -344,7 +339,8 @@ class Spectrogram(Array2D):
         matplotlib.figure.Figure.add_subplot
             for documentation of keyword arguments used to create the
             axes
-        gwpy.plot.Axes.imshow or gwpy.plot.Axes.pcolormesh
+        gwpy.plot.Axes.imshow
+        gwpy.plot.Axes.pcolormesh
             for documentation of keyword arguments used in rendering the
             `Spectrogram` data
         """
@@ -355,7 +351,7 @@ class Spectrogram(Array2D):
             kwargs.setdefault('method', 'imshow' if kwargs.pop('imshow') else
                                         'pcolormesh')
         kwargs.update(figsize=figsize, xscale=xscale)
-        return super(Spectrogram, self).plot(**kwargs)
+        return super().plot(**kwargs)
 
     @classmethod
     def from_spectra(cls, *spectra, **kwargs):
@@ -414,15 +410,15 @@ class Spectrogram(Array2D):
             the given percentile `FrequencySeries` calculated from this
             `SpectralVaraicence`
         """
-        out = scipy.percentile(self.value, percentile, axis=0)
+        out = numpy.percentile(self.value, percentile, axis=0)
         if self.name is not None:
             name = '{}: {} percentile'.format(self.name, _ordinal(percentile))
         else:
             name = None
-        return FrequencySeries(out, epoch=self.epoch, channel=self.channel,
-                               name=name, f0=self.f0, df=self.df,
-                               frequencies=(hasattr(self, '_frequencies') and
-                                            self.frequencies or None))
+        return FrequencySeries(
+            out, epoch=self.epoch, channel=self.channel, name=name,
+            f0=self.f0, df=self.df, unit=self.unit, frequencies=(
+                hasattr(self, '_frequencies') and self.frequencies or None))
 
     def zpk(self, zeros, poles, gain, analog=True):
         """Filter this `Spectrogram` by applying a zero-pole-gain filter
@@ -447,7 +443,7 @@ class Spectrogram(Array2D):
         specgram : `Spectrogram`
             the frequency-domain filtered version of the input data
 
-        See Also
+        See also
         --------
         Spectrogram.filter
             for details on how a digital ZPK-format filter is applied
@@ -527,9 +523,9 @@ class Spectrogram(Array2D):
         specvar : `SpectralVariance`
             2D-array of spectral frequency-amplitude counts
 
-        See Also
+        See also
         --------
-        :func:`numpy.histogram`
+        numpy.histogram
             for details on specifying bins and weights
         """
         from ..frequencyseries import SpectralVariance
@@ -594,8 +590,7 @@ class Spectrogram(Array2D):
     # -- Spectrogram ufuncs ---------------------
 
     def _wrap_function(self, function, *args, **kwargs):
-        out = super(Spectrogram, self)._wrap_function(
-            function, *args, **kwargs)
+        out = super()._wrap_function(function, *args, **kwargs)
         # requested frequency axis, return a FrequencySeries
         if out.ndim == 1 and out.x0.unit == self.y0.unit:
             return FrequencySeries(out.value, name=out.name, unit=out.unit,
@@ -612,7 +607,7 @@ class Spectrogram(Array2D):
     # -- other ----------------------------------
 
     def __getitem__(self, item):
-        out = super(Spectrogram, self).__getitem__(item)
+        out = super().__getitem__(item)
 
         # set epoch manually, because Spectrogram doesn't store self._epoch
         if isinstance(out, self._columnclass):
