@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,8 +19,15 @@
 """Input/output methods for tabular data.
 """
 
-from . import (  # pylint: disable=unused-import
-    ligolw,  # glue.ligolw XML format
+# utils.py provides some decorators, but importantly applies those
+# decorators _automatically_ to the existing registered readers
+# provided by astropy, so this needs to come first.
+from . import utils
+
+# other readers are defined in their own modules, and are responsible
+# for applying the decorators themselves.
+from . import (
+    ligolw,  # ligo.lw XML format
     root,  # generic ROOT stuff
     omicron,  # Omicron ROOT format
     omega,  # Omega ASCII format
@@ -29,6 +36,7 @@ from . import (  # pylint: disable=unused-import
     hacr,  # Hierarchichal Algorithm for Curves and Ridges
     gwf,  # GWF FrEvents (e.g. MBTA)
     gravityspy,  # Gravity Spy Triggers
+    snax,  # SNAX HDF5 features
 )
 
 __author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
