@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2014)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -20,17 +20,15 @@
 """
 
 import os.path
+import re
 import shutil
 import tempfile
 from io import BytesIO
-
-from six import PY2
+from unittest import mock
 
 import pytest
 
-import sqlparse
-
-from numpy import (random, isclose, dtype)
+from numpy import (random, isclose, dtype, asarray, all)
 
 import h5py
 
@@ -41,8 +39,8 @@ from astropy.table import vstack
 from ...frequencyseries import FrequencySeries
 from ...io import ligolw as io_ligolw
 from ...segments import (Segment, SegmentList)
-from ...tests import utils
-from ...tests.mocks import mock
+from ...testing import utils
+from ...testing.errors import pytest_skip_network_error
 from ...time import LIGOTimeGPS
 from ...timeseries import (TimeSeries, TimeSeriesDict)
 from .. import (Table, EventTable, filters)
@@ -66,20 +64,23 @@ def mock_hacr_connection(table, start, stop):
     cursor = mock.MagicMock()
 
     def execute(qstr):
-        cursor._query = sqlparse.parse(qstr)[0]
+        cursor._query = qstr
         return len(table)
 
     cursor.execute = execute
+    column_regex = re.compile(r"\Aselect (.*) from ", re.I)
+    select_regex = re.compile(r"where (.*) (order by .*)?\Z", re.I)
 
     def fetchall():
-        if cursor._query.get_real_name() == 'job':
+        q = cursor._query
+        if "from job" in q:
             return [(1, start, stop)]
-        if cursor._query.get_real_name() == 'mhacr':
-            columns = list(map(
-                str, list(cursor._query.get_sublists())[0].get_identifiers()))
-            selections = list(map(
-                str, list(cursor._query.get_sublists())[2].get_sublists()))
-            return filter_table(table, selections[3:])[columns]
+        if "from mhacr" in q:
+            columns = column_regex.match(q).groups()[0].split(", ")
+            selections = (
+                select_regex.search(q).groups()[0].strip().split(" and ")
+            )
+            return map(tuple, filter_table(table, selections[3:])[columns])
 
     cursor.fetchall = fetchall
 
@@ -115,9 +116,16 @@ class TestTable(object):
     def table(cls):
         return cls.create(100, ['time', 'snr', 'frequency'])
 
+    @classmethod
+    @pytest.fixture()
+    def clustertable(cls):
+        return cls.TABLE(data=[[11, 1, 1, 10, 1, 1, 9],
+                               [0.0, 1.9, 1.95, 2.0, 2.05, 2.1, 4.0]],
+                         names=['amplitude', 'time'])
+
     # -- test I/O -------------------------------
 
-    @utils.skip_missing_dependency('glue.ligolw.lsctables')
+    @utils.skip_missing_dependency('ligo.lw.lsctables')
     @pytest.mark.parametrize('ext', ['xml', 'xml.gz'])
     def test_read_write_ligolw(self, ext):
         table = self.create(
@@ -144,7 +152,7 @@ class TestTable(object):
             assert t2.meta.get('tablename', None) == 'sngl_burst'
 
             # check numpy type casting works
-            from glue.ligolw.lsctables import LIGOTimeGPS as LigolwGPS
+            from ligo.lw.lsctables import LIGOTimeGPS as LigolwGPS
             t3 = _read(columns=['peak'])
             assert isinstance(t3['peak'][0], LigolwGPS)
             t3 = _read(columns=['peak'], use_numpy_dtypes=True)
@@ -153,13 +161,8 @@ class TestTable(object):
                 t3['peak'], table['peak_time'] + table['peak_time_ns'] * 1e-9)
 
             # check reading multiple tables works
-            try:
-                t3 = self.TABLE.read([tmp, tmp], format='ligolw',
-                                     tablename='sngl_burst')
-            except NameError as e:
-                if not PY2:  # ligolw not patched for python3 just yet
-                    pytest.xfail(str(e))
-                raise
+            t3 = self.TABLE.read([tmp, tmp], format='ligolw',
+                                 tablename='sngl_burst')
             utils.assert_table_equal(vstack((t2, t2)), t3)
 
             # check writing to existing file raises IOError
@@ -168,14 +171,7 @@ class TestTable(object):
             assert str(exc.value) == 'File exists: %s' % tmp
 
             # check overwrite=True, append=False rewrites table
-            try:
-                _write(overwrite=True)
-            except TypeError as e:
-                # ligolw is not python3-compatbile, so skip if it fails
-                if not PY2 and (
-                        str(e) == 'write() argument must be str, not bytes'):
-                    pytest.xfail(str(e))
-                raise
+            _write(overwrite=True)
             t3 = _read()
             utils.assert_table_equal(t2, t3)
 
@@ -205,7 +201,53 @@ class TestTable(object):
             assert str(exc.value) == ('document must contain exactly '
                                       'one sngl_burst table')
 
-    @utils.skip_missing_dependency('glue.ligolw.lsctables')
+    @utils.skip_missing_dependency('ligo.lw.lsctables')
+    def test_read_write_ligolw_ilwdchar_compat(self):
+        from glue.ligolw.ilwd import get_ilwdchar_class
+        from glue.ligolw.lsctables import SnglBurstTable
+
+        eid_type = get_ilwdchar_class("sngl_burst", "event_id")
+
+        table = self.create(
+            100,
+            ["peak", "snr", "central_freq", "event_id"],
+            ["f8", "f4", "f4", "i8"],
+        )
+        with tempfile.NamedTemporaryFile(suffix=".xml") as tmp:
+            # write table with ilwdchar_compat=True
+            table.write(tmp, format="ligolw", tablename="sngl_burst",
+                        ilwdchar_compat=True)
+
+            # read raw ligolw and check type is correct
+            llw = io_ligolw.read_table(tmp, tablename="sngl_burst",
+                                       ilwdchar_compat=True)
+            assert type(llw.getColumnByName("event_id")[0]) is eid_type
+
+            # reset IDs to 0
+            SnglBurstTable.reset_next_id()
+
+            # read without explicit use of ilwdchar_compat
+            t2 = self.TABLE.read(tmp, columns=table.colnames)
+            assert type(t2[0]["event_id"]) is eid_type
+
+            # read again with explicit use of ilwdchar_compat
+            SnglBurstTable.reset_next_id()
+            utils.assert_table_equal(
+                self.TABLE.read(tmp, columns=table.colnames,
+                                ilwdchar_compat=True),
+                t2,
+            )
+
+            # and check that ilwdchar_compat=True, use_numpy_dtypes=True works
+            SnglBurstTable.reset_next_id()
+            utils.assert_table_equal(
+                self.TABLE.read(tmp, columns=table.colnames,
+                                ilwdchar_compat=True, use_numpy_dtypes=True),
+                table,
+                almost_equal=True,
+            )
+
+    @utils.skip_missing_dependency('ligo.lw.lsctables')
     def test_read_write_ligolw_property_columns(self):
         table = self.create(100, ['peak', 'snr', 'central_freq'],
                             ['f8', 'f4', 'f4'])
@@ -218,13 +260,42 @@ class TestTable(object):
             for col in ('peak_time', 'peak_time_ns'):
                 assert col in llw.columnnames
             with io_ligolw.patch_ligotimegps():
-                utils.assert_array_equal(llw.get_peak(), table['peak'])
+                utils.assert_array_equal(
+                    asarray([row.peak for row in llw]),
+                    table['peak'],
+                )
 
             # read table and assert gpsproperty was repacked properly
             t2 = self.TABLE.read(f, columns=table.colnames,
                                  use_numpy_dtypes=True)
             utils.assert_table_equal(t2, table, almost_equal=True)
 
+    @utils.skip_missing_dependency('ligo.lw.lsctables')
+    @pytest.mark.parametrize('ilwdchar_compat', (False, True))
+    def test_read_ligolw_get_as_exclude(self, ilwdchar_compat):
+        table = self.TABLE(
+            rows=[
+                ("H1", 0.0, 4, 0),
+                ("L1", 0.62831, 4, 0),
+                ("V1", 0.31415, 4, 0),
+            ],
+            names=("instrument", "offset", "process_id", "time_slide_id"),
+        )
+        with tempfile.NamedTemporaryFile(suffix=".xml") as f:
+            table.write(
+                f,
+                format="ligolw",
+                tablename="time_slide",
+                ilwdchar_compat=ilwdchar_compat,
+            )
+            t2 = table.read(
+                f,
+                tablename="time_slide",
+                use_numpy_dtypes=ilwdchar_compat,
+            )
+            t2.sort("instrument")
+            utils.assert_table_equal(t2, table)
+
     @utils.skip_missing_dependency('root_numpy')
     def test_read_write_root(self, table):
         with utils.TemporaryFilename(suffix='.root') as tmp:
@@ -256,6 +327,7 @@ class TestTable(object):
                                  ('time', filters.in_segmentlist, segs)),
             )
 
+    @utils.skip_missing_dependency('LDAStools.frameCPP')
     def test_read_write_gwf(self):
         table = self.create(100, ['time', 'blah', 'frequency'])
         columns = table.dtype.names
@@ -263,8 +335,12 @@ class TestTable(object):
             # check write
             try:
                 table.write(tmp, 'test_read_write_gwf')
-            except ImportError as e:
-                pytest.skip(str(e))
+            except TypeError as exc:  # pragma: no-cover
+                if 'ParamList' in str(exc):  # frameCPP broken (2.6.7)
+                    pytest.skip(
+                        "bug in python-ldas-tools-framecpp: {!s}".format(exc),
+                    )
+                raise
 
             # check read gives back same table
             t2 = self.TABLE.read(tmp, 'test_read_write_gwf', columns=columns)
@@ -293,13 +369,9 @@ class TestEventTable(TestTable):
         assert t._get_time_column() == 'blah2'
 
         # check that two GPS columns causes issues
-        try:
-            t.add_column(t['blah2'], name='blah3')
-        except TypeError:  # astropy < 2.0 (or something like that)
-            pass
-        else:
-            with pytest.raises(ValueError):
-                t._get_time_column()
+        t.add_column(t['blah2'], name='blah3')
+        with pytest.raises(ValueError):
+            t._get_time_column()
 
     def test_filter(self, table):
         # check simple filter
@@ -314,8 +386,10 @@ class TestEventTable(TestTable):
         # check compounding works
         loud = table.filter('snr > 100')
         lowfloud = table.filter('frequency < 100', 'snr > 100')
-        brute = type(table)(rows=[row for row in lowf if row in loud],
-                            names=table.dtype.names)
+        brute = type(table)(
+            rows=[tuple(row) for row in lowf if row in loud],
+            names=table.dtype.names,
+        )
         utils.assert_table_equal(brute, lowfloud)
 
         # check double-ended filter
@@ -323,15 +397,14 @@ class TestEventTable(TestTable):
         utils.assert_table_equal(
             midf, table.filter('frequency > 100').filter('frequency < 1000'))
 
-        # check unicode parsing (PY2)
-        table.filter(u'snr > 100')
-
     def test_filter_in_segmentlist(self, table):
         # check filtering on segments works
         segs = SegmentList([Segment(100, 200), Segment(400, 500)])
         inseg = table.filter(('time', filters.in_segmentlist, segs))
-        brute = type(table)(rows=[row for row in table if row['time'] in segs],
-                            names=table.colnames)
+        brute = type(table)(
+            rows=[tuple(row) for row in table if row['time'] in segs],
+            names=table.colnames,
+        )
         utils.assert_table_equal(inseg, brute)
 
         # check empty segmentlist is handled well
@@ -348,22 +421,29 @@ class TestEventTable(TestTable):
             table.filter(('time', filters.not_in_segmentlist, SegmentList())))
 
     def test_event_rates(self, table):
+        """Test :meth:`gwpy.table.EventTable.event_rate`
+        """
         rate = table.event_rate(1)
         assert isinstance(rate, TimeSeries)
         assert rate.sample_rate == 1 * units.Hz
 
-        # repeat with object dtype
-        try:
-            from lal import LIGOTimeGPS as LalGps
-        except ImportError:
-            return
+    @utils.skip_missing_dependency('lal')
+    def test_event_rates_gpsobject(self, table):
+        """Test that `EventTable.event_rate` can handle object dtypes
+        """
+        rate = table.event_rate(1)
+
+        from lal import LIGOTimeGPS as LalGps
         lgps = list(map(LalGps, table['time']))
         t2 = type(table)(data=[lgps], names=['time'])
         rate2 = t2.event_rate(1, start=table['time'].min())
+
         utils.assert_quantity_sub_equal(rate, rate2)
 
-        # check that method can function without explicit time column
-        # (and no data) if and only if start/end are both given
+    def test_event_rates_start_end(self):
+        """Check that `EventTable.event_rate` can function without explicit time column
+        (and no data) if and only if start/end are both given
+        """
         t2 = self.create(10, names=['a', 'b'])
         with pytest.raises(ValueError) as exc:
             t2.event_rate(1)
@@ -399,7 +479,7 @@ class TestEventTable(TestTable):
         t2.binned_event_rates(1, 'a', (10, 100), start=0, end=10)
 
     def test_plot(self, table):
-        with pytest.warns(DeprecationWarning):
+        with pytest.deprecated_call():
             plot = table.plot('time', 'frequency', color='snr')
             plot.close()
 
@@ -417,26 +497,87 @@ class TestEventTable(TestTable):
     def test_get_column(self, table):
         utils.assert_array_equal(table.get_column('snr'), table['snr'])
 
+    def test_cluster(self, clustertable):
+        # check that the central data points are all clustered away,
+        # the original table is unchanged, and all points return their
+        # intended values
+        t = clustertable.cluster('time', 'amplitude', 0.6)
+        assert len(t) == 3
+        assert len(clustertable) == 7
+        assert all(t['amplitude'] == [11, 10, 9])
+        assert all(t['time'] == [0.0, 2.0, 4.0])
+
+    def test_single_point_cluster(self, clustertable):
+        # check that a large cluster window returns at least one data point
+        t = clustertable.cluster('time', 'amplitude', 10)
+        assert len(t) == 1
+        assert all(t['amplitude'] == [11])
+        assert all(t['time'] == [0.0])
+
+    def test_cluster_window(self, clustertable):
+        # check that a non-positive window throws an appropriate ValueError
+        with pytest.raises(ValueError) as exc:
+            clustertable.cluster('time', 'amplitude', 0)
+        assert str(exc.value) == 'Window must be a positive value'
+
     # -- test I/O -------------------------------
 
+    def test_read_write_hdf5(self, table):
+        # check that our overrides of astropy's H5 reader
+        # didn't break everything
+        with utils.TemporaryFilename(suffix=".h5") as tmp:
+            table.write(tmp, path="/data")
+            t2 = self.TABLE.read(tmp, path="/data")
+            utils.assert_table_equal(t2, table)
+
+            t2 = self.TABLE.read(
+                tmp,
+                path="/data",
+                selection="frequency>500",
+                columns=["time", "snr"],
+            )
+            utils.assert_table_equal(
+                t2,
+                filter_table(table, "frequency>500")[("time", "snr")],
+            )
+
     @pytest.mark.parametrize('fmtname', ('Omega', 'cWB'))
     def test_read_write_ascii(self, table, fmtname):
-        fmt = 'ascii.%s' % fmtname.lower()
+        fmt = 'ascii.{}'.format(fmtname.lower())
         with utils.TemporaryFilename(suffix='.txt') as tmp:
             # check write/read returns the same table
             with open(tmp, 'w') as fobj:
                 table.write(fobj, format=fmt)
-            utils.assert_table_equal(table, self.TABLE.read(tmp, format=fmt),
-                                     almost_equal=True)
+            t2 = self.TABLE.read(tmp, format=fmt)
+            utils.assert_table_equal(table, t2, almost_equal=True)
 
+            # check that we can use selections and column filtering
+            t2 = self.TABLE.read(
+                tmp,
+                format=fmt,
+                selection="frequency>500",
+                columns=["time", "snr"],
+            )
+            utils.assert_table_equal(
+                t2,
+                filter_table(table, "frequency>500")[("time", "snr")],
+                almost_equal=True,
+            )
+
+    @pytest.mark.parametrize('fmtname', ('Omega', 'cWB'))
+    def test_read_write_ascii_error(self, table, fmtname):
         with utils.TemporaryFilename(suffix='.txt') as tmp:
-            with open(tmp, 'w') as f:
+            with open(tmp, 'w'):
                 pass  # write empty file
             # assert reading blank file doesn't work with column name error
             with pytest.raises(InconsistentTableError) as exc:
-                self.TABLE.read(tmp, format=fmt)
-            assert str(exc.value) == ('No column names found in %s header'
-                                      % fmtname)
+                self.TABLE.read(
+                    tmp,
+                    format="ascii.{}".format(fmtname.lower()),
+                )
+            assert str(exc.value) == (
+                'No column names found in {} header'.format(fmtname)
+            )
 
     def test_read_pycbc_live(self):
         table = self.create(
@@ -456,22 +597,25 @@ class TestEventTable(TestTable):
                 group['psd'].attrs['delta_f'] = psd.df.to('Hz').value
 
             # check that we can read
-            t2 = self.TABLE.read(fp)
+            t2 = self.TABLE.read(fp, format="hdf5.pycbc_live")
             utils.assert_table_equal(table, t2)
             # and check metadata was recorded correctly
             assert t2.meta['ifo'] == 'X1'
 
             # check keyword arguments result in same table
-            t2 = self.TABLE.read(fp, format='hdf5.pycbc_live')
-            utils.assert_table_equal(table, t2)
             t2 = self.TABLE.read(fp, format='hdf5.pycbc_live', ifo='X1')
+            utils.assert_table_equal(table, t2)
 
             # assert loudest works
-            t2 = self.TABLE.read(fp, loudest=True)
+            t2 = self.TABLE.read(fp, format="hdf5.pycbc_live", loudest=True)
             utils.assert_table_equal(table.filter('snr > 500'), t2)
 
             # check extended_metadata=True works (default)
-            t2 = self.TABLE.read(fp, extended_metadata=True)
+            t2 = self.TABLE.read(
+                fp,
+                format="hdf5.pycbc_live",
+                extended_metadata=True,
+            )
             utils.assert_table_equal(table, t2)
             utils.assert_array_equal(t2.meta['loudest'], loudest)
             utils.assert_quantity_sub_equal(
@@ -479,20 +623,29 @@ class TestEventTable(TestTable):
                 exclude=['name', 'channel', 'unit', 'epoch'])
 
             # check extended_metadata=False works
-            t2 = self.TABLE.read(fp, extended_metadata=False)
+            t2 = self.TABLE.read(
+                fp,
+                format="hdf5.pycbc_live",
+                extended_metadata=False,
+            )
             assert t2.meta == {'ifo': 'X1'}
 
             # double-check that loudest and extended_metadata=False work
-            t2 = self.TABLE.read(fp, loudest=True, extended_metadata=False)
+            t2 = self.TABLE.read(
+                fp,
+                format="hdf5.pycbc_live",
+                loudest=True,
+                extended_metadata=False,
+            )
             utils.assert_table_equal(table.filter('snr > 500'), t2)
             assert t2.meta == {'ifo': 'X1'}
 
             # add another IFO, then assert that reading the table without
             # specifying the IFO fails
-            with h5py.File(fp) as h5f:
+            with h5py.File(fp, "r+") as h5f:
                 h5f.create_group('Z1')
             with pytest.raises(ValueError) as exc:
-                self.TABLE.read(fp)
+                self.TABLE.read(fp, format="hdf5.pycbc_live")
             assert str(exc.value).startswith(
                 'PyCBC live HDF5 file contains dataset groups')
 
@@ -501,44 +654,141 @@ class TestEventTable(TestTable):
             utils.assert_table_equal(table, t2)
 
             # assert processed colums works
-            t2 = self.TABLE.read(fp, ifo='X1', columns=['mchirp', 'new_snr'])
+            t2 = self.TABLE.read(
+                fp,
+                format="hdf5.pycbc_live",
+                ifo="X1",
+                columns=["mchirp", "new_snr"],
+            )
             mchirp = (table['mass1'] * table['mass2']) ** (3/5.) / (
                 table['mass1'] + table['mass2']) ** (1/5.)
             utils.assert_array_equal(t2['mchirp'], mchirp)
 
-            # test with selection
-            t2 = self.TABLE.read(fp, format='hdf5.pycbc_live',
-                                 ifo='X1', selection='snr>.5')
-            utils.assert_table_equal(filter_table(table, 'snr>.5'), t2)
+            # test with selection and columns
+            t2 = self.TABLE.read(
+                fp,
+                format='hdf5.pycbc_live',
+                ifo='X1',
+                selection='snr>.5',
+                columns=("a", "b", "mass1"),
+            )
+            utils.assert_table_equal(
+                t2,
+                filter_table(table, 'snr>.5')[("a", "b", "mass1")],
+            )
+
+            # regression test: gwpy/gwpy#1081
+            t2 = self.TABLE.read(
+                fp,
+                format='hdf5.pycbc_live',
+                ifo='X1',
+                selection='snr>.5',
+                columns=("a", "b", "snr"),
+            )
+            utils.assert_table_equal(
+                t2,
+                filter_table(table, 'snr>.5')[("a", "b", "snr")],
+            )
+
         finally:
             if os.path.isdir(os.path.dirname(fp)):
                 shutil.rmtree(os.path.dirname(fp))
 
-    def test_fetch_hacr(self):
-        table = self.create(100, names=HACR_COLUMNS)
+    def test_read_snax(self):
+        table = self.create(
+            100, names=['time', 'snr', 'frequency'])
+        fp = os.path.join(tempfile.mkdtemp(), 'SNAX-0-0.h5')
         try:
-            from pymysql import connect
-        except ImportError:
-            mockee = 'gwpy.table.io.hacr.connect'
-        else:
-            mockee = 'pymysql.connect'
-        with mock.patch(mockee) as mock_connect:
-            mock_connect.return_value = mock_hacr_connection(
-                table, 123, 456)
+            # write table in snax format (by hand)
+            with h5py.File(fp, 'w') as h5f:
+                group = h5f.create_group('H1:FAKE')
+                group.create_dataset(data=table, name='0.0_20.0')
 
-            # test simple query returns the full table
-            t2 = self.TABLE.fetch('hacr', 'X1:TEST-CHANNEL', 123, 456)
+            # check that we can read
+            t2 = self.TABLE.read(fp, 'H1:FAKE', format='hdf5.snax')
             utils.assert_table_equal(table, t2)
 
-            # test column selection works
-            t2 = self.TABLE.fetch('hacr', 'X1:TEST-CHANNEL', 123, 456,
-                                  columns=['gps_start', 'snr'])
-            utils.assert_table_equal(table['gps_start', 'snr'], t2)
-
-            # test column selection works
-            t2 = self.TABLE.fetch('hacr', 'X1:TEST-CHANNEL', 123, 456,
-                                  columns=['gps_start', 'snr'],
-                                  selection='freq_central>500')
+            # test with selection and columns
+            t2 = self.TABLE.read(
+                fp,
+                'H1:FAKE',
+                format='hdf5.snax',
+                selection='snr>.5',
+                columns=('time', 'snr'),
+            )
             utils.assert_table_equal(
-                filter_table(table, 'freq_central>500')['gps_start', 'snr'],
-                t2)
+                t2,
+                filter_table(table, 'snr>.5')[('time', 'snr')],
+            )
+
+        finally:
+            if os.path.isdir(os.path.dirname(fp)):
+                shutil.rmtree(os.path.dirname(fp))
+
+    @pytest.fixture(scope="module")
+    def hacr_table(self):
+        """Create a table of HACR-like data, and patch
+        `pymysql.connect` to return it
+        """
+        table = self.create(100, names=HACR_COLUMNS)
+        connect = mock.patch(
+            "pymysql.connect",
+            return_value=mock_hacr_connection(table, 123, 456),
+        )
+        try:
+            connect.start()
+        except ImportError as exc:  # pragma: no-cover
+            pytest.skip(str(exc))
+        yield table
+        connect.stop()
+
+    def test_fetch_hacr(self, hacr_table):
+        t2 = self.TABLE.fetch('hacr', 'X1:TEST-CHANNEL', 123, 456)
+
+        # check type matches
+        assert type(t2) is self.TABLE
+
+        # check response is correct
+        utils.assert_table_equal(hacr_table, t2)
+
+    def test_fetch_hacr_columns(self, hacr_table):
+        t2 = self.TABLE.fetch('hacr', 'X1:TEST-CHANNEL', 123, 456,
+                              columns=['gps_start', 'snr'],
+                              selection='freq_central>500')
+        utils.assert_table_equal(
+            filter_table(hacr_table, 'freq_central>500')['gps_start', 'snr'],
+            t2,
+        )
+
+    @pytest_skip_network_error
+    def test_fetch_open_data(self):
+        table = self.TABLE.fetch_open_data("GWTC-1-confident")
+        assert len(table)
+        assert {
+            "mass_1_source",
+            "luminosity_distance",
+            "chi_eff"
+        }.intersection(table.colnames)
+        # check unit parsing worked
+        assert table["luminosity_distance"].unit == "Mpc"
+
+    @pytest_skip_network_error
+    def test_fetch_open_data_kwargs(self):
+        table = self.TABLE.fetch_open_data(
+            "GWTC-1-confident",
+            selection="mass_1_source < 5",
+            columns=[
+                "name",
+                "mass_1_source",
+                "mass_2_source",
+                "luminosity_distance"
+            ],
+        )
+        assert len(table) == 1
+        assert table[0]["name"] == "GW170817-v3"
+        assert set(table.colnames) == {
+            "name",
+            "mass_1_source",
+            "mass_2_source",
+            "luminosity_distance"
+        }
