@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -37,8 +37,6 @@ This module defines the following classes
 user-facing objects.**
 """
 
-from __future__ import (division, print_function)
-
 import os
 import sys
 import warnings
@@ -48,20 +46,20 @@ from math import ceil
 import numpy
 
 from astropy import units
-from astropy import __version__ as astropy_version
 from astropy.io import registry as io_registry
 
-from ..types import (Array2D, Series)
+from gwosc.api import DEFAULT_URL as GWOSC_DEFAULT_HOST
+
+from ..types import Series
 from ..detector import (Channel, ChannelList)
-from ..io import datafind
+from ..segments import SegmentList
 from ..time import (Time, LIGOTimeGPS, gps_types, to_gps)
 from ..utils import gprint
 
 __author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
 
-__all__ = ['TimeSeriesBase', 'ArrayTimeSeries', 'TimeSeriesBaseDict']
+__all__ = ['TimeSeriesBase', 'TimeSeriesBaseDict', 'TimeSeriesBaseList']
 
-ASTROPY_2_0 = astropy_version >= '2.0'
 
 _UFUNC_STRING = {
     'less': '<',
@@ -72,6 +70,8 @@ _UFUNC_STRING = {
 }
 
 
+# -- utilities ----------------------------------------------------------------
+
 def _format_time(gps):
     if isinstance(gps, gps_types):
         return float(gps)
@@ -80,6 +80,39 @@ def _format_time(gps):
     return gps
 
 
+def _dynamic_scaled(scaled, channel):
+    """Determine default for scaled based on channel name
+
+    This is mainly to work around LIGO not correctly recording ADC
+    scaling parameters for most of Advanced LIGO (through 2019).
+
+    Parameters
+    ----------
+    scaled : `bool`, `None`
+        the scaled argument as given by the user
+
+    channel : `str`
+        the name of the channel to be read
+
+    Returns
+    -------
+    scaled : `bool`
+        `False` if channel is from LIGO, otherwise `True`
+
+    Examples
+    --------
+    >>> _dynamic_scaled(None, "H1:channel")
+    False
+    >>> _dynamic_scaled(None, "V1:channel")
+    True
+    """
+    if scaled is not None:
+        return scaled
+    return not str(channel).startswith(("H1", "L1"))
+
+
+# -- TimeSeriesBase------------------------------------------------------------
+
 class TimeSeriesBase(Series):
     """An `Array` with time-domain metadata.
 
@@ -149,9 +182,8 @@ class TimeSeriesBase(Series):
             kwargs['xindex'] = times
 
         # generate TimeSeries
-        new = super(TimeSeriesBase, cls).__new__(cls, data, name=name,
-                                                 unit=unit, channel=channel,
-                                                 **kwargs)
+        new = super().__new__(cls, data, name=name, unit=unit,
+                              channel=channel, **kwargs)
 
         # manually set sample_rate if given
         if sample_rate is not None:
@@ -221,7 +253,8 @@ class TimeSeriesBase(Series):
 
         :type: `~astropy.units.Quantity` scalar
         """
-        return units.Quantity(self.span[1] - self.span[0], self.xunit)
+        return units.Quantity(self.span[1] - self.span[0], self.xunit,
+                              dtype=float)
 
     # -- TimeSeries accessors -------------------
 
@@ -262,17 +295,14 @@ class TimeSeriesBase(Series):
             number of parallel processes to use, serial process by
             default.
 
-        gap : `str`, optional
-            how to handle gaps in the cache, one of
-
-            - 'ignore': do nothing, let the undelying reader method handle it
-            - 'warn': do nothing except print a warning to the screen
-            - 'raise': raise an exception upon finding a gap (default)
-            - 'pad': insert a value to fill the gaps
-
         pad : `float`, optional
-            value with which to fill gaps in the source data, only used if
-            gap is not given, or `gap='pad'` is given
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
+
+        Raises
+        ------
+        IndexError
+            if ``source`` is an empty list
 
         Notes
         -----"""
@@ -299,7 +329,7 @@ class TimeSeriesBase(Series):
     @classmethod
     def fetch(cls, channel, start, end, host=None, port=None, verbose=False,
               connection=None, verify=False, pad=None, allow_tape=None,
-              type=None, dtype=None):
+              scaled=None, type=None, dtype=None):
         """Fetch data from NDS
 
         Parameters
@@ -325,11 +355,17 @@ class TimeSeriesBase(Series):
         verify : `bool`, optional, default: `False`
             check channels exist in database before asking for data
 
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect
+
         connection : `nds2.connection`, optional
             open NDS connection to use
 
         verbose : `bool`, optional
-            print verbose output about NDS progress, useful for debugging
+            print verbose output about NDS progress, useful for debugging;
+            if ``verbose`` is specified as a string, this defines the
+            prefix for the progress meter
 
         type : `int`, optional
             NDS2 channel type integer
@@ -339,13 +375,13 @@ class TimeSeriesBase(Series):
         """
         return cls.DictClass.fetch(
             [channel], start, end, host=host, port=port, verbose=verbose,
-            connection=connection, verify=verify, pad=pad,
+            connection=connection, verify=verify, pad=pad, scaled=scaled,
             allow_tape=allow_tape, type=type, dtype=dtype)[str(channel)]
 
     @classmethod
     def fetch_open_data(cls, ifo, start, end, sample_rate=4096,
                         tag=None, version=None,
-                        format='hdf5', host='https://losc.ligo.org',
+                        format='hdf5', host=GWOSC_DEFAULT_HOST,
                         verbose=False, cache=None, **kwargs):
         """Fetch open-access data from the LIGO Open Science Center
 
@@ -462,8 +498,8 @@ class TimeSeriesBase(Series):
                                host=host, cls=cls, **kwargs)
 
     @classmethod
-    def find(cls, channel, start, end, frametype=None,
-             pad=None, dtype=None, nproc=1, verbose=False, **readargs):
+    def find(cls, channel, start, end, frametype=None, pad=None,
+             scaled=None, dtype=None, nproc=1, verbose=False, **readargs):
         """Find and read data from frames for a channel
 
         Parameters
@@ -483,14 +519,14 @@ class TimeSeriesBase(Series):
             name of frametype in which this channel is stored, will search
             for containing frame types if necessary
 
-        pad : `float`, optional
-            value with which to fill gaps in the source data, only used if
-            gap is not given, or `gap='pad'` is given
-
         nproc : `int`, optional, default: `1`
             number of parallel processes to use, serial process by
             default.
 
+        pad : `float`, optional
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
+
         dtype : `numpy.dtype`, `str`, `type`, or `dict`
             numeric data type for returned data, e.g. `numpy.float`, or
             `dict` of (`channel`, `dtype`) pairs
@@ -499,18 +535,27 @@ class TimeSeriesBase(Series):
             allow reading from frame files on (slow) magnetic tape
 
         verbose : `bool`, optional
-            print verbose output about NDS progress.
+            print verbose output about read progress, if ``verbose``
+            is specified as a string, this defines the prefix for the
+            progress meter
 
         **readargs
             any other keyword arguments to be passed to `.read()`
         """
         return cls.DictClass.find(
-            [channel], start, end, frametype=frametype, verbose=verbose,
-            pad=pad, dtype=dtype, nproc=nproc, **readargs)[str(channel)]
+            [channel], start, end,
+            frametype=frametype,
+            verbose=verbose,
+            pad=pad,
+            scaled=scaled,
+            dtype=dtype,
+            nproc=nproc,
+            **readargs
+        )[str(channel)]
 
     @classmethod
-    def get(cls, channel, start, end, pad=None, dtype=None, verbose=False,
-            allow_tape=None, **kwargs):
+    def get(cls, channel, start, end, pad=None, scaled=None,
+            dtype=None, verbose=False, allow_tape=None, **kwargs):
         """Get data for this channel from frames or NDS
 
         This method dynamically accesses either frames on disk, or a
@@ -530,8 +575,12 @@ class TimeSeriesBase(Series):
             any input parseable by `~gwpy.time.to_gps` is fine
 
         pad : `float`, optional
-            value with which to fill gaps in the source data, default to
-            'don't fill gaps'
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
+
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect
 
         dtype : `numpy.dtype`, `str`, `type`, or `dict`
             numeric data type for returned data, e.g. `numpy.float`, or
@@ -549,14 +598,16 @@ class TimeSeriesBase(Series):
             retrieving data from tape if required
 
         verbose : `bool`, optional
-            print verbose output about NDS progress.
+            print verbose output about data access progress, if ``verbose``
+            is specified as a string, this defines the prefix for the
+            progress meter
 
         **kwargs
             other keyword arguments to pass to either
             :meth:`.find` (for direct GWF file access) or
             :meth:`.fetch` for remote NDS2 access
 
-        See Also
+        See also
         --------
         TimeSeries.fetch
             for grabbing data from a remote NDS2 server
@@ -564,48 +615,21 @@ class TimeSeriesBase(Series):
             for discovering and reading data from local GWF files
         """
         return cls.DictClass.get(
-            [channel], start, end, pad=pad, dtype=dtype, verbose=verbose,
-            allow_tape=allow_tape, **kwargs)[str(channel)]
+            [channel], start, end, pad=pad, scaled=scaled, dtype=dtype,
+            verbose=verbose, allow_tape=allow_tape, **kwargs)[str(channel)]
 
     # -- utilities ------------------------------
 
-    def shift(self, delta):
-        """Shift this `TimeSeries` forward in time by ``delta``
-
-        This modifies the series in-place.
-
-        Parameters
-        ----------
-        delta : `float`, `~astropy.units.Quantity`, `str`
-            The amount by which to shift (in seconds if `float`), give
-            a negative value to shift backwards in time
-
-        Examples
-        --------
-        >>> from gwpy.timeseries import TimeSeries
-        >>> a = TimeSeries([1, 2, 3, 4, 5], t0=0, dt=1)
-        >>> print(a.t0)
-        0.0 s
-        >>> a.shift(5)
-        >>> print(a.t0)
-        5.0 s
-        >>> a.shift('-1 hour')
-        -3595.0 s
-        """
-        delta = units.Quantity(delta, 's')
-        self.t0 += delta
-
-    def plot(self, **kwargs):
+    def plot(self, method='plot', figsize=(12, 4), xscale='auto-gps',
+             **kwargs):
         """Plot the data for this timeseries
 
-        All keywords are passed to `~gwpy.plotter.TimeSeriesPlot`
-
         Returns
         -------
-        plot : `~gwpy.plotter.TimeSeriesPlot`
+        figure : `~matplotlib.figure.Figure`
             the newly created figure, with populated Axes.
 
-        See Also
+        See also
         --------
         matplotlib.pyplot.figure
             for documentation of keyword arguments used to create the
@@ -616,17 +640,27 @@ class TimeSeriesBase(Series):
         matplotlib.axes.Axes.plot
             for documentation of keyword arguments used in rendering the data
         """
-        from ..plotter import TimeSeriesPlot
-        return TimeSeriesPlot(self, **kwargs)
+        kwargs.update(figsize=figsize, xscale=xscale)
+        return super().plot(method=method, **kwargs)
 
     @classmethod
-    def from_nds2_buffer(cls, buffer_, **metadata):
-        """Construct a new `TimeSeries` from an `nds2.buffer` object
+    def from_nds2_buffer(cls, buffer_, scaled=None, copy=True, **metadata):
+        """Construct a new series from an `nds2.buffer` object
+
+        **Requires:** |nds2|_
 
         Parameters
         ----------
         buffer_ : `nds2.buffer`
             the input NDS2-client buffer to read
+
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect
+
+        copy : `bool`, optional
+            if `True`, copy the contained data array to new  to a new array
+
         **metadata
             any other metadata keyword arguments to pass to the `TimeSeries`
             constructor
@@ -636,20 +670,31 @@ class TimeSeriesBase(Series):
         timeseries : `TimeSeries`
             a new `TimeSeries` containing the data from the `nds2.buffer`,
             and the appropriate metadata
-
-        Notes
-        -----
-        This classmethod requires the nds2-client package
         """
-        # cast as TimeSeries and return
+        # get Channel from buffer
         channel = Channel.from_nds2(buffer_.channel)
+
+        # set default metadata
         metadata.setdefault('channel', channel)
         metadata.setdefault('epoch', LIGOTimeGPS(buffer_.gps_seconds,
                                                  buffer_.gps_nanoseconds))
         metadata.setdefault('sample_rate', channel.sample_rate)
         metadata.setdefault('unit', channel.unit)
-        metadata.setdefault('name', str(channel))
-        return cls(buffer_.data, **metadata)
+        metadata.setdefault('name', buffer_.name)
+
+        # unwrap data
+        scaled = _dynamic_scaled(scaled, channel.name)
+        slope = buffer_.signal_slope
+        offset = buffer_.signal_offset
+        null_scaling = slope == 1. and offset == 0.
+        if scaled and not null_scaling:
+            data = buffer_.data.copy() * slope + offset
+            copy = False
+        else:
+            data = buffer_.data
+
+        # construct new TimeSeries-like object
+        return cls(data, copy=copy, **metadata)
 
     @classmethod
     def from_lal(cls, lalts, copy=True):
@@ -663,7 +708,7 @@ class TimeSeriesBase(Series):
             unit = None
         channel = Channel(lalts.name, sample_rate=1/lalts.deltaT, unit=unit,
                           dtype=lalts.data.data.dtype)
-        out = cls(lalts.data.data, channel=channel, t0=float(lalts.epoch),
+        out = cls(lalts.data.data, channel=channel, t0=lalts.epoch,
                   dt=lalts.deltaT, unit=unit, name=lalts.name, copy=False)
         if copy:
             return out.copy()
@@ -684,8 +729,14 @@ class TimeSeriesBase(Series):
 
         # create TimeSeries
         create = find_typed_function(self.dtype, 'Create', 'TimeSeries')
-        lalts = create(self.name, lal.LIGOTimeGPS(self.epoch.gps), 0,
-                       self.dt.value, unit, self.shape[0])
+        lalts = create(
+            self.name,
+            LIGOTimeGPS(to_gps(self.epoch.gps)),
+            0,
+            self.dt.value,
+            unit,
+            self.shape[0],
+        )
         lalts.data.data = self.value
         return lalts
 
@@ -730,26 +781,22 @@ class TimeSeriesBase(Series):
 
     # -- TimeSeries operations ------------------
 
-    if ASTROPY_2_0:
-        def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
-            # this is new in numpy 1.13, astropy 2.0 adopts it, we need to
-            # work out how to handle this and __array_wrap__ together properly
-            out = super(TimeSeriesBase, self).__array_ufunc__(
-                ufunc, method, *inputs, **kwargs)
-            if out.dtype is numpy.dtype(bool) and len(inputs) == 2:
-                from .statevector import StateTimeSeries
-                orig, value = inputs
-                try:
-                    op_ = _UFUNC_STRING[ufunc.__name__]
-                except KeyError:
-                    op_ = ufunc.__name__
-                out = out.view(StateTimeSeries)
-                out.__metadata_finalize__(orig)
-                out.override_unit('')
-                oname = orig.name if isinstance(orig, type(self)) else orig
-                vname = value.name if isinstance(value, type(self)) else value
-                out.name = '{0!s} {1!s} {2!s}'.format(oname, op_, vname)
-            return out
+    def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
+        out = super().__array_ufunc__(ufunc, method, *inputs, **kwargs)
+        if out.dtype is numpy.dtype(bool) and len(inputs) == 2:
+            from .statevector import StateTimeSeries
+            orig, value = inputs
+            try:
+                op_ = _UFUNC_STRING[ufunc.__name__]
+            except KeyError:
+                op_ = ufunc.__name__
+            out = out.view(StateTimeSeries)
+            out.__metadata_finalize__(orig)
+            out.override_unit('')
+            oname = orig.name if isinstance(orig, type(self)) else orig
+            vname = value.name if isinstance(value, type(self)) else value
+            out.name = '{0!s} {1!s} {2!s}'.format(oname, op_, vname)
+        return out
 
     def __array_wrap__(self, obj, context=None):
         # if output type is boolean, return a `StateTimeSeries`
@@ -768,8 +815,7 @@ class TimeSeriesBase(Series):
             result.name = '{0!s} {1!s} {2!s}'.format(oname, op_, vname)
         # otherwise, return a regular TimeSeries
         else:
-            result = super(TimeSeriesBase, self).__array_wrap__(
-                obj, context=context)
+            result = super().__array_wrap__(obj, context=context)
         return result
 
 
@@ -804,6 +850,24 @@ class TimeSeriesBaseDict(OrderedDict):
     """
     EntryClass = TimeSeriesBase
 
+    @property
+    def span(self):
+        """The GPS ``[start, stop)`` extent of data in this `dict`
+
+        :type: `~gwpy.segments.Segment`
+        """
+        span = SegmentList()
+        for value in self.values():
+            span.append(value.span)
+        try:
+            return span.extent()
+        except ValueError as exc:  # empty list
+            exc.args = (
+                'cannot calculate span for empty {0}'.format(
+                    type(self).__name__),
+            )
+            raise
+
     @classmethod
     def read(cls, source, *args, **kwargs):
         """Read data for multiple channels into a `TimeSeriesDict`
@@ -837,17 +901,9 @@ class TimeSeriesBaseDict(OrderedDict):
             number of parallel processes to use, serial process by
             default.
 
-        gap : `str`, optional
-            how to handle gaps in the cache, one of
-
-            - 'ignore': do nothing, let the undelying reader method handle it
-            - 'warn': do nothing except print a warning to the screen
-            - 'raise': raise an exception upon finding a gap (default)
-            - 'pad': insert a value to fill the gaps
-
         pad : `float`, optional
-            value with which to fill gaps in the source data, only used if
-            gap is not given, or `gap='pad'` is given
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
 
         Returns
         -------
@@ -906,7 +962,7 @@ class TimeSeriesBaseDict(OrderedDict):
         **kwargs
             other keyword arguments to send to `TimeSeries.append`
 
-        See Also
+        See also
         --------
         TimeSeries.append
             for details of the underlying series append operation
@@ -935,7 +991,7 @@ class TimeSeriesBaseDict(OrderedDict):
         **kwargs
             other keyword arguments to send to `TimeSeries.prepend`
 
-        See Also
+        See also
         --------
         TimeSeries.prepend
             for details of the underlying series prepend operation
@@ -963,7 +1019,7 @@ class TimeSeriesBaseDict(OrderedDict):
             GPS end time of required data, defaults to end of data found;
             any input parseable by `~gwpy.time.to_gps` is fine
 
-        See Also
+        See also
         --------
         TimeSeries.crop
             for more details
@@ -996,7 +1052,7 @@ class TimeSeriesBaseDict(OrderedDict):
     @classmethod
     def fetch(cls, channels, start, end, host=None, port=None,
               verify=False, verbose=False, connection=None,
-              pad=None, allow_tape=None, type=None,
+              pad=None, scaled=None, allow_tape=None, type=None,
               dtype=None):
         """Fetch data from NDS for a number of channels.
 
@@ -1024,11 +1080,17 @@ class TimeSeriesBaseDict(OrderedDict):
             check channels exist in database before asking for data
 
         verbose : `bool`, optional
-            print verbose output about NDS progress.
+            print verbose output about NDS download progress, if ``verbose``
+            is specified as a string, this defines the prefix for the
+            progress meter
 
         connection : `nds2.connection`, optional
             open NDS connection to use.
 
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect.
+
         allow_tape : `bool`, optional
             allow data access from slow tapes. If `host` or `connection` is
             given, the default is to do whatever the server default is,
@@ -1075,19 +1137,21 @@ class TimeSeriesBaseDict(OrderedDict):
             else:
                 tapes = [allow_tape]
             for allow_tape_ in tapes:
+                error = ""  # container for error message from cls.fetch()
                 for host_, port_ in hostlist:
                     try:
                         return cls.fetch(channels, start, end, host=host_,
                                          port=port_, verbose=verbose,
                                          type=type, dtype=dtype, pad=pad,
-                                         allow_tape=allow_tape_)
+                                         scaled=scaled, allow_tape=allow_tape_)
                     except (RuntimeError, ValueError) as exc:
-                        warnings.warn(str(exc).split('\n')[0],
+                        error = str(exc)  # need to assign to take out of scope
+                        warnings.warn(error.split('\n', 1)[0],
                                       io_nds2.NDSWarning)
 
                 # if failing occurred because of data on tape, don't try
                 # reading channels individually, the same error will occur
-                if not allow_tape_ and 'Requested data is on tape' in str(exc):
+                if not allow_tape_ and 'Requested data is on tape' in error:
                     continue
 
                 # if we got this far, we can't get all channels in one go
@@ -1097,6 +1161,7 @@ class TimeSeriesBaseDict(OrderedDict):
                                                  verbose=verbose, type=type,
                                                  verify=verify,
                                                  dtype=dtype.get(c), pad=pad,
+                                                 scaled=scaled,
                                                  allow_tape=allow_tape_))
                         for c in channels)
             err = "Cannot find all relevant data on any known server."
@@ -1115,11 +1180,12 @@ class TimeSeriesBaseDict(OrderedDict):
         return fetch(channels, istart, iend, connection=connection,
                      host=host, port=port, verbose=verbose, type=type,
                      dtype=dtype, pad=pad, allow_tape=allow_tape,
+                     scaled=scaled,
                      series_class=cls.EntryClass).crop(start, end)
 
     @classmethod
     def find(cls, channels, start, end, frametype=None,
-             frametype_match=None, pad=None, dtype=None, nproc=1,
+             frametype_match=None, pad=None, scaled=None, dtype=None, nproc=1,
              verbose=False, allow_tape=True, observatory=None, **readargs):
         """Find and read data from frames for a number of channels.
 
@@ -1144,8 +1210,12 @@ class TimeSeriesBaseDict(OrderedDict):
             regular expression to use for frametype matching
 
         pad : `float`, optional
-            value with which to fill gaps in the source data, defaults to
-            'don't fill gaps'
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
+
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect.
 
         dtype : `numpy.dtype`, `str`, `type`, or `dict`
             numeric data type for returned data, e.g. `numpy.float`, or
@@ -1159,17 +1229,21 @@ class TimeSeriesBaseDict(OrderedDict):
             allow reading from frame files on (slow) magnetic tape
 
         verbose : `bool`, optional
-            print verbose output about NDS progress.
+            print verbose output about read progress, if ``verbose``
+            is specified as a string, this defines the prefix for the
+            progress meter
 
         **readargs
             any other keyword arguments to be passed to `.read()`
         """
+        from ..io import datafind as io_datafind
+
         start = to_gps(start)
         end = to_gps(end)
 
         # -- find frametype(s)
         if frametype is None:
-            matched = datafind.find_best_frametype(
+            matched = io_datafind.find_best_frametype(
                 channels, start, end, frametype_match=frametype_match,
                 allow_tape=allow_tape)
             frametypes = {}
@@ -1206,25 +1280,29 @@ class TimeSeriesBaseDict(OrderedDict):
                     exc.args = "Cannot parse list of IFOs from channel names",
                     raise
             # find frames
-            connection = datafind.connect()
-            cache = connection.find_frame_urls(observatory, frametype, start,
-                                               end, urltype='file')
+            cache = io_datafind.find_urls(
+                observatory,
+                frametype,
+                start,
+                end,
+                on_gaps="error" if pad is None else "warn",
+            )
             if not cache:
                 raise RuntimeError("No %s-%s frame files found for [%d, %d)"
                                    % (observatory, frametype, start, end))
             # read data
             readargs.setdefault('format', 'gwf')
             new = cls.read(cache, names, start=start, end=end, pad=pad,
-                           dtype=dtype, nproc=nproc, verbose=verbose,
-                           **readargs)
+                           scaled=scaled, dtype=dtype, nproc=nproc,
+                           verbose=verbose, **readargs)
             # map back to user-given channel name and append
             out.append(type(new)((key, new[chan]) for
                                  (key, chan) in zip(clist, names)))
         return out
 
     @classmethod
-    def get(cls, channels, start, end, pad=None, dtype=None, verbose=False,
-            allow_tape=None, **kwargs):
+    def get(cls, channels, start, end, pad=None, scaled=None,
+            dtype=None, verbose=False, allow_tape=None, **kwargs):
         """Retrieve data for multiple channels from frames or NDS
 
         This method dynamically accesses either frames on disk, or a
@@ -1248,8 +1326,12 @@ class TimeSeriesBaseDict(OrderedDict):
             will search for all required frame types
 
         pad : `float`, optional
-            value with which to fill gaps in the source data, only used if
-            gap is not given, or `gap='pad'` is given
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
+
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect.
 
         dtype : `numpy.dtype`, `str`, `type`, or `dict`
             numeric data type for returned data, e.g. `numpy.float`, or
@@ -1267,7 +1349,9 @@ class TimeSeriesBaseDict(OrderedDict):
             retrieving data from tape if required
 
         verbose : `bool`, optional
-            print verbose output about NDS progress.
+            print verbose output about data access progress, if ``verbose``
+            is specified as a string, this defines the prefix for the
+            progress meter
 
         **kwargs
             other keyword arguments to pass to either
@@ -1282,12 +1366,15 @@ class TimeSeriesBaseDict(OrderedDict):
                 nds_kw[key] = val
 
         # try and find from frames
-        if os.getenv('LIGO_DATAFIND_SERVER') and not nds_kw:
+        if not nds_kw and (
+                os.getenv('LIGO_DATAFIND_SERVER')
+                or os.getenv('VIRGODATA')
+        ):
             if verbose:
                 gprint("Attempting to access data from frames...")
             try:
-                return cls.find(channels, start, end, pad=pad, dtype=dtype,
-                                verbose=verbose,
+                return cls.find(channels, start, end, pad=pad, scaled=scaled,
+                                dtype=dtype, verbose=verbose,
                                 allow_tape=allow_tape or False,
                                 **kwargs)
             except (ImportError, RuntimeError, ValueError) as exc:
@@ -1302,8 +1389,9 @@ class TimeSeriesBaseDict(OrderedDict):
 
         # otherwise fetch from NDS
         try:
-            return cls.fetch(channels, start, end, pad=pad, dtype=dtype,
-                             allow_tape=allow_tape, verbose=verbose, **kwargs)
+            return cls.fetch(channels, start, end, pad=pad, scaled=scaled,
+                             dtype=dtype, allow_tape=allow_tape,
+                             verbose=verbose, **kwargs)
         except RuntimeError as exc:
             # if all else fails, try and get each channel individually
             if len(channels) == 1:
@@ -1314,12 +1402,47 @@ class TimeSeriesBaseDict(OrderedDict):
                     gprint("Failed to access data for all channels as a "
                            "group, trying individually:")
                 return cls(
-                    (c, cls.EntryClass.get(c, start, end, pad=pad, dtype=dtype,
+                    (c, cls.EntryClass.get(c, start, end, pad=pad,
+                                           scaled=scaled, dtype=dtype,
                                            allow_tape=allow_tape,
                                            verbose=verbose, **kwargs))
                     for c in channels)
 
-    def plot(self, label='key', **kwargs):
+    @classmethod
+    def from_nds2_buffers(cls, buffers, scaled=None, copy=True, **metadata):
+        """Construct a new dict from a list of `nds2.buffer` objects
+
+        **Requires:** |nds2|_
+
+        Parameters
+        ----------
+        buffers : `list` of `nds2.buffer`
+            the input NDS2-client buffers to read
+
+        scaled : `bool`, optional
+            apply slope and bias calibration to ADC data, for non-ADC data
+            this option has no effect.
+
+        copy : `bool`, optional
+            if `True`, copy the contained data array to new  to a new array
+
+        **metadata
+            any other metadata keyword arguments to pass to the `TimeSeries`
+            constructor
+
+        Returns
+        -------
+        dict : `TimeSeriesDict`
+            a new `TimeSeriesDict` containing the data from the given buffers
+        """
+        tsd = cls()
+        for buf in buffers:
+            tsd[buf.channel.name] = tsd.EntryClass.from_nds2_buffer(
+                buf, scaled=scaled, copy=copy, **metadata)
+        return tsd
+
+    def plot(self, label='key', method='plot', figsize=(12, 4),
+             xscale='auto-gps', **kwargs):
         """Plot the data for this `TimeSeriesBaseDict`.
 
         Parameters
@@ -1337,20 +1460,54 @@ class TimeSeriesBaseDict(OrderedDict):
             all other keyword arguments are passed to the plotter as
             appropriate
         """
-        from ..plotter import TimeSeriesPlot
-        figargs = dict()
-        for key in ['figsize', 'dpi']:
-            if key in kwargs:
-                figargs[key] = kwargs.pop(key)
-        plot_ = TimeSeriesPlot(**figargs)
-        ax = plot_.gca()
-        for lab, series in self.items():
+        # make plot
+        from ..plot import Plot
+        plot = Plot(self.values(), method=method, label=label, **kwargs)
+
+        # update labels
+        artmap = {'plot': 'lines', 'scatter': 'collections'}
+        artists = [x for ax in plot.axes for
+                   x in getattr(ax, artmap.get(method, 'lines'))]
+        for key, artist in zip(self, artists):
             if label.lower() == 'name':
-                lab = series.name
-            elif label.lower() != 'key':
+                lab = self[key].name
+            elif label.lower() == 'key':
+                lab = key
+            else:
                 lab = label
-            ax.plot(series, label=lab, **kwargs)
-        return plot_
+            artist.set_label(lab)
+
+        return plot
+
+    def step(self, label='key', where='post', figsize=(12, 4),
+             xscale='auto-gps', **kwargs):
+        """Create a step plot of this dict.
+
+        Parameters
+        ----------
+        label : `str`, optional
+            labelling system to use, or fixed label for all elements
+            Special values include
+
+            - ``'key'``: use the key of the `TimeSeriesBaseDict`,
+            - ``'name'``: use the :attr:`~TimeSeries.name` of each element
+
+            If anything else, that fixed label will be used for all lines.
+
+        **kwargs
+            all other keyword arguments are passed to the plotter as
+            appropriate
+        """
+        kwargs.setdefault(
+            "drawstyle",
+            "steps-{}".format(where),
+        )
+        tmp = type(self)()
+        for key, series in self.items():
+            tmp[key] = series.append(series.value[-1:], inplace=False)
+
+        return tmp.plot(label=label, figsize=figsize, xscale=xscale,
+                        **kwargs)
 
 
 # -- TimeSeriesBaseList -------------------------------------------------------
@@ -1381,7 +1538,7 @@ class TimeSeriesBaseList(list):
     def __init__(self, *items):
         """Initialise a new list
         """
-        super(TimeSeriesBaseList, self).__init__()
+        super().__init__()
         for item in items:
             self.append(item)
 
@@ -1396,13 +1553,13 @@ class TimeSeriesBaseList(list):
         if not isinstance(item, self.EntryClass):
             raise TypeError("Cannot append type '%s' to %s"
                             % (type(item).__name__, type(self).__name__))
-        super(TimeSeriesBaseList, self).append(item)
+        super().append(item)
         return self
     append.__doc__ = list.append.__doc__
 
     def extend(self, item):
         item = TimeSeriesBaseList(*item)
-        super(TimeSeriesBaseList, self).extend(item)
+        super().extend(item)
     extend.__doc__ = list.extend.__doc__
 
     def coalesce(self):
@@ -1433,31 +1590,33 @@ class TimeSeriesBaseList(list):
         del self[i:]
         return self
 
-    def join(self, pad=0.0, gap='raise'):
+    def join(self, pad=None, gap=None):
         """Concatenate all of the elements of this list into a single object
 
         Parameters
         ----------
-        pad : `float`, optional, default: `0.0`
-            value with which to pad gaps
+        pad : `float`, optional
+            value with which to fill gaps in the source data,
+            by default gaps will result in a `ValueError`.
 
         gap : `str`, optional, default: `'raise'`
-            what to do in the event of a discontguity in the data, one of
+            what to do if there are gaps in the data, one of
 
-            - 'raise': raise an exception
-            - 'warn': print a warning
-            - 'ignore': append series as if there was no gap
-            - 'pad': pad the gap with a value
+            - ``'raise'`` - raise a `ValueError`
+            - ``'ignore'`` - remove gap and join data
+            - ``'pad'`` - pad gap with zeros
+
+            If `pad` is given and is not `None`, the default is ``'pad'``,
+            otherwise ``'raise'``.
 
         Returns
         -------
-        `TimeSeriesBase`
-             a single `TimeSeriesBase` covering the full span of all entries
-             in this list
+        series : `gwpy.types.TimeSeriesBase` subclass
+             a single series containing all data from each entry in this list
 
-        See Also
+        See also
         --------
-        TimeSeriesBase.append
+        TimeSeries.append
             for details on how the individual series are concatenated together
         """
         if not self:
@@ -1469,13 +1628,13 @@ class TimeSeriesBaseList(list):
         return out
 
     def __getslice__(self, i, j):
-        return type(self)(*super(TimeSeriesBaseList, self).__getslice__(i, j))
+        return type(self)(*super().__getslice__(i, j))
 
     def __getitem__(self, key):
         if isinstance(key, slice):
             return type(self)(
-                *super(TimeSeriesBaseList, self).__getitem__(key))
-        return super(TimeSeriesBaseList, self).__getitem__(key)
+                *super().__getitem__(key))
+        return super().__getitem__(key)
 
     def copy(self):
         """Return a copy of this list with each element copied to new memory
