@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2017)
+# Copyright (C) Duncan Macleod (2017-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,8 +19,6 @@
 """I/O utilities for reading `TimeSeries` from a `list` of file paths.
 """
 
-from six import string_types
-
 from ...io.cache import (FILE_LIKE, read_cache, file_segment, sieve)
 from ...segments import Segment
 
@@ -51,16 +49,26 @@ def preformat_cache(cache, start=None, end=None):
     modcache : `list`
         A parsed, sieved list of paths based on the input arguments.
     """
-    # format cache file
-    if isinstance(cache, FILE_LIKE + string_types):  # open cache file
-        cache = read_cache(cache)
-    cache = type(cache)(cache)  # copy cache
-    cache.sort(key=file_segment)  # sort
+    # open cache file
+    if isinstance(cache, (str,) + FILE_LIKE):
+        return read_cache(cache, sort=file_segment,
+                          segment=Segment(start, end))
 
-    # get timing
+    # format existing cache file
+    cache = type(cache)(cache)  # copy cache
+
+    # sort cache
+    try:
+        cache.sort(key=file_segment)  # sort
+    except ValueError:
+        # if this failed, then the sieving will also fail, but lets proceed
+        # anyway, since the user didn't actually ask us to do this (but
+        # its a very good idea)
+        return cache
+
+    # sieve cache
     if start is None:  # start time of earliest file
         start = file_segment(cache[0])[0]
     if end is None:  # end time of latest file
         end = file_segment(cache[-1])[-1]
-
-    return sieve(cache, segment=Segment(start, end))  # sieve
+    return sieve(cache, segment=Segment(start, end))
