@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -21,20 +21,17 @@
 The frame format is defined in LIGO-T970130 available from dcc.ligo.org.
 """
 
-from __future__ import (absolute_import, division)
-
 import os.path
-
-from six import string_types
+import warnings
 
 # import in this order so that lalframe throws the ImportError
 # to give the user a bit more information
 import lalframe
 import lal
-from lal.utils import CacheEntry
 
 
-from ....io import cache as io_cache
+from ....io.cache import is_cache
+from ....io.utils import (file_list, file_path)
 from ....utils import lal as lalutils
 from ... import TimeSeries
 
@@ -63,29 +60,35 @@ def open_data_source(source):
     ValueError
         If the input format cannot be identified.
     """
-    if isinstance(source, io_cache.FILE_LIKE):
-        source = source.name
-    if isinstance(source, CacheEntry):
-        source = source.path
+    # -- preformatting
 
-    # read cache file
-    if (isinstance(source, string_types) and
-            source.endswith(('.lcf', '.cache'))):
-        return lalframe.FrStreamCacheOpen(lal.CacheImport(source))
+    try:
+        source = file_path(source)
+    except ValueError:  # not parsable as a single file
+        pass
 
-    # read glue cache object
-    if isinstance(source, list) and io_cache.is_cache(source):
+    # import cache from file
+    if (
+        isinstance(source, str)
+        and source.endswith(('.lcf', '.cache'))
+    ):
+        source = lal.CacheImport(source)
+
+    # reformat cache (or any list of files) as a lal cache object
+    if isinstance(source, list) and is_cache(source):
         cache = lal.Cache()
-        for entry in io_cache.file_list(source):
+        for entry in file_list(source):
             cache = lal.CacheMerge(cache, lal.CacheGlob(*os.path.split(entry)))
-        return lalframe.FrStreamCacheOpen(cache)
+        source = cache
+
+    # -- now we have a lal.Cache or a filename
 
     # read lal cache object
     if isinstance(source, lal.Cache):
         return lalframe.FrStreamCacheOpen(source)
 
     # read single file
-    if isinstance(source, string_types):
+    if isinstance(source, str):
         return lalframe.FrStreamOpen(*map(str, os.path.split(source)))
 
     raise ValueError("Don't know how to open data source of type %r"
@@ -109,7 +112,7 @@ def get_stream_duration(stream):
                             stream.epoch.gpsNanoSeconds)
     # loop over each file in the stream cache and query its duration
     nfile = stream.cache.length
-    duration = 0
+    duration = 0.
     for dummy_i in range(nfile):
         for dummy_j in range(lalframe.FrFileQueryNFrame(stream.file)):
             duration += lalframe.FrFileQueryDt(stream.file, 0)
@@ -121,9 +124,18 @@ def get_stream_duration(stream):
 
 # -- read ---------------------------------------------------------------------
 
-def read(source, channels, start=None, end=None, series_class=TimeSeries):
+def read(source, channels, start=None, end=None, series_class=TimeSeries,
+         scaled=None):
     """Read data from one or more GWF files using the LALFrame API
     """
+    # scaled must be provided to provide a consistent API with frameCPP
+    if scaled is not None:
+        warnings.warn(
+            "the `scaled` keyword argument is not supported by lalframe, "
+            "if you require ADC scaling, please install "
+            "python-ldas-tools-framecpp",
+        )
+
     stream = open_data_source(source)
 
     # parse times and restrict to available data
@@ -132,14 +144,10 @@ def read(source, channels, start=None, end=None, series_class=TimeSeries):
     streamdur = get_stream_duration(stream)
     if start is None:
         start = epoch
-    else:
-        start = max(epoch, lalutils.to_lal_ligotimegps(start))
+    start = max(epoch, lalutils.to_lal_ligotimegps(start))
     if end is None:
-        offset = float(start - epoch)
-        duration = streamdur - offset
-    else:
-        end = min(epoch + streamdur, end)
-        duration = float(end - start)
+        end = epoch + streamdur
+    duration = float(end - start)
 
     # read data
     out = series_class.DictClass()
@@ -152,7 +160,12 @@ def read(source, channels, start=None, end=None, series_class=TimeSeries):
 
 
 def _read_channel(stream, channel, start, duration):
-    dtype = lalframe.FrStreamGetTimeSeriesType(channel, stream)
+    try:
+        dtype = lalframe.FrStreamGetTimeSeriesType(channel, stream)
+    except RuntimeError as exc:
+        if str(exc).lower() == "wrong name":
+            exc.args = "channel '{}' not found".format(channel),
+        raise
     reader = lalutils.find_typed_function(dtype, 'FrStreamRead', 'TimeSeries',
                                           module=lalframe)
     return reader(stream, channel, start, duration, 0)
@@ -160,8 +173,12 @@ def _read_channel(stream, channel, start, duration):
 
 # -- write --------------------------------------------------------------------
 
-def write(tsdict, outfile, start=None, end=None,
-          name='gwpy', run=0):
+def write(
+        tsdict, outfile,
+        start=None, end=None,
+        type=None,
+        name='gwpy', run=0,
+):
     """Write data to a GWF file using the LALFrame API
     """
     if not start:
@@ -172,23 +189,37 @@ def write(tsdict, outfile, start=None, end=None,
 
     # get ifos list
     detectors = 0
+    _detidx = list(lalutils.LAL_DETECTORS.keys())
     for series in tsdict.values():
+        ifo = getattr(series.channel, "ifo", None)
         try:
-            idx = list(lalutils.LAL_DETECTORS.keys()).index(series.channel.ifo)
-            detectors |= 1 << 2*idx
-        except (KeyError, AttributeError):
+            idx = _detidx.index(ifo)
+        except ValueError:  # don't worry about mismatched detectors
             continue
+        else:
+            detectors |= 1 << 2 * idx
 
     # create new frame
     frame = lalframe.FrameNew(start, duration, name, run, 0, detectors)
 
     for series in tsdict.values():
+        # get type
+        ctype = (
+            type
+            or getattr(series.channel, "_ctype", "proc")
+            or "proc"
+        ).title()
+
         # convert to LAL
         lalseries = series.to_lal()
 
         # find adder
         add_ = lalutils.find_typed_function(
-            series.dtype, 'FrameAdd', 'TimeSeriesProcData', module=lalframe)
+            series.dtype,
+            'FrameAdd',
+            'TimeSeries{}Data'.format(ctype),
+            module=lalframe,
+        )
 
         # add time series to frame
         add_(frame, lalseries)
