@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2017)
+# Copyright (C) Duncan Macleod (2017-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,12 +19,11 @@
 """NDS2 data query routines for the TimeSeries
 """
 
-from __future__ import division
-
 import operator
 import warnings
+from functools import reduce
 
-from six.moves import reduce
+from numpy import ones as numpy_ones
 
 from ...detector import Channel
 from ...io import nds2 as io_nds2
@@ -32,6 +31,7 @@ from ...segments import (Segment, SegmentList)
 from ...utils import gprint
 from ...utils.progress import progress_bar
 from .. import (TimeSeries)
+from .core import _pad_series
 
 __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 
@@ -39,7 +39,7 @@ __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 def print_verbose(*args, **kwargs):
     """Utility to print something only if verbose=True is given
     """
-    if kwargs.pop('verbose', False):
+    if kwargs.pop('verbose', False) is True:
         gprint(*args, **kwargs)
 
 
@@ -78,15 +78,17 @@ def set_parameter(connection, parameter, value, verbose=False):
             'failed to set {}={!r}: {}'.format(parameter, value, str(exc)),
             io_nds2.NDSWarning)
     else:
-        if verbose:
-            print('    [{}] set {}={!r}'.format(connection.get_host(),
-                                                parameter, value))
+        print_verbose(
+            '    [{}] set {}={!r}'.format(
+                connection.get_host(), parameter, value),
+            verbose=verbose,
+        )
 
 
 @io_nds2.open_connection
 def fetch(channels, start, end, type=None, dtype=None, allow_tape=None,
-          connection=None, host=None, port=None, pad=None, verbose=False,
-          series_class=TimeSeries):
+          connection=None, host=None, port=None, pad=None, scaled=True,
+          verbose=False, series_class=TimeSeries):
     # host and port keywords are used by the decorator only
     # pylint: disable=unused-argument
     """Fetch a dict of data series from NDS2
@@ -107,16 +109,16 @@ def fetch(channels, start, end, type=None, dtype=None, allow_tape=None,
                   verbose=verbose)
     utype = reduce(operator.or_, type.values())  # logical OR of types
     udtype = reduce(operator.or_, dtype.values())
+    epoch = (start, end) if connection.get_protocol() > 1 else None
     ndschannels = io_nds2.find_channels(channels, connection=connection,
-                                        type=utype, dtype=udtype, unique=True,
-                                        epoch=(start, end))
+                                        epoch=epoch, type=utype, dtype=udtype,
+                                        unique=True)
 
     names = [Channel.from_nds2(c).ndsname for c in ndschannels]
     print_verbose('done', verbose=verbose)
 
     # handle minute trend timing
-    if (any(c.endswith('m-trend') for c in names) and
-            (start % 60 or end % 60)):
+    if any(c.endswith('m-trend') for c in names) and (start % 60 or end % 60):
         warnings.warn("Requested at least one minute trend, but "
                       "start and stop GPS times are not multiples of "
                       "60. Times will be expanded outwards to compensate")
@@ -127,6 +129,9 @@ def fetch(channels, start, end, type=None, dtype=None, allow_tape=None,
     if pad is None:
         qsegs = span
         gap = 'raise'
+    elif connection.get_protocol() == 1:
+        qsegs = span
+        gap = 'pad'
     else:
         print_verbose("Querying for data availability...", end=' ',
                       verbose=verbose)
@@ -144,13 +149,18 @@ def fetch(channels, start, end, type=None, dtype=None, allow_tape=None,
 
     # query for each segment
     out = series_class.DictClass()
-    with progress_bar(total=float(abs(qsegs)), desc='Downloading data',
+    desc = verbose if isinstance(verbose, str) else 'Downloading data'
+    with progress_bar(total=float(abs(qsegs)), desc=desc,
                       unit='s', disable=not bool(verbose)) as bar:
         for seg in qsegs:
             total = 0.
             for buffers in connection.iterate(int(seg[0]), int(seg[1]), names):
                 for buffer_, chan in zip(buffers, channels):
-                    series = series_class.from_nds2_buffer(buffer_)
+                    series = series_class.from_nds2_buffer(
+                        buffer_,
+                        scaled=scaled,
+                        copy=chan not in out,  # only copy if first buffer
+                    )
                     out.append({chan: series}, pad=pad, gap=gap)
                 new = buffer_.length / buffer_.channel.sample_rate
                 total += new
@@ -160,9 +170,33 @@ def fetch(channels, start, end, type=None, dtype=None, allow_tape=None,
                 raise RuntimeError("no data received from {0} for {1}".format(
                     connection.get_host(), seg))
 
+    # finalise timeseries to make sure each channel has the correct limits
+    # only if user asked to pad gaps
+    if pad is not None:
+        for chan, ndschan in zip(channels, ndschannels):
+            try:
+                ts = out[chan]
+            except KeyError:
+                out[chan] = _create_series(ndschan, pad, start, end,
+                                           series_class=series_class)
+            else:
+                out[chan] = _pad_series(ts, pad, start, end)
+
     return out
 
 
+def _create_series(ndschan, value, start, end, series_class=TimeSeries):
+    """Create a timeseries to cover the specified [start, end) limits
+
+    To cover a gap in data returned from NDS
+    """
+    channel = Channel.from_nds2(ndschan)
+    nsamp = int((end - start) * channel.sample_rate.value)
+    return series_class(numpy_ones(nsamp) * value, t0=start,
+                        sample_rate=channel.sample_rate, unit=channel.unit,
+                        channel=channel)
+
+
 def _get_data_segments(channels, start, end, connection):
     """Get available data segments for the given channels
     """
