@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -27,16 +27,15 @@ arrays, representing a bit mask of states that combine to make a detailed
 statement of instrumental operation
 """
 
+from functools import wraps
 from math import (ceil, log)
 
-from six.moves import range
-
 import numpy
 
 from astropy import units
 
 from .core import (TimeSeriesBase, TimeSeriesBaseDict, TimeSeriesBaseList,
-                   as_series_dict_class, ASTROPY_2_0)
+                   as_series_dict_class)
 from ..types import Array2D
 from ..detector import Channel
 from ..time import Time
@@ -48,6 +47,70 @@ __all__ = ['StateTimeSeries', 'StateTimeSeriesDict',
            'StateVector', 'StateVectorDict', 'StateVectorList', 'Bits']
 
 
+# -- utilities ----------------------------------------------------------------
+
+def _bool_segments(array, start=0, delta=1, minlen=1):
+    """Yield segments of consecutive `True` values in a boolean array
+
+    Parameters
+    ----------
+    array : `iterable`
+        An iterable of boolean-castable values.
+
+    start : `float`
+        The value of the first sample on the indexed axis
+        (e.g.the GPS start time of the array).
+
+    delta : `float`
+        The step size on the indexed axis (e.g. sample duration).
+
+    minlen : `int`, optional
+        The minimum number of consecutive `True` values for a segment.
+
+    Yields
+    ------
+    segment : `tuple`
+        ``(start + i * delta, start + (i + n) * delta)`` for a sequence
+        of ``n`` consecutive True values starting at position ``i``.
+
+    Notes
+    -----
+    This method is adapted from original code written by Kipp Cannon and
+    distributed under GPLv3.
+
+    The datatype of the values returned will be the larger of the types
+    of ``start`` and ``delta``.
+
+    Examples
+    --------
+    >>> print(list(_bool_segments([0, 1, 0, 0, 0, 1, 1, 1, 0, 1]))
+    [(1, 2), (5, 8), (9, 10)]
+    >>> print(list(_bool_segments([0, 1, 0, 0, 0, 1, 1, 1, 0, 1]
+    ...                           start=100., delta=0.1))
+    [(100.1, 100.2), (100.5, 100.8), (100.9, 101.0)]
+    """
+    array = iter(array)
+    i = 0
+    while True:
+        try:  # get next value
+            val = next(array)
+        except StopIteration:  # end of array
+            return
+
+        if val:  # start of new segment
+            n = 1  # count consecutive True
+            try:
+                while next(array):  # run until segment will end
+                    n += 1
+            except StopIteration:  # have reached the end
+                return  # stop
+            finally:  # yield segment (including at StopIteration)
+                if n >= minlen:  # ... if long enough
+                    yield (start + i * delta, start + (i + n) * delta)
+            i += n
+        i += 1
+
+
 # -- StateTimeSeries ----------------------------------------------------------
 
 class StateTimeSeries(TimeSeriesBase):
@@ -103,14 +166,14 @@ class StateTimeSeries(TimeSeriesBase):
                 channel=None, name=None, **kwargs):
         """Generate a new StateTimeSeries
         """
-        if 'unit' in kwargs:
+        if kwargs.pop('unit', None) is not None:
             raise TypeError("%s does not accept keyword argument 'unit'"
                             % cls.__name__)
         if isinstance(data, (list, tuple)):
             data = numpy.asarray(data)
         if not isinstance(data, cls):
             data = data.astype(bool)
-        return super(StateTimeSeries, cls).__new__(
+        return super().__new__(
             cls, data, t0=t0, dt=dt, sample_rate=sample_rate, times=times,
             name=name, channel=channel, **kwargs)
 
@@ -133,14 +196,14 @@ class StateTimeSeries(TimeSeriesBase):
 
     # -- math handling (always boolean) ---------
 
-    if ASTROPY_2_0:
-        def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
-            return super(StateTimeSeries, self).__array_ufunc__(
-                ufunc, method, *inputs, **kwargs).view(bool)
+    def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
+        out = super().__array_ufunc__(ufunc, method, *inputs, **kwargs)
+        if out.ndim:
+            return out.view(bool)
+        return out
 
     def __array_wrap__(self, obj, context=None):
-        return super(StateTimeSeries, self).__array_wrap__(
-            obj, context=context).view(bool)
+        return super().__array_wrap__(obj, context=context).view(bool)
 
     def diff(self, n=1, axis=-1):
         slice1 = (slice(1, None),)
@@ -156,11 +219,15 @@ class StateTimeSeries(TimeSeriesBase):
         return new
     diff.__doc__ = TimeSeriesBase.diff.__doc__
 
+    @wraps(numpy.ndarray.all, assigned=('__doc__',))
+    def all(self, axis=None, out=None):
+        return numpy.all(self.value, axis=axis, out=out)
+
     # -- useful methods -------------------------
 
     def to_dqflag(self, name=None, minlen=1, dtype=None, round=False,
                   label=None, description=None):
-        """Convert this series into a `~gwpy.segments.DataQualityFlag`
+        """Convert this series into a `~gwpy.segments.DataQualityFlag`.
 
         Each contiguous set of `True` values are grouped as a
         `~gwpy.segments.Segment` running from the GPS time the first
@@ -169,7 +236,7 @@ class StateTimeSeries(TimeSeriesBase):
 
         Parameters
         ----------
-        minlen : `int`, optional, default: 1
+        minlen : `int`, optional
             minimum number of consecutive `True` values to identify as a
             `~gwpy.segments.Segment`. This is useful to ignore single
             bit flips, for example.
@@ -179,10 +246,18 @@ class StateTimeSeries(TimeSeriesBase):
             casting, or a callable function that accepts a float and returns
             another numeric type, defaults to the `dtype` of the time index
 
-        round : `bool`, optional, default: False
+        round : `bool`, optional
             choose to round each `~gwpy.segments.Segment` to its
             inclusive integer boundaries
 
+        label : `str`, optional
+            the :attr:`~gwpy.segments.DataQualityFlag.label` for the
+            output flag.
+
+        description : `str`, optional
+            the :attr:`~gwpy.segments.DataQualityFlag.description` for the
+            output flag.
+
         Returns
         -------
         dqflag : `~gwpy.segments.DataQualityFlag`
@@ -190,28 +265,27 @@ class StateTimeSeries(TimeSeriesBase):
             defines the `known` segments, while the contiguous `True`
             sets defined each of the `active` segments
         """
-        from glue.segmentsUtils import from_bitstream
-        from ..segments import (Segment, SegmentList, DataQualityFlag)
+        from ..segments import DataQualityFlag
 
         # format dtype
         if dtype is None:
-            dtype = self.t0.dtype.type
-        elif isinstance(dtype, numpy.dtype):
+            dtype = self.t0.dtype
+        if isinstance(dtype, numpy.dtype):  # use callable dtype
             dtype = dtype.type
-        start = dtype(self.t0.value)  # <-- sets the dtype for the segments
-        dt = self.dt.value
+        start = dtype(self.t0.value)
+        dt = dtype(self.dt.value)
 
-        # build segmentlists
-        active = from_bitstream(self.value, start, dt, minlen=int(minlen))
-        known = SegmentList([Segment(*map(dtype, self.span))])
+        # build segmentlists (can use simple objects since DQFlag converts)
+        active = _bool_segments(self.value, start, dt, minlen=int(minlen))
+        known = [tuple(map(dtype, self.span))]
 
         # build flag and return
         out = DataQualityFlag(name=name or self.name, active=active,
                               known=known, label=label or self.name,
                               description=description)
         if round:
-            out = out.round()
-        return out.coalesce()
+            return out.round()
+        return out
 
     def to_lal(self, *args, **kwargs):
         """Bogus function inherited from superclass, do not use.
@@ -221,11 +295,22 @@ class StateTimeSeries(TimeSeriesBase):
                                   "StateTimeSeries because LAL has no "
                                   "BooleanTimeSeries structure")
 
+    @classmethod
+    @wraps(TimeSeriesBase.from_nds2_buffer)
+    def from_nds2_buffer(cls, buffer, **metadata):
+        metadata.setdefault('unit', None)
+        return super().from_nds2_buffer(buffer, **metadata)
+
     def __getitem__(self, item):
         if isinstance(item, (float, int)):
             return numpy.ndarray.__getitem__(self, item)
         else:
-            return super(StateTimeSeries, self).__getitem__(item)
+            return super().__getitem__(item)
+
+    def tolist(self):
+        return self.value.tolist()
+
+    tolist.__doc__ = numpy.ndarray.tolist.__doc__
 
 
 # -- Bits ---------------------------------------------------------------------
@@ -417,10 +502,10 @@ class StateVector(TimeSeriesBase):
                 times=None, channel=None, name=None, **kwargs):
         """Generate a new `StateVector`.
         """
-        new = super(StateVector, cls).__new__(cls, data, t0=t0, dt=dt,
-                                              sample_rate=sample_rate,
-                                              times=times, channel=channel,
-                                              name=name, **kwargs)
+        new = super().__new__(cls, data, t0=t0, dt=dt,
+                              sample_rate=sample_rate,
+                              times=times, channel=channel,
+                              name=name, **kwargs)
         new.bits = bits
         return new
 
@@ -506,7 +591,7 @@ class StateVector(TimeSeriesBase):
             a `dict` of `StateTimeSeries`, one for each given bit
         """
         if bits is None:
-            bits = [b for b in self.bits if b is not None and b is not '']
+            bits = [b for b in self.bits if b not in {None, ''}]
         bindex = []
         for bit in bits:
             try:
@@ -561,7 +646,7 @@ class StateVector(TimeSeriesBase):
         gap : `str`, optional
             how to handle gaps in the cache, one of
 
-            - 'ignore': do nothing, let the undelying reader method handle it
+            - 'ignore': do nothing, let the underlying reader method handle it
             - 'warn': do nothing except print a warning to the screen
             - 'raise': raise an exception upon finding a gap (default)
             - 'pad': insert a value to fill the gaps
@@ -570,6 +655,11 @@ class StateVector(TimeSeriesBase):
             value with which to fill gaps in the source data, only used if
             gap is not given, or `gap='pad'` is given
 
+        Raises
+        ------
+        IndexError
+            if ``source`` is an empty list
+
         Examples
         --------
         To read the S6 state vector, with names for all the bits::
@@ -598,7 +688,7 @@ class StateVector(TimeSeriesBase):
 
         Notes
         -----"""
-        return super(StateVector, cls).read(source, *args, **kwargs)
+        return super().read(source, *args, **kwargs)
 
     def to_dqflags(self, bits=None, minlen=1, dtype=float, round=False):
         """Convert this `StateVector` into a `~gwpy.segments.DataQualityDict`
@@ -623,9 +713,9 @@ class StateVector(TimeSeriesBase):
             a list of `~gwpy.segments.flag.DataQualityFlag`
             reprensentations for each bit in this `StateVector`
 
-        See Also
+        See also
         --------
-        :meth:`StateTimeSeries.to_dqflag`
+        StateTimeSeries.to_dqflag
             for details on the segment representation method for
             `StateVector` bits
         """
@@ -727,7 +817,7 @@ class StateVector(TimeSeriesBase):
             :meth:`.find` (for direct GWF file access) or
             :meth:`.fetch` for remote NDS2 access
 
-        See Also
+        See also
         --------
         StateVector.fetch
             for grabbing data from a remote NDS2 server
@@ -756,16 +846,16 @@ class StateVector(TimeSeriesBase):
 
         **kwargs
             Other keyword arguments to be passed to either
-            `~gwpy.plotter.segments.SegmentPlot` or
-            `~gwpy.plotter.TimeSeriesPlot`, depending
+            `~gwpy.plot.SegmentAxes.plot` or
+            `~gwpy.plot.Axes.plot`, depending
             on ``format``.
 
         Returns
         -------
-        plot : `~gwpy.plotter.SegmentPlot`, or `~gwpy.plotter.TimeSeriesPlot`
-            output plot object, some subclass of `~gwpy.plotter.Plot`
+        plot : `~gwpy.plot.Plot`
+            output plot object
 
-        See Also
+        See also
         --------
         matplotlib.pyplot.figure
             for documentation of keyword arguments used to create the
@@ -773,19 +863,17 @@ class StateVector(TimeSeriesBase):
         matplotlib.figure.Figure.add_subplot
             for documentation of keyword arguments used to create the
             axes
-        gwpy.plotter.SegmentAxes.plot_dqflag
+        gwpy.plot.SegmentAxes.plot_flag
             for documentation of keyword arguments used in rendering each
             statevector flag.
         """
         if format == 'timeseries':
-            return super(StateVector, self).plot(**kwargs)
-        elif format == 'segments':
-            kwargs.setdefault('facecolor', 'green')
-            kwargs.setdefault('edgecolor', 'black')
-            kwargs.setdefault('known', {'facecolor': 'red',
-                                        'edgecolor': 'black'})
-            from ..plotter import SegmentPlot
-            return SegmentPlot(*self.to_dqflags(bits=bits).values(), **kwargs)
+            return super().plot(**kwargs)
+        if format == 'segments':
+            from ..plot import Plot
+            kwargs.setdefault('xscale', 'auto-gps')
+            return Plot(*self.to_dqflags(bits=bits).values(),
+                        projection='segments', **kwargs)
         raise ValueError("'format' argument must be one of: 'timeseries' or "
                          "'segments'")
 
@@ -911,7 +999,7 @@ class StateVectorDict(TimeSeriesBaseDict):
         gap : `str`, optional
             how to handle gaps in the cache, one of
 
-            - 'ignore': do nothing, let the undelying reader method handle it
+            - 'ignore': do nothing, let the underlying reader method handle it
             - 'warn': do nothing except print a warning to the screen
             - 'raise': raise an exception upon finding a gap (default)
             - 'pad': insert a value to fill the gaps
@@ -928,7 +1016,7 @@ class StateVectorDict(TimeSeriesBaseDict):
 
         Notes
         -----"""
-        return super(StateVectorDict, cls).read(source, *args, **kwargs)
+        return super().read(source, *args, **kwargs)
 
 
 class StateVectorList(TimeSeriesBaseList):
