@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013-2016)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -21,9 +21,6 @@
 
 from warnings import warn
 
-import numpy
-
-from matplotlib import __version__ as mpl_version
 
 from astropy.units import (Unit, Quantity)
 
@@ -33,8 +30,6 @@ from .index import Index
 
 __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 
-DEFAULT_IMAGE_METHOD = 'imshow' if mpl_version >= '2.0' else 'pcolormesh'
-
 
 class Array2D(Series):
     """A two-dimensional array with metadata
@@ -115,8 +110,8 @@ class Array2D(Series):
         """
 
         # create new object
-        new = super(Array2D, cls).__new__(cls, data, unit=unit, xindex=xindex,
-                                          xunit=xunit, x0=x0, dx=dx, **kwargs)
+        new = super().__new__(cls, data, unit=unit, xindex=xindex,
+                              xunit=xunit, x0=x0, dx=dx, **kwargs)
 
         # set y-axis metadata from yindex
         if yindex is not None:
@@ -150,7 +145,7 @@ class Array2D(Series):
 
     # rebuild getitem to handle complex slicing
     def __getitem__(self, item):
-        new = super(Array2D, self).__getitem__(item)
+        new = super().__getitem__(item)
 
         # slice axis 1 metadata
         colslice, rowslice = sliceutils.format_nd_slice(item, self.ndim)
@@ -174,7 +169,7 @@ class Array2D(Series):
         return new
 
     def __array_finalize__(self, obj):
-        super(Array2D, self).__array_finalize__(obj)
+        super().__array_finalize__(obj)
         # Series.__array_finalize__ might set _yindex to None, so delete it
         if getattr(self, '_yindex', 0) is None:
             del self.yindex
@@ -201,23 +196,7 @@ class Array2D(Series):
 
     @y0.setter
     def y0(self, value):
-        if value is None:
-            del self.y0
-            return
-        if not isinstance(value, Quantity):
-            try:
-                value = Quantity(value, self.yunit)
-            except TypeError:
-                value = Quantity(float(value), self.yunit)
-        # if setting new y0, delete yindex
-        try:
-            y0 = self._y0
-        except AttributeError:
-            del self.yindex
-        else:
-            if value is None or self.y0 is None or value != y0:
-                del self.yindex
-        self._y0 = value
+        self._update_index("y", "y0", value)
 
     @y0.deleter
     def y0(self):
@@ -250,22 +229,7 @@ class Array2D(Series):
 
     @dy.setter
     def dy(self, value):
-        # delete if None
-        if value is None:
-            del self.dy
-            return
-        # convert float to Quantity
-        if not isinstance(value, Quantity):
-            value = Quantity(value).to(self.yunit)
-        # if value is changing, delete yindex
-        try:
-            dy = self._dy
-        except AttributeError:
-            del self.yindex
-        else:
-            if value is None or self.dy is None or value != dy:
-                del self.yindex
-        self._dy = value
+        self._update_index("y", "dy", value)
 
     @dy.deleter
     def dy(self):
@@ -298,28 +262,12 @@ class Array2D(Series):
         try:
             return self._yindex
         except AttributeError:
-            # create regular index on-the-fly
-            self._yindex = Index(
-                self.y0 + (numpy.arange(self.shape[1]) * self.dy), copy=False)
+            self._yindex = Index.define(self.y0, self.dy, self.shape[1])
             return self._yindex
 
     @yindex.setter
     def yindex(self, index):
-        if index is None:
-            del self.yindex
-            return
-        if not isinstance(index, Index):
-            try:
-                unit = index.unit
-            except AttributeError:
-                unit = self._default_yunit
-            index = Index(index, unit=unit, copy=False)
-        self.y0 = index[0]
-        if index.regular:
-            self.dy = index[1] - index[0]
-        else:
-            del self.dy
-        self._yindex = index
+        self._set_index("yindex", index)
 
     @yindex.deleter
     def yindex(self):
@@ -334,19 +282,7 @@ class Array2D(Series):
 
         :type: `~gwpy.segments.Segment`
         """
-        from ..segments import Segment
-        try:
-            dy = self.dy.to(self.yunit).value
-        except AttributeError:  # irregular yindex
-            try:
-                dy = self.yindex.value[-1] - self.yindex.value[-2]
-            except IndexError:
-                raise ValueError("Cannot determine y-axis stride (dy)"
-                                 "from a single data point")
-            return Segment(self.yindex.value[0], self.yindex.value[-1] + dy)
-        else:
-            y0 = self.y0.to(self.yunit).value
-            return Segment(y0, y0+self.shape[1]*dy)
+        return self._index_span("y")
 
     @property
     def T(self):
@@ -369,7 +305,7 @@ class Array2D(Series):
     def is_compatible(self, other):
         """Check whether this array and ``other`` have compatible metadata
         """
-        super(Array2D, self).is_compatible(other)
+        super().is_compatible(other)
         # check y-axis metadata
         if isinstance(other, type(self)):
             try:
@@ -418,7 +354,7 @@ class Array2D(Series):
     def pcolormesh(self, **kwargs):
         return self.plot(method='pcolormesh', **kwargs)
 
-    def plot(self, method=DEFAULT_IMAGE_METHOD, **kwargs):
+    def plot(self, method="imshow", **kwargs):
         from ..plot import Plot
 
         # correct for log scales and zeros
@@ -434,7 +370,7 @@ class Array2D(Series):
     # all of these try to return Quantities rather than simple numbers
 
     def _wrap_function(self, function, *args, **kwargs):
-        out = super(Array2D, self)._wrap_function(function, *args, **kwargs)
+        out = super()._wrap_function(function, *args, **kwargs)
         if out.ndim == 1:  # return Series
             # HACK: need to check astropy will always pass axis as first arg
             axis = args[0]
