@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2013-2016)
+# Copyright (C) Duncan Macleod (2014-2021)
 #
 # This file is part of GWpy.
 #
@@ -19,7 +19,6 @@
 """The `Series` is a one-dimensional array with metadata
 """
 
-from numbers import Number
 from warnings import warn
 from math import floor
 
@@ -120,7 +119,7 @@ class Series(Array):
                              % (cls.__name__, len(shape)))
 
         # create new object
-        new = super(Series, cls).__new__(cls, value, unit=unit, **kwargs)
+        new = super().__new__(cls, value, unit=unit, **kwargs)
 
         # set x-axis metadata from xindex
         if xindex is not None:
@@ -154,13 +153,98 @@ class Series(Array):
     # -- series creation ------------------------
 
     def __array_finalize__(self, obj):
-        super(Series, self).__array_finalize__(obj)
+        super().__array_finalize__(obj)
         # Array.__array_finalize__ might set _xindex to None, so delete it
         if getattr(self, '_xindex', None) is None:
             del self.xindex
 
     # -- series properties ----------------------
 
+    def _update_index(self, axis, key, value):
+        """Update the current axis index based on a given key or value
+
+        This is an internal method designed to set the origin or step for
+        an index, whilst updating existing Index arrays as appropriate
+
+        Examples
+        --------
+        >>> self._update_index("x0", 0)
+        >>> self._update_index("dx", 0)
+
+        To actually set an index array, use `_set_index`
+        """
+        # delete current value if given None
+        if value is None:
+            return delattr(self, key)
+
+        _key = "_{}".format(key)
+        index = "{[0]}index".format(axis)
+        unit = "{[0]}unit".format(axis)
+
+        # convert float to Quantity
+        if not isinstance(value, Quantity):
+            try:
+                value = Quantity(value, getattr(self, unit))
+            except TypeError:
+                value = Quantity(float(value), getattr(self, unit))
+
+        # if value is changing, delete current index
+        try:
+            curr = getattr(self, _key)
+        except AttributeError:
+            delattr(self, index)
+        else:
+            if (
+                    value is None
+                    or getattr(self, key) is None
+                    or not value.unit.is_equivalent(curr.unit)
+                    or value != curr
+            ):
+                delattr(self, index)
+
+        # set new value
+        setattr(self, _key, value)
+        return value
+
+    def _set_index(self, key, index):
+        """Set a new index array for this series
+        """
+        axis = key[0]
+        origin = "{}0".format(axis)
+        delta = "d{}".format(axis)
+        if index is None:
+            return delattr(self, key)
+        if not isinstance(index, Index):
+            try:
+                unit = index.unit
+            except AttributeError:
+                unit = getattr(self, "_default_{}unit".format(axis))
+            index = Index(index, unit=unit, copy=False)
+        setattr(self, origin, index[0])
+        if index.regular:
+            setattr(self, delta, index[1] - index[0])
+        else:
+            delattr(self, delta)
+        setattr(self, "_{}".format(key), index)
+
+    def _index_span(self, axis):
+        from ..segments import Segment
+        axisidx = ("x", "y", "z").index(axis)
+        unit = getattr(self, "{}unit".format(axis))
+        try:
+            delta = getattr(self, "d{}".format(axis)).to(unit).value
+        except AttributeError:  # irregular xindex
+            index = getattr(self, "{}index".format(axis))
+            try:
+                delta = index.value[-1] - index.value[-2]
+            except IndexError:
+                raise ValueError("Cannot determine x-axis stride (dx)"
+                                 "from a single data point")
+            return Segment(index.value[0], index.value[-1] + delta)
+        else:
+            origin = getattr(self, "{}0".format(axis)).to(unit).value
+            return Segment(origin, origin + self.shape[axisidx] * delta)
+
     # x0
     @property
     def x0(self):
@@ -176,23 +260,7 @@ class Series(Array):
 
     @x0.setter
     def x0(self, value):
-        if value is None:
-            del self.x0
-            return
-        if not isinstance(value, Quantity):
-            try:
-                value = Quantity(value, self.xunit)
-            except TypeError:
-                value = Quantity(float(value), self.xunit)
-        # if setting new x0, delete xindex
-        try:
-            x0 = self._x0
-        except AttributeError:
-            del self.xindex
-        else:
-            if value is None or self.x0 is None or value != x0:
-                del self.xindex
-        self._x0 = value
+        self._update_index("x", "x0", value)
 
     @x0.deleter
     def x0(self):
@@ -224,22 +292,7 @@ class Series(Array):
 
     @dx.setter
     def dx(self, value):
-        # delete if None
-        if value is None:
-            del self.dx
-            return
-        # convert float to Quantity
-        if not isinstance(value, Quantity):
-            value = Quantity(value, self.xunit)
-        # if value is changing, delete xindex
-        try:
-            dx = self._dx
-        except AttributeError:
-            del self.xindex
-        else:
-            if value is None or self.dx is None or value != dx:
-                del self.xindex
-        self._dx = value
+        self._update_index("x", "dx", value)
 
     @dx.deleter
     def dx(self):
@@ -258,28 +311,12 @@ class Series(Array):
         try:
             return self._xindex
         except AttributeError:
-            # create regular index on-the-fly
-            self._xindex = Index(
-                self.x0 + (numpy.arange(self.shape[0]) * self.dx), copy=False)
+            self._xindex = Index.define(self.x0, self.dx, self.shape[0])
             return self._xindex
 
     @xindex.setter
     def xindex(self, index):
-        if index is None:
-            del self.xindex
-            return
-        if not isinstance(index, Index):
-            try:
-                unit = index.unit
-            except AttributeError:
-                unit = self._default_xunit
-            index = Index(index, unit=unit, copy=False)
-        self.x0 = index[0]
-        if index.regular:
-            self.dx = index[1] - index[0]
-        else:
-            del self.dx
-        self._xindex = index
+        self._set_index("xindex", index)
 
     @xindex.deleter
     def xindex(self):
@@ -318,19 +355,7 @@ class Series(Array):
 
         :type: `~gwpy.segments.Segment`
         """
-        from ..segments import Segment
-        try:
-            dx = self.dx.to(self.xunit).value
-        except AttributeError:  # irregular xindex
-            try:
-                dx = self.xindex.value[-1] - self.xindex.value[-2]
-            except IndexError:
-                raise ValueError("Cannot determine x-axis stride (dx)"
-                                 "from a single data point")
-            return Segment(self.xindex.value[0], self.xindex.value[-1] + dx)
-        else:
-            x0 = self.x0.to(self.xunit).value
-            return Segment(x0, x0+self.shape[0]*dx)
+        return self._index_span("x")
 
     # -- series i/o -----------------------------
 
@@ -367,6 +392,11 @@ class Series(Array):
         -------
         data : `Series`
 
+        Raises
+        ------
+        IndexError
+            if ``source`` is an empty list
+
         Notes
         -----"""
         return io_registry.read(cls, source, *args, **kwargs)
@@ -392,8 +422,80 @@ class Series(Array):
         -----"""
         return io_registry.write(self, target, *args, **kwargs)
 
+    # -- series plotting ------------------------
+
+    def plot(self, method='plot', **kwargs):
+        """Plot the data for this series
+
+        Returns
+        -------
+        figure : `~matplotlib.figure.Figure`
+            the newly created figure, with populated Axes.
+
+        See also
+        --------
+        matplotlib.pyplot.figure
+            for documentation of keyword arguments used to create the
+            figure
+        matplotlib.figure.Figure.add_subplot
+            for documentation of keyword arguments used to create the
+            axes
+        matplotlib.axes.Axes.plot
+            for documentation of keyword arguments used in rendering the data
+        """
+        from ..plot import Plot
+        from ..plot.text import default_unit_label
+
+        # correct for log scales and zeros
+        if kwargs.get('xscale') == 'log' and self.x0.value == 0:
+            kwargs.setdefault('xlim', (self.dx.value, self.xspan[1]))
+
+        # make plot
+        plot = Plot(self, method=method, **kwargs)
+
+        # set default y-axis label (xlabel is set by Plot())
+        default_unit_label(plot.gca().yaxis, self.unit)
+
+        return plot
+
+    def step(self, **kwargs):
+        """Create a step plot of this series
+        """
+        where = kwargs.pop("where", "post")
+        kwargs.setdefault(
+            "drawstyle",
+            "steps-{}".format(where),
+        )
+        data = self.append(self.value[-1:], inplace=False)
+        return data.plot(**kwargs)
+
     # -- series methods -------------------------
 
+    def shift(self, delta):
+        """Shift this `Series` forward on the X-axis by ``delta``
+
+        This modifies the series in-place.
+
+        Parameters
+        ----------
+        delta : `float`, `~astropy.units.Quantity`, `str`
+            The amount by which to shift (in x-axis units if `float`), give
+            a negative value to shift backwards in time
+
+        Examples
+        --------
+        >>> from gwpy.types import Series
+        >>> a = Series([1, 2, 3, 4, 5], x0=0, dx=1, xunit='m')
+        >>> print(a.x0)
+        0.0 m
+        >>> a.shift(5)
+        >>> print(a.x0)
+        5.0 m
+        >>> a.shift('-1 km')
+        -995.0 m
+        """
+        self.x0 = self.x0 + Quantity(delta, self.xunit)
+
     def value_at(self, x):
         """Return the value of this `Series` at the given `xindex` value
 
@@ -416,7 +518,7 @@ class Series(Array):
         return self[idx]
 
     def copy(self, order='C'):
-        new = super(Series, self).copy(order=order)
+        new = super().copy(order=order)
         try:
             new._xindex = self._xindex.copy()
         except AttributeError:
@@ -468,12 +570,12 @@ class Series(Array):
             as the input, except along `axis` where the dimension is
             smaller by `n`.
 
-        See Also
+        See also
         --------
         numpy.diff
             for documentation on the underlying method
         """
-        out = super(Array, self).diff(n=n, axis=axis)
+        out = super().diff(n=n, axis=axis)
         try:
             out.x0 = self.x0 + self.dx * n
         except AttributeError:  # irregular xindex
@@ -481,7 +583,7 @@ class Series(Array):
         return out
 
     def __getslice__(self, i, j):
-        new = super(Series, self).__getslice__(i, j)
+        new = super().__getslice__(i, j)
         if i:
             try:
                 new.x0 = self.x0 + i * self.dx
@@ -490,10 +592,10 @@ class Series(Array):
         return new
 
     def __getitem__(self, item):
-        new = super(Series, self).__getitem__(item)
+        new = super().__getitem__(item)
 
         # slice axis 0 metadata
-        slice_ = sliceutils.format_nd_slice(item, self.ndim)[0]
+        slice_, = sliceutils.format_nd_slice(item, 1)
         if not sliceutils.null_slice(slice_):
             sliceutils.slice_axis_attributes(self, 'x', new, 'x', slice_)
 
@@ -554,8 +656,9 @@ class Series(Array):
                                  "be compatible")
             # check units
             if not self.unit == other.unit and not (
-                    self.unit in [dimensionless_unscaled, None] and
-                    other.unit in [dimensionless_unscaled, None]):
+                    self.unit in [dimensionless_unscaled, None]
+                    and other.unit in [dimensionless_unscaled, None]
+            ):
                 raise ValueError("%s units do not match: %s vs %s."
                                  % (type(self).__name__, str(self.unit),
                                     str(other.unit)))
@@ -570,7 +673,7 @@ class Series(Array):
                      % (self.dtype, other.dtype))
         return True
 
-    def append(self, other, gap='raise', inplace=True, pad=0, resize=True):
+    def append(self, other, inplace=True, pad=None, gap=None, resize=True):
         """Connect another series onto the end of the current one.
 
         Parameters
@@ -578,37 +681,47 @@ class Series(Array):
         other : `Series`
             another series of the same type to connect to this one
 
-        gap : `str`, optional, default: ``'raise'``
-            action to perform if there's a gap between the other series
-            and this one. One of
-
-                - ``'raise'`` - raise an `Exception`
-                    - ``'ignore'`` - remove gap and join data
-                - ``'pad'`` - pad gap with zeros
-
-        inplace : `bool`, optional, default: `True`
-            perform operation in-place, modifying current `Series`,
-            otherwise copy data and return new `Series`
+        inplace : `bool`, optional
+            perform operation in-place, modifying current series,
+            otherwise copy data and return new series, default: `True`
 
             .. warning::
 
-               inplace append bypasses the reference check in
+               `inplace` append bypasses the reference check in
                `numpy.ndarray.resize`, so be carefully to only use this
                for arrays that haven't been sharing their memory!
 
-        pad : `float`, optional, default: ``0.0``
-            value with which to pad discontiguous series
+        pad : `float`, optional
+            value with which to pad discontiguous series,
+            by default gaps will result in a `ValueError`.
 
-        resize : `bool`, optional, default: `True`
+        gap : `str`, optional
+            action to perform if there's a gap between the other series
+            and this one. One of
+
+            - ``'raise'`` - raise a `ValueError`
+            - ``'ignore'`` - remove gap and join data
+            - ``'pad'`` - pad gap with zeros
+
+            If ``pad`` is given and is not `None`, the default is ``'pad'``,
+            otherwise ``'raise'``. If ``gap='pad'`` is given, the default
+            for ``pad`` is ``0``.
+
+        resize : `bool`, optional
             resize this array to accommodate new data, otherwise shift the
             old data to the left (potentially falling off the start) and
-            put the new data in at the end
+            put the new data in at the end, default: `True`.
 
         Returns
         -------
         series : `Series`
             a new series containing joined data sets
         """
+        if gap is None:
+            gap = 'raise' if pad is None else 'pad'
+        if pad is None and gap == 'pad':
+            pad = 0.
+
         # check metadata
         self.is_compatible(other)
         # make copy if needed
@@ -669,7 +782,7 @@ class Series(Array):
         if type(other) == type(self) and other.unit == self.unit:
             self.value[-N:] = other.value[-N:]
         # otherwise if its just a numpy array
-        elif type(other) is type(self.value) or (
+        elif type(other) is type(self.value) or (  # noqa: E721
                 other.dtype.name.startswith('uint')):
             self.value[-N:] = other[-N:]
         else:
@@ -705,7 +818,7 @@ class Series(Array):
                 self.x0 = self.xindex[0]
         return self
 
-    def prepend(self, other, gap='raise', inplace=True, pad=0, resize=True):
+    def prepend(self, other, inplace=True, pad=None, gap=None, resize=True):
         """Connect another series onto the start of the current one.
 
         Parameters
@@ -713,43 +826,50 @@ class Series(Array):
         other : `Series`
             another series of the same type as this one
 
-        gap : `str`, optional, default: ``'raise'``
-            action to perform if there's a gap between the other series
-            and this one. One of
-
-                - ``'raise'`` - raise an `Exception`
-                - ``'ignore'`` - remove gap and join data
-                - ``'pad'`` - pad gap with zeros
-
-        inplace : `bool`, optional, default: `True`
+        inplace : `bool`, optional
             perform operation in-place, modifying current series,
-            otherwise copy data and return new series
+            otherwise copy data and return new series, default: `True`
 
             .. warning::
 
-               inplace prepend bypasses the reference check in
+               `inplace` prepend bypasses the reference check in
                `numpy.ndarray.resize`, so be carefully to only use this
                for arrays that haven't been sharing their memory!
 
-        pad : `float`, optional, default: ``0.0``
-            value with which to pad discontiguous `Series`
-        resize : `bool`, optional, default: `True`
+        pad : `float`, optional
+            value with which to pad discontiguous series,
+            by default gaps will result in a `ValueError`.
+
+        gap : `str`, optional
+            action to perform if there's a gap between the other series
+            and this one. One of
+
+            - ``'raise'`` - raise a `ValueError`
+            - ``'ignore'`` - remove gap and join data
+            - ``'pad'`` - pad gap with zeros
+
+            If `pad` is given and is not `None`, the default is ``'pad'``,
+            otherwise ``'raise'``.
+
+        resize : `bool`, optional
+            resize this array to accommodate new data, otherwise shift the
+            old data to the left (potentially falling off the start) and
+            put the new data in at the end, default: `True`.
 
         Returns
         -------
         series : `TimeSeries`
             time-series containing joined data sets
         """
-        out = other.append(self, gap=gap, inplace=False,
-                           pad=pad, resize=resize)
+        out = other.append(self, inplace=False, gap=gap, pad=pad,
+                           resize=resize)
         if inplace:
             self.resize(out.shape, refcheck=False)
             self[:] = out[:]
             self.x0 = out.x0.copy()
             del out
             return self
-        else:
-            return out
+        return out
 
     def update(self, other, inplace=True):
         """Update this series by appending new data from an other
@@ -786,39 +906,49 @@ class Series(Array):
         `Series` span, warnings will be printed and the limits will
         be restricted to the :attr:`~Series.xspan`
         """
+        x0, x1 = self.xspan
+        xtype = type(x0)
+        if isinstance(start, Quantity):
+            start = start.to(self.xunit).value
+        if isinstance(end, Quantity):
+            end = end.to(self.xunit).value
+
         # pin early starts to time-series start
-        if start == self.xspan[0]:
+        if start == x0:
             start = None
-        elif start is not None and start < self.xspan[0]:
+        elif start is not None and xtype(start) < x0:
             warn('%s.crop given start smaller than current start, '
                  'crop will begin when the Series actually starts.'
                  % type(self).__name__)
             start = None
+
         # pin late ends to time-series end
-        if end == self.xspan[1]:
+        if end == x1:
             end = None
-        if end is not None and end > self.xspan[1]:
+        if end is not None and xtype(end) > x1:
             warn('%s.crop given end larger than current end, '
                  'crop will end when the Series actually ends.'
                  % type(self).__name__)
             end = None
+
         # find start index
         if start is None:
             idx0 = None
         else:
-            idx0 = int(float(start - self.xspan[0]) // self.dx.value)
+            idx0 = int((xtype(start) - x0) // self.dx.value)
+
         # find end index
         if end is None:
             idx1 = None
         else:
-            idx1 = int(float(end - self.xspan[0]) // self.dx.value)
+            idx1 = int((xtype(end) - x0) // self.dx.value)
             if idx1 >= self.size:
                 idx1 = None
+
         # crop
         if copy:
             return self[idx0:idx1].copy()
-        else:
-            return self[idx0:idx1]
+        return self[idx0:idx1]
 
     def pad(self, pad_width, **kwargs):
         """Pad this series to a new size
@@ -826,9 +956,10 @@ class Series(Array):
         Parameters
         ----------
         pad_width : `int`, pair of `ints`
-            number of samples by which to pad each end of the array.
-            Single int to pad both ends by the same amount, or
-            (before, after) `tuple` to give uneven padding
+            number of samples by which to pad each end of the array;
+            given a single `int` to pad both ends by the same amount,
+            or a (before, after) `tuple` for assymetric padding
+
         **kwargs
             see :meth:`numpy.pad` for kwarg documentation
 
@@ -847,12 +978,12 @@ class Series(Array):
         if isinstance(pad_width, int):
             pad_width = (pad_width,)
         # form pad and view to this type
-        new = numpy.pad(self, pad_width, **kwargs).view(type(self))
+        new = numpy.pad(self.value, pad_width, **kwargs).view(type(self))
         # numpy.pad has stripped all metadata, so copy it over
         new.__metadata_finalize__(self)
         new._unit = self.unit
         # finally move the starting index based on the amount of left-padding
-        new.x0 -= self.dx * pad_width[0]
+        new.x0 = new.x0 - self.dx * pad_width[0]
         return new
 
     def inject(self, other):
